@@ -11,13 +11,18 @@ version of the tortoisehg workbench.
 """
 
 
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function
 
 import os
 import sys
 
-from mercurial.i18n import _
-from mercurial import registrar, ui
+from mercurial import (
+    pycompat,
+    registrar,
+    ui,
+)
+
+from tortoisehg.util.i18n import agettext as _
 
 testedwith = '4.3'
 
@@ -30,7 +35,7 @@ if hasattr(sys, "frozen"):
         sys.stdin = None
         if 'THGDEBUG' in os.environ:
             import win32traceutil
-            print 'starting'
+            print('starting')
     # os.Popen() needs this, and Mercurial still uses os.Popen
     if 'COMSPEC' not in os.environ:
         comspec = os.path.join(os.environ.get('SystemRoot', r'C:\Windows'),
@@ -79,7 +84,7 @@ demandimport.enable()
 # Verify we can reach TortoiseHg sources first
 try:
     import tortoisehg.hgqt.run
-except ImportError, e:
+except ImportError as e:
     sys.stderr.write(str(e)+'\n')
     sys.stderr.write("abort: couldn't find tortoisehg libraries in [%s]\n" %
                      os.pathsep.join(sys.path))
@@ -96,19 +101,17 @@ def enforceversion():
     if errmsg:
         from tortoisehg.hgqt.bugreport import run
         from tortoisehg.hgqt.run import qtrun
-        opts = {}
-        opts['cmd'] = ' '.join(sys.argv)
-        opts['error'] = '\n' + errmsg + '\n'
-        opts['nofork'] = True
+        opts = {'cmd': ' '.join(sys.argv),
+                'error': '\n' + errmsg + '\n',
+                'nofork': True}
         qtrun(run, ui.ui(), **opts)
         sys.exit(1)
 
-@command('view', [], _("hg view`"), inferrepo=True)
+@command(b'view', [], _("hg view"), inferrepo=True)
 def cmdview(ui, repo, *pats, **opts):
     """start light interactive history viewer from tortoisehg"""
     enforceversion()
-    import cStringIO
-    mystderr = cStringIO.StringIO()
+    mystderr = pycompat.bytesio()
     origstderr = sys.stderr
     sys.stderr = mystderr
     sys.__stdout__ = sys.stdout
@@ -127,10 +130,9 @@ def cmdview(ui, repo, *pats, **opts):
         for l in stderrout.splitlines():
             if l.startswith(errors):
                 from tortoisehg.hgqt.bugreport import run
-                opts = {}
-                opts['cmd'] = ' '.join(sys.argv)
-                opts['error'] = 'Recoverable error (stderr):\n' + stderrout
-                opts['nofork'] = True
+                opts = {'cmd': ' '.join(sys.argv),
+                        'error': 'Recoverable error (stderr):\n' + stderrout,
+                        'nofork': True}
                 qtrun(run, ui.ui(), **opts)
                 break
         sys.exit(ret)
