@@ -31,9 +31,14 @@ Exceptions:
   * msgfmt.PoSyntaxError if the po file has syntax errors
 
 """
+from __future__ import print_function
+
 import struct
 import array
-from cStringIO import StringIO
+
+from mercurial import (
+    pycompat,
+)
 
 __version__ = "1.1-pythongettext"
 
@@ -57,7 +62,7 @@ class Msgfmt:
         """ read po data from self.po and return an iterator """
         output = []
         if isinstance(self.po, str):
-            output = open(self.po, 'rb')
+            output = open(self.po, 'r')
         elif isinstance(self.po, file):
             self.po.seek(0)
             self.openfile = True
@@ -65,7 +70,7 @@ class Msgfmt:
         elif isinstance(self.po, list):
             output = self.po
         if not output:
-            raise ValueError, "self.po is invalid! %s" % type(self.po)
+            raise ValueError("self.po is invalid! %s" % type(self.po))
         return output
 
     def add(self, context, id, str, fuzzy):
@@ -78,19 +83,20 @@ class Msgfmt:
 
     def generate(self):
         "Return the generated output."
-        keys = self.messages.keys()
         # the keys are sorted in the .mo file
-        keys.sort()
+        keys = sorted(self.messages.keys())
         offsets = []
-        ids = strs = ''
+        ids = strs = b''
         for id in keys:
+            id_bytes = pycompat.sysbytes(id)
+            message_bytes = pycompat.sysbytes(self.messages[id])
             # For each string, we need size and file offset. Each string is
             # NUL terminated; the NUL does not count into the size.
-            offsets.append((len(ids), len(id), len(strs),
-                            len(self.messages[id])))
-            ids += id + '\0'
-            strs += self.messages[id] + '\0'
-        output = ''
+            offsets.append((len(ids), len(id_bytes), len(strs),
+                            len(message_bytes)))
+            ids += id_bytes + b'\0'
+            strs += message_bytes + b'\0'
+        output = b''
         # The header is 7 32-bit unsigned integers. We don't use hash tables,
         # so the keys start right after the index tables.
         keystart = 7*4+16*len(keys)
@@ -108,13 +114,16 @@ class Msgfmt:
         # binary compatible with the gnu gettext format produced by:
         # msgfmt file.po --no-hash
         output = struct.pack("Iiiiiii",
-                             0x950412deL,       # Magic
+                             0x950412de,        # Magic
                              0,                 # Version
                              len(keys),         # # of entries
                              7*4,               # start of key index
                              7*4+len(keys)*8,   # start of value index
                              0, keystart)       # size and offset of hash table
-        output += array.array("i", offsets).tostring()
+        if pycompat.ispy3:
+            output += array.array("i", offsets).tobytes()
+        else:
+            output += array.array("i", offsets).tostring()
         output += ids
         output += strs
         return output
@@ -174,8 +183,8 @@ class Msgfmt:
                 # This is a message with plural forms
                 elif l.startswith('msgid_plural'):
                     if section != ID:
-                        print >> sys.stderr, 'msgid_plural not preceeded by msgid on %s:%d' %\
-                            (infile, lno)
+                        print('msgid_plural not preceeded by msgid on %s:%d' %
+                              (infile, lno), file=sys.stderr)
                         sys.exit(1)
                     l = l[12:]
                     msgid += '\0' # separator of singular and plural
@@ -185,16 +194,16 @@ class Msgfmt:
                     section = STR
                     if l.startswith('msgstr['):
                         if not is_plural:
-                            print >> sys.stderr, 'plural without msgid_plural on %s:%d' %\
-                                (infile, lno)
+                            print('plural without msgid_plural on %s:%d' %
+                                  (infile, lno), file=sys.stderr)
                             sys.exit(1)
                         l = l.split(']', 1)[1]
                         if msgstr:
                             msgstr += '\0' # Separator of the various plural forms
                     else:
                         if is_plural:
-                            print >> sys.stderr, 'indexed msgstr required for plural on  %s:%d' %\
-                                (infile, lno)
+                            print('indexed msgstr required for plural on %s:%d' %
+                                  (infile, lno), file=sys.stderr)
                             sys.exit(1)
                         l = l[6:]
             # Skip empty lines
@@ -204,7 +213,7 @@ class Msgfmt:
             # XXX: Does this always follow Python escape semantics?
             try:
                 l = eval(l)
-            except Exception, msg:
+            except Exception as msg:
                 raise PoSyntaxError('%s (line %d of po file %s): \n%s' % (msg, lno, self.name, l))
             if section == CTXT:
                 msgctxt += l
@@ -223,4 +232,4 @@ class Msgfmt:
             self.po.close()
 
     def getAsFile(self):
-        return StringIO(self.get())
+        return pycompat.bytesio(self.get())
