@@ -21,7 +21,6 @@ from .qtcore import (
     QSize,
     QT_VERSION_STR,
     QTimer,
-    QUrl,
     Qt,
     pyqtSlot,
 )
@@ -35,10 +34,6 @@ from .qtgui import (
     QPlainTextEdit,
     QVBoxLayout,
 )
-from .qtnetwork import (
-    QNetworkAccessManager,
-    QNetworkRequest,
-)
 
 from ..util import (
     hglib,
@@ -46,7 +41,10 @@ from ..util import (
     version,
 )
 from ..util.i18n import _
-from . import qtlib
+from . import (
+    qtlib,
+    updatecheck,
+)
 
 class AboutDialog(QDialog):
     """Dialog for showing info about TortoiseHg"""
@@ -79,7 +77,7 @@ class AboutDialog(QDialog):
         self.copyright_lbl = QLabel()
         self.copyright_lbl.setAlignment(Qt.AlignCenter)
         self.copyright_lbl.setText('\n'
-                + _('Copyright 2008-2018 Steve Borho and others'))
+                + _('Copyright 2008-2020 Steve Borho and others'))
         self.vbox.addWidget(self.copyright_lbl)
         self.courtesy_lbl = QLabel()
         self.courtesy_lbl.setAlignment(Qt.AlignCenter)
@@ -97,7 +95,25 @@ class AboutDialog(QDialog):
                                        _('You can visit our site here')))
         self.vbox.addWidget(self.download_url_lbl)
 
-        # Let's have some space between the url and the buttons.
+        # Extra space between the URL and hosting credits
+        self.vbox.addWidget(QLabel())
+
+        self.hosting_lbl = QLabel()
+        self.hosting_lbl.setAlignment(Qt.AlignCenter)
+        self.hosting_lbl.setText(
+            _('Hosting donated by %s and %s') % (
+                '<a href=https://clever-cloud.com>Clever Cloud</a>',
+                '<a href=https://octobus.net>Octobus</a>'
+            )
+        )
+        self.hosting_lbl.setMouseTracking(True)
+        self.hosting_lbl.setTextInteractionFlags(
+            Qt.LinksAccessibleByMouse
+        )
+        self.hosting_lbl.setOpenExternalLinks(True)
+        self.vbox.addWidget(self.hosting_lbl)
+
+        # Let's have some space between the hosting and the buttons.
         self.blancline_lbl = QLabel()
         self.vbox.addWidget(self.blancline_lbl)
 
@@ -116,16 +132,17 @@ class AboutDialog(QDialog):
         self._readsettings()
 
         # Spawn it later, so that the dialog gets visible quickly.
-        QTimer.singleShot(1000, self.getUpdateInfo)
-        self._newverreply = None
+        self._updatechecker = updatecheck.Checker()
+        self._updatechecker.updateAvailable.connect(self.updateAvailable)
+        QTimer.singleShot(1000, self._updatechecker.query)
 
     def getVersionInfo(self):
         def make_version(tuple):
             vers = ".".join([str(x) for x in tuple])
             return vers
         thgv = (_('version %s') % version.version())
-        libv = (_('with Mercurial-%s, Python-%s, PyQt-%s, Qt-%s') % \
-              (hglib.hgversion, make_version(sys.version_info[0:3]),
+        libv = (_('with Mercurial-%s, Python-%s, PyQt-%s, Qt-%s') %
+                (hglib.hgversion, make_version(sys.version_info[0:3]),
               PYQT_VERSION_STR, QT_VERSION_STR))
         par = ('<p style=\" margin-top:0px; margin-bottom:6px;\">'
                 '<span style=\"font-size:%spt; font-weight:600;\">'
@@ -135,57 +152,19 @@ class AboutDialog(QDialog):
         nvl = ''.join([name, thgv, libv])
         self.name_version_libs_lbl.setText(nvl)
 
-    @pyqtSlot()
-    def getUpdateInfo(self):
-        verurl = 'https://tortoisehg.bitbucket.io/curversion.txt'
-        # If we use QNetworkAcessManager elsewhere, it should be shared
-        # through the application.
-        self._netmanager = QNetworkAccessManager(self)
-        self._newverreply = self._netmanager.get(QNetworkRequest(QUrl(verurl)))
-        self._newverreply.finished.connect(self.uFinished)
-
-    @pyqtSlot()
-    def uFinished(self):
-        newver = (0,0,0)
-        newverstr = '0.0.0'
-        upgradeurl = ''
-        try:
-            f = self._newverreply.readAll().data().splitlines()
-            self._newverreply.close()
-            self._newverreply = None
-            newverstr = f[0]
-            newver = tuple([int(p) for p in newverstr.split('.')])
-            upgradeurl = f[1] # generic download URL
-            platform = sys.platform
-            if platform == 'win32':
-                from win32process import IsWow64Process as IsX64
-                platform = IsX64() and 'x64' or 'x86'
-            # linux2 for Linux, darwin for OSX
-            for line in f[2:]:
-                p, _url = line.split(':', 1)
-                if platform == p:
-                    upgradeurl = _url.strip()
-                    break
-        except (IndexError, ImportError, ValueError):
-            pass
-        try:
-            thgv = version.package_version()
-            curver = tuple([int(p) for p in thgv.split('.')])
-        except ValueError:
-            curver = (0,0,0)
-        if newver > curver:
-            url_lbl = _('A new version of TortoiseHg (%s) '
-                        'is ready for download!') % newverstr
-            urldata = ('<a href=%s>%s</a>' % (upgradeurl, url_lbl))
-            self.download_url_lbl.setText(urldata)
+    @pyqtSlot(str, str)
+    def updateAvailable(self, newverstr, upgradeurl):
+        url_lbl = _('A new version of TortoiseHg (%s) '
+                    'is ready for download!') % newverstr
+        urldata = ('<a href=%s>%s</a>' % (upgradeurl, url_lbl))
+        self.download_url_lbl.setText(urldata)
 
     def showLicense(self):
         ld = LicenseDialog(self)
         ld.exec_()
 
     def closeEvent(self, event):
-        if self._newverreply:
-            self._newverreply.abort()
+        self._updatechecker.close()
         self._writesettings()
         super(AboutDialog, self).closeEvent(event)
 
@@ -213,9 +192,10 @@ class LicenseDialog(QDialog):
         self.lic_txt.setTextInteractionFlags(
                 Qt.TextSelectableByKeyboard|Qt.TextSelectableByMouse)
         try:
-            lic = open(paths.get_license_path(), 'rb').read()
+            with open(paths.get_license_path(), 'r') as fp:
+                lic = fp.read()
             self.lic_txt.setPlainText(lic)
-        except (IOError):
+        except IOError:
             pass
 
         bbox = QDialogButtonBox(self)
