@@ -25,10 +25,12 @@ from .qtgui import (
     QRadioButton,
     QSizePolicy,
     QVBoxLayout,
+    QWidget,
 )
 
 from mercurial import (
     error,
+    pycompat,
     scmutil,
 )
 
@@ -40,6 +42,18 @@ from . import (
     qtlib,
 )
 
+if hglib.TYPE_CHECKING:
+    from typing import (
+        Optional,
+        Text,
+        Union,
+    )
+
+    from . import (
+        thgrepo,
+    )
+
+
 WD_PARENT = _('= Working Directory Parent =')
 
 _ARCHIVE_TYPES = [
@@ -65,18 +79,20 @@ class ArchiveWidget(cmdui.AbstractCmdWidget):
     _archive_content_touched_since = 2
 
     def __init__(self, repoagent, rev, parent=None, minrev=None):
+        # type: (thgrepo.RepoAgent, Optional[Union[Text]], Optional[QWidget], Optional[Union[Text]]) -> None
         super(ArchiveWidget, self).__init__(parent)
         self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
         self._repoagent = repoagent
-        if minrev is None:
+        if not minrev:
             minrev = rev
             archive_since = False
         else:
             archive_since = True
 
         possibleroots = []
-        if minrev is not None:
-            parents = scmutil.revsymbol(self.repo, str(minrev)).parents()
+        if minrev:
+            parents = scmutil.revsymbol(self.repo,
+                                        hglib.fromunicode(minrev)).parents()
             if parents:
                 for p in parents:
                     possibleroots.append(u'%d' % p.rev())
@@ -111,7 +127,7 @@ class ArchiveWidget(cmdui.AbstractCmdWidget):
         self.rootrev_combo = QComboBox(self)
         self.rootrev_combo.setEditable(True)
         self.rootrev_combo.setSizeAdjustPolicy(QComboBox.AdjustToContents)
-        if minrev is not None:
+        if minrev:
             for text in possibleroots:
                 self.rootrev_combo.addItem(text)
             self.rootrev_combo.setCurrentIndex(0)
@@ -158,14 +174,12 @@ class ArchiveWidget(cmdui.AbstractCmdWidget):
         # set default values
         self.prevtarget = None
         self.rev_combo.addItem(WD_PARENT)
-        self.rev_combo.addItems(map(hglib.tounicode,
+        self.rev_combo.addItems(pycompat.maplist(hglib.tounicode,
                                     hglib.namedbranches(self.repo)))
-        tags = list(self.repo.tags())
-        tags.sort(reverse=True)
-        for t in tags:
+        for t in sorted(self.repo.tags(), reverse=True):
             self.rev_combo.addItem(hglib.tounicode(t))
         if rev:
-            text = hglib.tounicode(str(rev))
+            text = rev
             selectindex = self.rev_combo.findText(text)
             if selectindex >= 0:
                 self.rev_combo.setCurrentIndex(selectindex)
@@ -197,7 +211,7 @@ class ArchiveWidget(cmdui.AbstractCmdWidget):
 
     def browse_clicked(self):
         """Select the destination directory or file"""
-        dest = unicode(self.dest_edit.text())
+        dest = pycompat.unicode(self.dest_edit.text())
         if not os.path.exists(dest):
             dest = os.path.dirname(dest)
         select = self.get_selected_archive_type()
@@ -211,7 +225,7 @@ class ArchiveWidget(cmdui.AbstractCmdWidget):
             filter = ';;'.join(['%s (%s)' % (select['label'], ext),
                                 _('All files (*)')])
         response, _filter = FD.getSaveFileName(
-            self, caption, dest, filter, None, FD.ReadOnly)
+            self, caption, dest, filter, '', FD.ReadOnly)
         if response:
             self.dest_edit.setText(response)
             self.update_path()
@@ -225,9 +239,10 @@ class ArchiveWidget(cmdui.AbstractCmdWidget):
         return '.hgsubstate' in ctx
 
     def get_selected_rev(self):
+        # type: () -> bytes
         rev = self.rev_combo.currentText()
         if rev == WD_PARENT:
-            rev = '.'
+            rev = b'.'
         else:
             rev = hglib.fromunicode(rev)
         return rev
@@ -248,8 +263,8 @@ class ArchiveWidget(cmdui.AbstractCmdWidget):
             return path
         def remove_rev(path):
             l = ''
-            for i in xrange(self.rev_combo.count() - 1):
-                l += unicode(self.rev_combo.itemText(i))
+            for i in pycompat.xrange(self.rev_combo.count() - 1):
+                l += pycompat.unicode(self.rev_combo.itemText(i))
             revs = [rev[0] for rev in l]
             revs.append(wdrev)
             if not self.prevtarget is None:
@@ -265,11 +280,11 @@ class ArchiveWidget(cmdui.AbstractCmdWidget):
             if select['type'] != 'files':
                 path += select['ext']
             return path
-        text = unicode(self.rev_combo.currentText())
+        text = pycompat.unicode(self.rev_combo.currentText())
         if len(text) == 0:
             self.commandChanged.emit()
             return
-        wdrev = str(self.repo['.'].rev())
+        wdrev = str(self.repo[b'.'].rev())
         if text == WD_PARENT:
             text = wdrev
         else:
@@ -278,7 +293,7 @@ class ArchiveWidget(cmdui.AbstractCmdWidget):
             except (error.RepoError, error.LookupError):
                 self.commandChanged.emit()
                 return
-        path = unicode(self.dest_edit.text())
+        path = pycompat.unicode(self.dest_edit.text())
         path = remove_ext(path)
         path = remove_rev(path)
         path = add_rev(path, text)
@@ -322,7 +337,7 @@ class ArchiveWidget(cmdui.AbstractCmdWidget):
     def runCommand(self):
         # verify input
         type = self.get_selected_archive_type()['type']
-        dest = unicode(self.dest_edit.text())
+        dest = pycompat.unicode(self.dest_edit.text())
         if os.path.exists(dest):
             if type == 'files':
                 if os.path.isfile(dest):
@@ -354,6 +369,7 @@ class ArchiveWidget(cmdui.AbstractCmdWidget):
 
 
 def createArchiveDialog(repoagent, rev=None, parent=None, minrev=None):
+    # type: (thgrepo.RepoAgent, Optional[Union[Text]], Optional[QWidget], Optional[Union[Text]]) -> cmdui.CmdControlDialog
     dlg = cmdui.CmdControlDialog(parent)
     dlg.setWindowTitle(_('Archive - %s') % repoagent.displayName())
     dlg.setWindowIcon(qtlib.geticon('hg-archive'))
