@@ -5,16 +5,45 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2, incorporated herein by reference.
 
+from __future__ import absolute_import
+
 import re
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui import *
+from .qtcore import (
+    QPoint,
+    Qt,
+    pyqtSlot,
+)
+from .qtgui import (
+    QAction,
+    QCheckBox,
+    QComboBox,
+    QDialog,
+    QDialogButtonBox,
+    QFormLayout,
+    QFrame,
+    QHBoxLayout,
+    QLabel,
+    QLayout,
+    QListWidget,
+    QLineEdit,
+    QMenu,
+    QSizePolicy,
+    QSplitter,
+    QVBoxLayout,
+    QWidget,
+)
 
-from mercurial import util
+from mercurial import (
+    pycompat,
+)
 
-from tortoisehg.util import hglib
-from tortoisehg.util.i18n import _
-from tortoisehg.hgqt import cmdcore, qtlib
+from ..util import hglib
+from ..util.i18n import _
+from . import (
+    cmdcore,
+    qtlib,
+)
 
 class BookmarkDialog(QDialog):
 
@@ -60,7 +89,7 @@ class BookmarkDialog(QDialog):
 
         ### Activate checkbox
         self.activateCheckBox = QCheckBox()
-        if self.node == self.repo['.'].node():
+        if self.node == self.repo[b'.'].node():
             self.activateCheckBox.setChecked(True)
         else:
             self.activateCheckBox.setChecked(False)
@@ -111,7 +140,7 @@ class BookmarkDialog(QDialog):
         return self._repoagent.rawRepo()
 
     def _allBookmarks(self):
-        return map(hglib.tounicode, self.repo._bookmarks)
+        return pycompat.maplist(hglib.tounicode, self.repo._bookmarks)
 
     @pyqtSlot()
     def refresh(self):
@@ -125,8 +154,8 @@ class BookmarkDialog(QDialog):
         else:
             ctx = self.repo[self.rev]
             cs_bookmarks = ctx.bookmarks()
-            if self.repo._bookmarkcurrent in cs_bookmarks:
-                bm = hglib.tounicode(self.repo._bookmarkcurrent)
+            if hglib.activebookmark(self.repo) in cs_bookmarks:
+                bm = hglib.tounicode(hglib.activebookmark(self.repo))
                 self.bookmarkCombo.setEditText(bm)
             elif cs_bookmarks:
                 bm = hglib.tounicode(cs_bookmarks[0])
@@ -181,7 +210,7 @@ class BookmarkDialog(QDialog):
 
     @pyqtSlot()
     def add_bookmark(self):
-        bookmark = unicode(self.bookmarkCombo.currentText())
+        bookmark = pycompat.unicode(self.bookmarkCombo.currentText())
         if bookmark in self._allBookmarks():
             self.set_status(_('A bookmark named "%s" already exists') %
                             bookmark, False)
@@ -195,7 +224,7 @@ class BookmarkDialog(QDialog):
 
     @pyqtSlot()
     def move_bookmark(self):
-        bookmark = unicode(self.bookmarkCombo.currentText())
+        bookmark = pycompat.unicode(self.bookmarkCombo.currentText())
         if bookmark not in self._allBookmarks():
             self.set_status(_('Bookmark named "%s" does not exist') %
                             bookmark, False)
@@ -209,7 +238,7 @@ class BookmarkDialog(QDialog):
 
     @pyqtSlot()
     def remove_bookmark(self):
-        bookmark = unicode(self.bookmarkCombo.currentText())
+        bookmark = pycompat.unicode(self.bookmarkCombo.currentText())
         if bookmark not in self._allBookmarks():
             self.set_status(_("Bookmark '%s' does not exist") % bookmark, False)
             return
@@ -219,12 +248,12 @@ class BookmarkDialog(QDialog):
 
     @pyqtSlot()
     def rename_bookmark(self):
-        name = unicode(self.bookmarkCombo.currentText())
+        name = pycompat.unicode(self.bookmarkCombo.currentText())
         if name not in self._allBookmarks():
             self.set_status(_("Bookmark '%s' does not exist") % name, False)
             return
 
-        newname = unicode(self.newNameEdit.text())
+        newname = pycompat.unicode(self.newNameEdit.text())
         if newname in self._allBookmarks():
             self.set_status(_('A bookmark named "%s" already exists') %
                             newname, False)
@@ -235,7 +264,7 @@ class BookmarkDialog(QDialog):
         self._runBookmark(name, newname, rename=True, finishmsg=finishmsg)
 
 
-_extractbookmarknames = re.compile(r'(.*) [0-9a-f]{12,}$',
+_extractbookmarknames = re.compile(br'(.*) [0-9a-f]{12,}$',
                                    re.MULTILINE).findall
 
 class SyncBookmarkDialog(QDialog):
@@ -267,7 +296,6 @@ class SyncBookmarkDialog(QDialog):
         outgoingFrame = QFrame(self.splitter)
         outgoingLayout = QVBoxLayout()
         outgoingLayout.setSpacing(2)
-        outgoingLayout.setMargin(2)
         outgoingLayout.setContentsMargins(2, 2, 2, 2)
         outgoingFrame.setLayout(outgoingLayout)
         outgoingLabel = QLabel(_('Outgoing Bookmarks'))
@@ -290,7 +318,6 @@ class SyncBookmarkDialog(QDialog):
         self.addActions(self._outactions)
         outgoingBtnLayout = QHBoxLayout()
         outgoingBtnLayout.setSpacing(2)
-        outgoingBtnLayout.setMargin(2)
         outgoingBtnLayout.setContentsMargins(2, 2, 2, 2)
         for a in self._outactions:
             outgoingBtnLayout.addWidget(qtlib.ActionPushButton(a, self))
@@ -300,7 +327,6 @@ class SyncBookmarkDialog(QDialog):
         incomingFrame = QFrame(self.splitter)
         incomingLayout = QVBoxLayout()
         incomingLayout.setSpacing(2)
-        incomingLayout.setMargin(2)
         incomingLayout.setContentsMargins(2, 2, 2, 2)
         incomingFrame.setLayout(incomingLayout)
         incomingLabel = QLabel(_('Incoming Bookmarks'))
@@ -323,7 +349,6 @@ class SyncBookmarkDialog(QDialog):
         self.addActions(self._inactions)
         incomingBtnLayout = QHBoxLayout()
         incomingBtnLayout.setSpacing(2)
-        incomingBtnLayout.setMargin(2)
         incomingBtnLayout.setContentsMargins(2, 2, 2, 2)
         for a in self._inactions:
             incomingBtnLayout.addWidget(qtlib.ActionPushButton(a, self))
@@ -364,7 +389,7 @@ class SyncBookmarkDialog(QDialog):
     def _onListBookmarksFinished(self, sess, worklist):
         ret = sess.exitCode()
         if ret == 0:
-            bookmarks = _extractbookmarknames(str(sess.readAll()))
+            bookmarks = _extractbookmarknames(bytes(sess.readAll()))
             self._updateBookmarkList(worklist, bookmarks)
         elif ret == 1:
             self._updateBookmarkList(worklist, [])
@@ -373,10 +398,12 @@ class SyncBookmarkDialog(QDialog):
         self._updateActions()
 
     def selectedOutgoingBookmarks(self):
-        return [unicode(x.text()) for x in self.outgoingList.selectedItems()]
+        return [pycompat.unicode(x.text())
+                for x in self.outgoingList.selectedItems()]
 
     def selectedIncomingBookmarks(self):
-        return [unicode(x.text()) for x in self.incomingList.selectedItems()]
+        return [pycompat.unicode(x.text())
+                for x in self.incomingList.selectedItems()]
 
     @pyqtSlot()
     def push_bookmark(self):
@@ -450,8 +477,8 @@ class SyncBookmarkDialog(QDialog):
 
     @pyqtSlot()
     def _updateActions(self):
-        state = util.all(sess.isFinished() for sess
-                         in [self._cmdsession, self._insess, self._outsess])
+        state = all(sess.isFinished() for sess
+                    in [self._cmdsession, self._insess, self._outsess])
         for a in self._outactions:
             a.setEnabled(state and bool(self.selectedOutgoingBookmarks()))
         for a in self._inactions:
