@@ -5,9 +5,8 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function
 
-import cgi
 import os
 import re
 import sys
@@ -17,7 +16,6 @@ from .qtcore import (
     QSettings,
     QT_VERSION_STR,
     QTimer,
-    QUrl,
     Qt,
     pyqtSlot,
 )
@@ -33,13 +31,8 @@ from .qtgui import (
     QVBoxLayout,
     qApp,
 )
-from .qtnetwork import (
-    QNetworkAccessManager,
-    QNetworkRequest,
-)
 
 from mercurial import (
-    encoding,
     extensions,
 )
 from ..util import (
@@ -47,7 +40,10 @@ from ..util import (
     version,
 )
 from ..util.i18n import _
-from . import qtlib
+from . import (
+    qtlib,
+    updatecheck,
+)
 
 try:
     from .qsci import QSCINTILLA_VERSION_STR
@@ -72,13 +68,13 @@ class BugReport(QDialog):
 
         lbl = QLabel(_('Please report this bug to our '
             '<a href="%s">bug tracker</a>') %
-            u'https://bitbucket.org/tortoisehg/thg/wiki/BugReport')
+            u'https://foss.heptapod.net/mercurial/tortoisehg/thg/-/issues')
         lbl.setOpenExternalLinks(True)
         self.layout().addWidget(lbl)
 
         tb = QTextBrowser()
         self.text = self.gettext(opts)
-        tb.setHtml('<pre>' + cgi.escape(self.text) + '</pre>')
+        tb.setHtml('<pre>' + qtlib.htmlescape(self.text, False) + '</pre>')
         tb.setWordWrapMode(QTextOption.NoWrap)
         layout.addWidget(tb)
 
@@ -103,49 +99,21 @@ class BugReport(QDialog):
                             ~Qt.WindowContextHelpButtonHint)
         self.resize(650, 400)
         self._readsettings()
-        QTimer.singleShot(0, self.getUpdateInfo)
 
-    def getUpdateInfo(self):
-        verurl = 'https://tortoisehg.bitbucket.io/curversion.txt'
-        # If we use QNetworkAcessManager elsewhere, it should be shared
-        # through the application.
-        self._netmanager = QNetworkAccessManager(self)
-        self._newverreply = self._netmanager.get(QNetworkRequest(QUrl(verurl)))
-        self._newverreply.finished.connect(self.uFinished)
+        self._updatechecker = updatecheck.Checker()
+        self._updatechecker.updateAvailable.connect(self.updateAvailable)
+        self._updatechecker.updateUnavailable.connect(self.updateUnavailable)
+        QTimer.singleShot(0, self._updatechecker.query)
 
-    def uFinished(self):
-        newver = (0,0,0)
-        try:
-            f = self._newverreply.readAll().data().splitlines()
-            self._newverreply.close()
-            self._newverreply = None
-            newver = tuple([int(p) for p in f[0].split('.')])
-            upgradeurl = f[1] # generic download URL
-            platform = sys.platform
-            if platform == 'win32':
-                from win32process import IsWow64Process as IsX64
-                platform = IsX64() and 'x64' or 'x86'
-            # linux2 for Linux, darwin for OSX
-            for line in f[2:]:
-                p, _url = line.split(':', 1)
-                if platform == p:
-                    upgradeurl = _url.strip()
-                    break
-        except (IndexError, ImportError, ValueError):
-            pass
-        try:
-            thgv = version.version()
-            if '+' in thgv:
-                thgv = thgv[:thgv.index('+')]
-            curver = tuple([int(p) for p in thgv.split('.')])
-        except ValueError:
-            curver = (0,0,0)
-        if newver > curver:
-            url_lbl = _('Upgrading to a more recent TortoiseHg is recommended.')
-            urldata = ('<a href=%s>%s</a>' % (upgradeurl, url_lbl))
-            self.download_url_lbl.setText(urldata)
-        else:
-            self.download_url_lbl.setText(_('Your TortoiseHg is up to date.'))
+    @pyqtSlot(str, str)
+    def updateAvailable(self, newverstr, upgradeurl):
+        url_lbl = _('Upgrading to a more recent TortoiseHg is recommended.')
+        urldata = ('<a href=%s>%s</a>' % (upgradeurl, url_lbl))
+        self.download_url_lbl.setText(urldata)
+
+    @pyqtSlot()
+    def updateUnavailable(self):
+        self.download_url_lbl.setText(_('Your TortoiseHg is up to date.'))
 
     def gettext(self, opts):
         # TODO: make this more uniformly unicode safe
@@ -154,8 +122,8 @@ class BugReport(QDialog):
                 hglib.hgversion, version.version())
         text += '** Command: %s\n' % (hglib.tounicode(opts.get('cmd', 'N/A')))
         text += '** CWD: %s\n' % hglib.tounicode(_safegetcwd())
-        text += '** Encoding: %s\n' % encoding.encoding
-        extlist = [x[0] for x in extensions.extensions()]
+        text += '** Encoding: %s\n' % hglib._encoding
+        extlist = [hglib.tounicode(x[0]) for x in extensions.extensions()]
         text += '** Extensions loaded: %s\n' % ', '.join(extlist)
         text += '** Python version: %s\n' % sys.version.replace('\n', '')
         if os.name == 'nt':
@@ -177,7 +145,7 @@ class BugReport(QDialog):
         text = '** Windows version: %s\n' % str(sys.getwindowsversion())
         arch = 'unknown (failed to import win32api)'
         try:
-            import win32api
+            import win32api  # pytype: disable=import-error
             arch = 'unknown'
             archval = win32api.GetNativeSystemInfo()[0]
             if archval == 9:
@@ -196,10 +164,15 @@ class BugReport(QDialog):
                         os.path.join(_safegetcwd(), 'bugreport.txt'),
                         _('Text files (*.txt)'))
             if fname:
-                open(fname, 'wb').write(hglib.fromunicode(self.text))
-        except (EnvironmentError), e:
+                with open(fname, 'wb') as fp:
+                    fp.write(hglib.fromunicode(self.text))
+        except EnvironmentError as e:
             QMessageBox.critical(self, _('Error writing file'), str(e))
 
+    def closeEvent(self, event):
+        self._updatechecker.close()
+        super(BugReport, self).closeEvent(event)
+
     def accept(self):
         self._writesettings()
         super(BugReport, self).accept()
@@ -233,13 +206,15 @@ class ExceptionMsgBox(QDialog):
             values = opts.get('values', [])
             msgopts = {}
             for i, val in enumerate(values):
-                msgopts['arg' + str(i)] = cgi.escape(hglib.tounicode(val))
+                msgopts['arg' + str(i)] = qtlib.htmlescape(hglib.tounicode(val),
+                                                           False)
             try:
                 text = text % msgopts
-            except Exception, e:
-                print e, msgopts
+            except Exception as e:
+                print(e, msgopts)
         else:
-            self._mainlabel = QLabel('<b>%s</b>' % cgi.escape(main),
+            self._mainlabel = QLabel('<b>%s</b>'
+                                     % qtlib.htmlescape(main, False),
                                      textInteractionFlags=labelflags)
             self.layout().addWidget(self._mainlabel)
 
@@ -267,20 +242,19 @@ class ExceptionMsgBox(QDialog):
             try:
                 # A chicken-egg problem here, we need a ui to get your
                 # editor in order to repair your ui config file.
-                from tortoisehg.hgqt import qtlib
                 class FakeRepo(object):
                     def __init__(self):
                         self.root = os.getcwd()
                         self.ui = hglib.loadui()
                 fake = FakeRepo()
                 qtlib.editfiles(fake, [fname], lineno, parent=self)
-            except Exception, e:
+            except Exception as e:
                 qtlib.openlocalurl(fname)
         if ref.startswith('#fix:'):
             from tortoisehg.hgqt import settings
             errtext = ref[5:].split(' ')[0]
             sd = settings.SettingsDialog(configrepo=False, focus=errtext,
-                                parent=self, root='')
+                                parent=self, root=b'')
             sd.exec_()
 
 def run(ui, *pats, **opts):
