@@ -34,6 +34,7 @@ from mercurial import (
     cmdutil,
     commands,
     hg,
+    pycompat,
 )
 
 from ..util import hglib
@@ -44,10 +45,18 @@ from . import (
     qtlib,
 )
 
+if hglib.TYPE_CHECKING:
+    from typing import (
+        Any,
+        Optional,
+        Text,
+        Tuple,
+    )
+
 def _startrev_available():
-    entry = cmdutil.findcmd('clone', commands.table)[1]
+    entry = cmdutil.findcmd(b'clone', commands.table)[1]
     longopts = set(e[1] for e in entry[1])
-    return 'startrev' in longopts
+    return b'startrev' in longopts
 
 def _suggesteddest(src, basedest):
     if '://' in basedest:
@@ -70,21 +79,11 @@ def _suggesteddest(src, basedest):
 
 class CloneWidget(cmdui.AbstractCmdWidget):
 
-    def __init__(self, ui, cmdagent, args=None, opts={}, parent=None):
+    def __init__(self, config, cmdagent, parent=None):
         super(CloneWidget, self).__init__(parent)
         self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
+        self._config = config
         self._cmdagent = cmdagent
-        self.ui = ui
-
-        dest = src = os.getcwd()
-        if args:
-            if len(args) > 1:
-                src = args[0]
-                dest = args[1]
-            else:
-                src = args[0]
-        udest = hglib.tounicode(dest)
-        usrc = hglib.tounicode(src)
 
         ## main layout
         form = QFormLayout()
@@ -119,8 +118,9 @@ class CloneWidget(cmdui.AbstractCmdWidget):
             qtlib.allowCaseChangingInput(combo)
             combo.installEventFilter(qtlib.BadCompletionBlocker(combo))
 
-        self.setSource(usrc)
-        self.setDestination(udest)
+        self.setSource(config.configString('tortoisehg', 'defaultclonedest')
+                       or hglib.getcwdu())
+        self.setDestination(self.source())
 
         ### options
         expander = qtlib.ExpanderLabel(_('Options'), False)
@@ -154,10 +154,24 @@ class CloneWidget(cmdui.AbstractCmdWidget):
                 hbox.addWidget(btn)
                 return chk, text, btn
             else:
-                return chk, text
+                return chk, text, None
 
-        self.rev_chk, self.rev_text = chktext(_('Clone to revision:'),
-                                              stretch=40)
+        def chktext2(chklabel, stretch=None):
+            # type: (Text, Optional[int]) -> Tuple[QCheckBox, QLineEdit]
+            # pytype gets confused if the returned tuple is sliced and returned
+            # without unpacking.
+            chk, text, _unused = chktext(chklabel, stretch=stretch)
+            return chk, text
+
+        def chktext3(chklabel, btnlabel, btnslot, stretch=None):
+            # type: (Text, Text, Any, Optional[int]) -> Tuple[QCheckBox, QLineEdit, QPushButton]
+            assert btnlabel
+            ret = chktext(chklabel, btnlabel, btnslot, stretch)
+            assert isinstance(ret[2], QPushButton)
+            return ret
+
+        self.rev_chk, self.rev_text = chktext2(_('Clone to revision:'),
+                                               stretch=40)
         self.rev_text.setToolTip(_('A revision identifier, bookmark, tag or '
                                    'branch name'))
 
@@ -167,14 +181,19 @@ class CloneWidget(cmdui.AbstractCmdWidget):
         optbox.addWidget(self.noupdate_chk)
         optbox.addWidget(self.pproto_chk)
         optbox.addWidget(self.stream_chk)
+        self._opt_checks = {
+            'noupdate': self.noupdate_chk,
+            'pull': self.pproto_chk,
+            'stream': self.stream_chk,
+        }
 
         self.qclone_chk, self.qclone_txt, self.qclone_btn = \
-                chktext(_('Include patch queue'), btnlabel=_('Browse...'),
-                        btnslot=self._browsePatchQueue)
+                chktext3(_('Include patch queue'), btnlabel=_('Browse...'),
+                         btnslot=self._browsePatchQueue)
 
         self.proxy_chk = QCheckBox(_('Use proxy server'))
         optbox.addWidget(self.proxy_chk)
-        useproxy = bool(self.ui.config('http_proxy', 'host'))
+        useproxy = bool(config.configString('http_proxy', 'host'))
         self.proxy_chk.setEnabled(useproxy)
         self.proxy_chk.setChecked(useproxy)
 
@@ -182,14 +201,14 @@ class CloneWidget(cmdui.AbstractCmdWidget):
         optbox.addWidget(self.insecure_chk)
         self.insecure_chk.setEnabled(False)
 
-        self.remote_chk, self.remote_text = chktext(_('Remote command:'))
+        self.remote_chk, self.remote_text = chktext2(_('Remote command:'))
 
         self.largefiles_chk = QCheckBox(_('Use largefiles'))
         optbox.addWidget(self.largefiles_chk)
 
         # allow to specify start revision for p4 & svn repos.
-        self.startrev_chk, self.startrev_text = chktext(_('Start revision:'),
-                                                        stretch=40)
+        self.startrev_chk, self.startrev_text = chktext2(_('Start revision:'),
+                                                         stretch=40)
 
         self.hgcmd_txt = QLineEdit()
         self.hgcmd_txt.setReadOnly(True)
@@ -221,13 +240,6 @@ class CloneWidget(cmdui.AbstractCmdWidget):
         # prepare to show
         optwidget.hide()
 
-        rev = opts.get('rev')
-        if rev:
-            self.rev_chk.setChecked(True)
-            self.rev_text.setText(hglib.tounicode(rev))
-        self.noupdate_chk.setChecked(bool(opts.get('noupdate')))
-        self.pproto_chk.setChecked(bool(opts.get('pull')))
-        self.stream_chk.setChecked(bool(opts.get('stream')))
         self.startrev_chk.setVisible(_startrev_available())
         self.startrev_text.setVisible(_startrev_available())
 
@@ -250,19 +262,19 @@ class CloneWidget(cmdui.AbstractCmdWidget):
         for key, combo in [('source', self.src_combo),
                            ('dest', self.dest_combo)]:
             l = [combo.currentText()]
-            l.extend(combo.itemText(i) for i in xrange(combo.count())
+            l.extend(combo.itemText(i) for i in pycompat.xrange(combo.count())
                      if combo.itemText(i) != combo.currentText())
             qs.setValue(key, l[:10])
 
     def source(self):
-        return unicode(self.src_combo.currentText()).strip()
+        return self.src_combo.currentText().strip()
 
     def setSource(self, url):
         self.src_combo.setCurrentIndex(self.src_combo.findText(url))
         self.src_combo.setEditText(url)
 
     def destination(self):
-        return unicode(self.dest_combo.currentText()).strip()
+        return self.dest_combo.currentText().strip()
 
     def setDestination(self, url):
         self.dest_combo.setCurrentIndex(self.dest_combo.findText(url))
@@ -272,26 +284,40 @@ class CloneWidget(cmdui.AbstractCmdWidget):
     def _suggestDestination(self):
         self.setDestination(_suggesteddest(self.source(), self.destination()))
 
+    def revSymbol(self):
+        if not self.rev_chk.isChecked():
+            return ''
+        return self.rev_text.text().strip()
+
+    def setRevSymbol(self, rev):
+        self.rev_chk.setChecked(bool(rev))
+        self.rev_text.setText(rev)
+
+    def testOption(self, key):
+        return self._opt_checks[key].isChecked()
+
+    def setOption(self, key, on):
+        self._opt_checks[key].setChecked(on)
+
     @pyqtSlot()
     def _composeCommand(self):
         opts = {
-            'noupdate': self.noupdate_chk.isChecked(),
-            'stream': self.stream_chk.isChecked(),
-            'pull': self.pproto_chk.isChecked(),
             'verbose': True,
             'config': [],
             }
-        if (self.ui.config('http_proxy', 'host')
+        for k in self._opt_checks:
+            opts[k] = self.testOption(k)
+        if (self._config.configString('http_proxy', 'host')
             and not self.proxy_chk.isChecked()):
+            assert isinstance(opts['config'], list)  # help pytype
             opts['config'].append('http_proxy.host=')
         if self.remote_chk.isChecked():
-            opts['remotecmd'] = unicode(self.remote_text.text()).strip() or None
-        if self.rev_chk.isChecked():
-            opts['rev'] = unicode(self.rev_text.text()).strip() or None
+            opts['remotecmd'] = self.remote_text.text().strip() or None
+        opts['rev'] = self.revSymbol() or None
         if self.startrev_chk.isChecked():
-            opts['startrev'] = (unicode(self.startrev_text.text()).strip()
-                                or None)
+            opts['startrev'] = self.startrev_text.text().strip() or None
         if self.largefiles_chk.isChecked():
+            assert isinstance(opts['config'], list)  # help pytype
             opts['config'].append('extensions.largefiles=')
 
         src = self.source()
@@ -301,7 +327,7 @@ class CloneWidget(cmdui.AbstractCmdWidget):
 
         if self.qclone_chk.isChecked():
             name = 'qclone'
-            opts['patches'] = unicode(self.qclone_txt.text()).strip() or None
+            opts['patches'] = self.qclone_txt.text().strip() or None
         else:
             name = 'clone'
 
@@ -322,7 +348,7 @@ class CloneWidget(cmdui.AbstractCmdWidget):
     def _browseSource(self):
         FD = QFileDialog
         caption = _("Select source repository")
-        path = FD.getExistingDirectory(self, caption, \
+        path = FD.getExistingDirectory(self, caption,
             self.src_combo.currentText(), QFileDialog.ShowDirsOnly)
         if path:
             self.src_combo.setEditText(QDir.toNativeSeparators(path))
@@ -333,7 +359,7 @@ class CloneWidget(cmdui.AbstractCmdWidget):
     def _browseDestination(self):
         FD = QFileDialog
         caption = _("Select destination repository")
-        path = FD.getExistingDirectory(self, caption, \
+        path = FD.getExistingDirectory(self, caption,
             self.dest_combo.currentText(), QFileDialog.ShowDirsOnly)
         if path:
             self.dest_combo.setEditText(QDir.toNativeSeparators(path))
@@ -344,7 +370,7 @@ class CloneWidget(cmdui.AbstractCmdWidget):
     def _browsePatchQueue(self):
         FD = QFileDialog
         caption = _("Select patch folder")
-        upatchroot = os.path.join(unicode(self.src_combo.currentText()), '.hg')
+        upatchroot = os.path.join(self.src_combo.currentText(), '.hg')
         upath = FD.getExistingDirectory(self, caption, upatchroot,
                                         QFileDialog.ShowDirsOnly)
         if upath:
@@ -361,7 +387,7 @@ class CloneDialog(cmdui.CmdControlDialog):
 
     clonedRepository = pyqtSignal(str, str)
 
-    def __init__(self, ui, args=None, opts={}, parent=None):
+    def __init__(self, ui, config, parent=None):
         super(CloneDialog, self).__init__(parent)
 
         cwd = os.getcwd()
@@ -373,7 +399,7 @@ class CloneDialog(cmdui.CmdControlDialog):
         self.setRunButtonText(_('&Clone'))
         self._cmdagent = cmdagent = cmdcore.CmdAgent(ui, self)
         cmdagent.serviceStopped.connect(self.reject)
-        self.setCommandWidget(CloneWidget(ui, cmdagent, args, opts, self))
+        self.setCommandWidget(CloneWidget(config, cmdagent, self))
         self.commandFinished.connect(self._emitCloned)
 
     def source(self):
@@ -390,6 +416,20 @@ class CloneDialog(cmdui.CmdControlDialog):
         assert self.isCommandFinished()
         self.commandWidget().setDestination(url)
 
+    def revSymbol(self):
+        return self.commandWidget().revSymbol()
+
+    def setRevSymbol(self, rev):
+        assert self.isCommandFinished()
+        self.commandWidget().setRevSymbol(rev)
+
+    def testOption(self, key):
+        return self.commandWidget().testOption(key)
+
+    def setOption(self, key, on):
+        assert self.isCommandFinished()
+        self.commandWidget().setOption(key, on)
+
     @pyqtSlot(int)
     def _emitCloned(self, ret):
         if ret == 0:
