@@ -13,6 +13,7 @@ import os
 import re
 
 from .qtcore import (
+    QDir,
     QMimeData,
     QObject,
     Qt,
@@ -27,6 +28,10 @@ from .qtgui import (
     QWidget,
 )
 
+from mercurial import (
+    pycompat,
+)
+
 from ..util import (
     hglib,
     shlib,
@@ -140,7 +145,7 @@ class FilectxActions(QObject):
 
         self._actions = {}
         self._customactions = {}
-        for name, d in self._actiontable.iteritems():
+        for name, d in self._actiontable.items():
             desc, icon, key, tip, fdfilters = d
             # QAction must be owned by QWidget; otherwise statusTip for context
             # menu cannot be displayed (QTBUG-16114)
@@ -162,6 +167,12 @@ class FilectxActions(QObject):
         # override to add actions that cannot be declared as actionSlot
         pass
 
+    def _parentWidget(self):
+        # type: () -> QWidget
+        p = self.parent()
+        assert isinstance(p, QWidget)
+        return p
+
     @property
     def _ui(self):
         repo = self._repoagent.rawRepo()
@@ -176,7 +187,8 @@ class FilectxActions(QObject):
     def _updateActions(self):
         idle = self._cmdsession.isFinished()
         selfds = self._selfds
-        allactions = self._actions.values() + self._customactions.values()
+        allactions = (list(self._actions.values())
+                    + list(self._customactions.values()))
         for act, fdfilters in allactions:
             act.setEnabled(idle and bool(_filterby(fdfilters, selfds)))
 
@@ -190,13 +202,13 @@ class FilectxActions(QObject):
 
     def actions(self):
         """List of the actions; The owner widget should register them"""
-        return [a for a, _f in self._actions.itervalues()]
+        return [a for a, _f in self._actions.values()]
 
     def action(self, name):
         return self._actions[name][0]
 
     def _addAction(self, name, action, fdfilters):
-        assert name not in self._actions
+        assert name not in self._actions, name
         self._actions[name] = action, fdfilters
 
     def _runCommand(self, cmdline):
@@ -222,7 +234,7 @@ class FilectxActions(QObject):
     @pyqtSlot(int)
     def _onCommandFinished(self, ret):
         if ret == 255:
-            cmdui.errorMessageBox(self._cmdsession, self.parent())
+            cmdui.errorMessageBox(self._cmdsession, self._parentWidget())
         self._updateActions()
 
     @actionSlot(_('File &History / Annotate'), 'hg-log', 'Shift+Return',
@@ -270,36 +282,37 @@ class FilectxActions(QObject):
     @actionSlot(_('Diff &Changeset to Parent'), 'visualdiff', None, '',
                 _notpatch)
     def visualDiff(self, fds):
-        self._visualDiffToBase(fds, [])
+        self._visualDiffToBase(fds[0], [])
 
     @actionSlot(_('Diff Changeset to Loc&al'), 'ldiff', None, '',
                 _committed)
     def visualDiffToLocal(self, fds):
-        self._visualDiff(fds, [], rev=['rev(%d)' % fds[0].rev()])
+        self._visualDiff(fds[0], [], rev=['rev(%d)' % fds[0].rev()])
 
     @actionSlot(_('&Diff to Parent'), 'visualdiff', 'Ctrl+D',
                 _('View file changes in external diff tool'),
                 (_notpatch, _notsubroot, _filestatus('MAR!')))
     def visualDiffFile(self, fds):
-        self._visualDiffToBase(fds, _lcanonpaths(fds))
+        self._visualDiffToBase(fds[0], fds)
 
     @actionSlot(_('Diff to &Local'), 'ldiff', 'Shift+Ctrl+D',
                 _('View changes to current in external diff tool'),
                 _committed)
     def visualDiffFileToLocal(self, fds):
-        self._visualDiff(fds, _lcanonpaths(fds), rev=['rev(%d)' % fds[0].rev()])
+        self._visualDiff(fds[0], fds, rev=['rev(%d)' % fds[0].rev()])
 
-    def _visualDiffToBase(self, fds, filenames):
-        if fds[0].baseRev() == fds[0].parentRevs()[0]:
-            self._visualDiff(fds, filenames, change=fds[0].rev())  # can 3-way
+    def _visualDiffToBase(self, an_fd, fds):
+        if an_fd.baseRev() == an_fd.parentRevs()[0]:
+            self._visualDiff(an_fd, fds, change=an_fd.rev())  # can 3-way
         else:
-            revs = [fds[0].baseRev()]
-            if fds[0].rev() is not None:
-                revs.append(fds[0].rev())
-            self._visualDiff(fds, filenames, rev=['rev(%d)' % r for r in revs])
+            revs = [an_fd.baseRev()]
+            if an_fd.rev() is not None:
+                revs.append(an_fd.rev())
+            self._visualDiff(an_fd, fds, rev=['rev(%d)' % r for r in revs])
 
-    def _visualDiff(self, fds, filenames, **opts):
-        repo = self._repoAgentFor(fds[0]).rawRepo()
+    def _visualDiff(self, an_fd, fds, **opts):
+        filenames = _lcanonpaths(fds)
+        repo = self._repoAgentFor(an_fd).rawRepo()
         dlg = visdiff.visualdiff(repo.ui, repo, filenames, opts)
         if dlg:
             dlg.exec_()
@@ -316,7 +329,7 @@ class FilectxActions(QObject):
         base, _ = visdiff.snapshot(repo, filenames, ctx)
         files = [os.path.join(base, filename)
                  for filename in filenames]
-        qtlib.editfiles(repo, files, parent=self.parent())
+        qtlib.editfiles(repo, files, parent=self._parentWidget())
 
     @actionSlot(_('&Save at Revision...'), None, 'Shift+Ctrl+S',
                 _('Save file as it appeared at this revision'),
@@ -331,7 +344,7 @@ class FilectxActions(QObject):
                 extfilter.insert(0, "*%s" % ext)
 
             result, _filter = QFileDialog.getSaveFileName(
-                self.parent(), _("Save file to"), filename,
+                self._parentWidget(), _("Save file to"), filename,
                 ";;".join(extfilter))
             if not result:
                 continue
@@ -349,15 +362,15 @@ class FilectxActions(QObject):
     def editLocalFile(self, fds):
         repo = self._repoAgentFor(fds[0]).rawRepo()
         filenames = _lcanonpaths(fds)
-        qtlib.editfiles(repo, filenames, parent=self.parent())
+        qtlib.editfiles(repo, filenames, parent=self._parentWidget())
 
     @actionSlot(_('&Open Local'), None, 'Shift+Ctrl+L',
                 _('Edit current file in working copy'),
                 (_isfile, _filestatus('MACI?')))
     def openLocalFile(self, fds):
         repo = self._repoAgentFor(fds[0]).rawRepo()
-        filenames = _lcanonpaths(fds)
-        qtlib.openfiles(repo, filenames)
+        for fd in fds:
+            qtlib.openlocalurl(fd.absoluteFilePath())
 
     @actionSlot(_('E&xplore Local'), 'system-file-manager', None,
                 _('Open parent folder of current file in the system file '
@@ -371,7 +384,8 @@ class FilectxActions(QObject):
                 (_notpatch, _notsubroot, _filestatus('MAR!')))
     def copyPatch(self, fds):
         paths = [hglib.escapepath(fd.filePath()) for fd in fds]
-        revs = map(hglib.escaperev, [fds[0].baseRev(), fds[0].rev()])
+        revs = pycompat.maplist(hglib.escaperev,
+                                [fds[0].baseRev(), fds[0].rev()])
         cmdline = hglib.buildcmdargs('diff', *paths, r=revs)
         sess = self._runCommand(cmdline)
         sess.setCaptureOutput(True)
@@ -387,24 +401,30 @@ class FilectxActions(QObject):
         mdata.setText(hglib.tounicode(str(output)))
         QApplication.clipboard().setMimeData(mdata)
 
-    @actionSlot(_('Copy &Path'), None, 'Shift+Ctrl+C',
+    @actionSlot(_('Copy Absolute &Path'), None, 'Shift+Ctrl+C',
                 _('Copy full path of file(s) to the clipboard'))
     def copyPath(self, fds):
         paths = [e.absoluteFilePath() for e in fds]
         QApplication.clipboard().setText(os.linesep.join(paths))
 
+    @actionSlot(_('Copy Relative Path'), None, None,
+                _('Copy repository relative path of file(s) to the clipboard'))
+    def copyRelativePath(self, fds):
+        paths = [QDir.toNativeSeparators(e.filePath()) for e in fds]
+        QApplication.clipboard().setText(os.linesep.join(paths))
+
     @actionSlot(_('&Revert to Revision...'), 'hg-revert', 'Shift+Ctrl+R',
                 _('Revert file(s) to contents at this revision'),
                 _notpatch)
     def revertFile(self, fds):
         repoagent = self._repoAgentFor(fds[0])
-        fileSelection = _lcanonpaths(fds)
+        fileSelection = [e.canonicalFilePath() for e in fds]
         rev = fds[0].rev()
         if rev is None:
             repo = repoagent.rawRepo()
             rev = repo[rev].p1().rev()
         dlg = revert.RevertDialog(repoagent, fileSelection, rev,
-                                  parent=self.parent())
+                                  parent=self._parentWidget())
         dlg.exec_()
 
     @actionSlot(_('Open S&ubrepository'), 'thg-repository-open', None,
@@ -439,7 +459,7 @@ class FilectxActions(QObject):
 
     def setupCustomToolsMenu(self, location):
         tools, toollist = hglib.tortoisehgtools(self._ui, location)
-        submenu = QMenu(_('Custom Tools'), self.parent())
+        submenu = QMenu(_('Custom Tools'), self._parentWidget())
         submenu.triggered.connect(self._runCustomCommandByMenu)
         for name in toollist:
             if name == '|':
@@ -491,7 +511,7 @@ class WctxActions(FilectxActions):
         qtlib.setContextMenuShortcut(a, 'Ctrl+Shift+E')
         a.setShortcutContext(Qt.WidgetWithChildrenShortcut)
         a = self.action('addLargefile')
-        a.setVisible('largefiles' in repo.extensions())
+        a.setVisible(b'largefiles' in repo.extensions())
         self._addAction('renameFileMenu', *self._createRenameFileMenu())
         self._addAction('remergeFileMenu', *self._createRemergeFileMenu())
 
@@ -525,7 +545,7 @@ class WctxActions(FilectxActions):
     @actionSlot(_('Diff &Local'), 'ldiff', 'Ctrl+Shift+D', '',
                 (_indirectbaserev, _notsubroot, _filestatus('MARC!')))
     def visualDiffLocalFile(self, fds):
-        self._visualDiff(fds, _lcanonpaths(fds))
+        self._visualDiff(fds[0], fds)
 
     @actionSlot(_('&View Missing'), None, None, '',
                 (_isfile, _filestatus('R!')))
@@ -543,14 +563,14 @@ class WctxActions(FilectxActions):
                 (_notsubroot, _filestatus('RI?')))
     def addFile(self, fds):
         repo = self._repoAgentFor(fds[0]).rawRepo()
-        if 'largefiles' in repo.extensions():
+        if b'largefiles' in repo.extensions():
             self._addFileWithPrompt(fds)
         else:
             self._runWorkingFileCommand('add', fds)
 
     def _addFileWithPrompt(self, fds):
         repo = self._repoAgentFor(fds[0]).rawRepo()
-        result = lfprompt.promptForLfiles(self.parent(), repo.ui, repo,
+        result = lfprompt.promptForLfiles(self._parentWidget(), repo.ui, repo,
                                           _lcanonpaths(fds))
         if not result:
             return
@@ -576,7 +596,7 @@ class WctxActions(FilectxActions):
     @actionSlot(_('&Delete Unversioned...'), 'hg-purge', 'Delete', '',
                 (_notsubroot, _filestatus('?I')))
     def purgeFile(self, fds):
-        parent = self.parent()
+        parent = self._parentWidget()
         files = [hglib.fromunicode(fd.filePath()) for fd in fds]
         res = qtlib.CustomPrompt(
             _('Confirm Delete Unversioned'),
@@ -595,7 +615,7 @@ class WctxActions(FilectxActions):
     @actionSlot(_('&Revert...'), 'hg-revert', None, '',
                 _filestatus('MAR!'))
     def revertWorkingFile(self, fds):
-        parent = self.parent()
+        parent = self._parentWidget()
         files = _lcanonpaths(fds)
         wctx = fds[0].rawContext()
         revertopts = {'date': None, 'rev': '.', 'all': False}
@@ -643,8 +663,8 @@ class WctxActions(FilectxActions):
         from tortoisehg.hgqt.rename import RenameDialog
         srcfd, = fds
         repoagent = self._repoAgentFor(srcfd)
-        dlg = RenameDialog(repoagent, self.parent(), srcfd.canonicalFilePath(),
-                           iscopy=iscopy)
+        dlg = RenameDialog(repoagent, self._parentWidget(),
+                           srcfd.canonicalFilePath(), iscopy=iscopy)
         if dlg.exec_() == 0:
             self._notifyChanges()
 
@@ -653,7 +673,7 @@ class WctxActions(FilectxActions):
     def editHgignore(self, fds):
         from tortoisehg.hgqt.hgignore import HgignoreDialog
         repoagent = self._repoAgentFor(fds[0])
-        parent = self.parent()
+        parent = self._parentWidget()
         files = _lcanonpaths(fds)
         dlg = HgignoreDialog(repoagent, parent, *files)
         dlg.finished.connect(dlg.deleteLater)
@@ -665,7 +685,7 @@ class WctxActions(FilectxActions):
                 (_single, _isfile, _filestatus('?I'), _filepath(r'\.rej$')))
     def editRejects(self, fds):
         lpath = hglib.fromunicode(fds[0].absoluteFilePath()[:-4])  # drop .rej
-        dlg = rejects.RejectsDialog(self._ui, lpath, self.parent())
+        dlg = rejects.RejectsDialog(self._ui, lpath, self._parentWidget())
         if dlg.exec_():
             self._notifyChanges()
 
@@ -674,7 +694,7 @@ class WctxActions(FilectxActions):
     def guessRename(self, fds):
         from tortoisehg.hgqt.guess import DetectRenameDialog
         repoagent = self._repoAgentFor(fds[0])
-        parent = self.parent()
+        parent = self._parentWidget()
         files = _lcanonpaths(fds)
         dlg = DetectRenameDialog(repoagent, parent, *files)
         def matched():
@@ -702,7 +722,7 @@ class WctxActions(FilectxActions):
         self._runWorkingFileCommand('resolve', fds)
 
     def _createRenameFileMenu(self):
-        menu = QMenu(_('Was renamed from'), self.parent())
+        menu = QMenu(_('Was renamed from'), self._parentWidget())
         menu.aboutToShow.connect(self._updateRenameFileMenu)
         menu.triggered.connect(self._renameFrom)
         fdfilters = (_single, _isfile, _filestatus('?'), _anydeleted)
@@ -711,7 +731,7 @@ class WctxActions(FilectxActions):
     @pyqtSlot()
     def _updateRenameFileMenu(self):
         menu = self.sender()
-        assert isinstance(menu, QMenu)
+        assert isinstance(menu, QMenu), repr(menu)
         menu.clear()
         fds = self.fileDataListForAction('renameFileMenu')
         if not fds:
@@ -734,7 +754,7 @@ class WctxActions(FilectxActions):
         sess.commandFinished.connect(self._notifyChangesOnCommandFinished)
 
     def _createRemergeFileMenu(self):
-        menu = QMenu(_('Restart Merge &with'), self.parent())
+        menu = QMenu(_('Restart Merge &with'), self._parentWidget())
         menu.aboutToShow.connect(self._populateRemergeFileMenu)  # may be slow
         menu.triggered.connect(self._remergeFileWith)
         return menu.menuAction(), (_notsubroot, _mergestatus('U'))
@@ -742,7 +762,7 @@ class WctxActions(FilectxActions):
     @pyqtSlot()
     def _populateRemergeFileMenu(self):
         menu = self.sender()
-        assert isinstance(menu, QMenu)
+        assert isinstance(menu, QMenu), repr(menu)
         menu.aboutToShow.disconnect(self._populateRemergeFileMenu)
         for tool in hglib.mergetools(self._ui):
             menu.addAction(hglib.tounicode(tool))
