@@ -33,6 +33,10 @@ from .qtgui import (
     QVBoxLayout,
 )
 
+from hgext.largefiles import (
+    lfutil,
+)
+
 from mercurial import (
     commands,
     error,
@@ -92,9 +96,9 @@ class HgignoreDialog(QDialog):
         repo = repoagent.rawRepo()
         hbox = QHBoxLayout()
         vbox.addLayout(hbox)
-        ignorefiles = [repo.wjoin('.hgignore')]
-        for name, value in repo.ui.configitems('ui'):
-            if name == 'ignore' or name.startswith('ignore.'):
+        ignorefiles = [repo.wjoin(b'.hgignore')]
+        for name, value in repo.ui.configitems(b'ui'):
+            if name == b'ignore' or name.startswith(b'ignore.'):
                 ignorefiles.append(util.expandpath(value))
 
         filecombo = QComboBox()
@@ -200,6 +204,7 @@ class HgignoreDialog(QDialog):
                 dirname = os.path.dirname(dirname)
             base, ext = os.path.splitext(local)
             if ext:
+                ext = hglib.tounicode(ext)
                 filters.append(['*'+ext])
                 filters.append(['**'+ext])
         else:
@@ -214,14 +219,14 @@ class HgignoreDialog(QDialog):
     def unknownDoubleClicked(self, item):
         self.insertFilters([hglib.fromunicode(item.text())])
 
-    def insertFilters(self, pats=False, isregexp=False):
-        if pats is False:
+    def insertFilters(self, pats=None, isregexp=False):
+        if not pats:
             pats = self.sender()._patterns
-        h = isregexp and 'syntax: regexp' or 'syntax: glob'
+        h = isregexp and b'syntax: regexp' or b'syntax: glob'
         if h in self.ignorelines:
             l = self.ignorelines.index(h)
             for i, line in enumerate(self.ignorelines[l+1:]):
-                if line.startswith('syntax:'):
+                if line.startswith(b'syntax:'):
                     for pat in pats:
                         self.ignorelines.insert(l+i+1, pat)
                     break
@@ -244,38 +249,40 @@ class HgignoreDialog(QDialog):
         self.refresh()
 
     def editClicked(self):
-        if qscilib.fileEditor(self.ignorefile) == QDialog.Accepted:
+        ignfile = hglib.tounicode(self.ignorefile)
+        if qscilib.fileEditor(ignfile) == QDialog.Accepted:
             self.refresh()
 
     def addEntry(self):
         newfilter = hglib.fromunicode(self.le.text()).strip()
-        if newfilter == '':
+        if newfilter == b'':
             return
         self.le.clear()
         if self.recombo.currentIndex() == 0:
-            test = 'glob:' + newfilter
+            test = b'glob:' + newfilter
             try:
-                match.match(self.repo.root, '', [], [test])
+                match.match(self.repo.root, b'', [], [test])
                 self.insertFilters([newfilter], False)
-            except error.Abort, inst:
+            except error.Abort as inst:
                 qtlib.WarningMsgBox(_('Invalid glob expression'), str(inst),
                                     parent=self)
                 return
         else:
-            test = 'relre:' + newfilter
+            test = b'relre:' + newfilter
             try:
-                match.match(self.repo.root, '', [], [test])
+                match.match(self.repo.root, b'', [], [test])
                 re.compile(test)
                 self.insertFilters([newfilter], True)
-            except (error.Abort, re.error), inst:
+            except (error.Abort, re.error) as inst:
                 qtlib.WarningMsgBox(_('Invalid regexp expression'), str(inst),
                                     parent=self)
                 return
 
     def refresh(self):
         try:
-            l = open(self.ignorefile, 'rb').readlines()
-            self.doseoln = l[0].endswith('\r\n')
+            with open(self.ignorefile, 'rb') as fp:
+                l = fp.readlines()
+            self.doseoln = l[0].endswith(b'\r\n')
         except (IOError, ValueError, IndexError):
             self.doseoln = os.name == 'nt'
             l = []
@@ -288,13 +295,12 @@ class HgignoreDialog(QDialog):
 
         try:
             self.repo.thginvalidate()
-            self.repo.lfstatus = True
-            self.lclunknowns = self.repo.status(unknown=True)[4]
-            self.repo.lfstatus = False
-        except (EnvironmentError, error.RepoError), e:
+            with lfutil.lfstatus(self.repo):
+                self.lclunknowns = self.repo.status(unknown=True).unknown
+        except (EnvironmentError, error.RepoError) as e:
             qtlib.WarningMsgBox(_('Unable to read repository status'),
                                 uni(str(e)), parent=self)
-        except error.Abort, e:
+        except error.Abort as e:
             if e.hint:
                 err = _('%s (hint: %s)') % (uni(str(e)), uni(e.hint))
             else:
@@ -317,16 +323,16 @@ class HgignoreDialog(QDialog):
                 item = self.unknownlist.item(i)
                 item.setSelected(True)
                 self.unknownlist.setCurrentItem(item)
-                self.le.setText(u)
+                self.le.setText(hglib.tounicode(u))
         self.pats = []
 
     def writeIgnoreFile(self):
-        eol = self.doseoln and '\r\n' or '\n'
+        eol = self.doseoln and b'\r\n' or b'\n'
         out = eol.join(self.ignorelines) + eol
         hasignore = os.path.exists(self.repo.vfs.join(self.ignorefile))
 
         try:
-            f = util.atomictempfile(self.ignorefile, 'wb', createmode=None)
+            f = util.atomictempfile(self.ignorefile, b'wb', createmode=None)
             f.write(out)
             f.close()
             if not hasignore:
@@ -339,7 +345,7 @@ class HgignoreDialog(QDialog):
                     commands.add(hglib.loadui(), self.repo, self.ignorefile)
             shlib.shell_notify([self.ignorefile])
             self.ignoreFilterUpdated.emit()
-        except EnvironmentError, e:
+        except EnvironmentError as e:
             qtlib.WarningMsgBox(_('Unable to write .hgignore file'),
                                 hglib.tounicode(str(e)), parent=self)
 
