@@ -5,51 +5,61 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2, incorporated herein by reference.
 
-import os, cgi, time
+from __future__ import absolute_import, print_function
 
-from mercurial import ui
+import time
+
+from mercurial import (
+    ui as ui_mod,
+    url as url_mod,
+)
 from tortoisehg.hgqt import qtlib
 from tortoisehg.util import hglib
 
-BEGINTAG = '\033' + str(time.time())
-ENDTAG = '\032' + str(time.time())
+BEGINTAG = b'\033' + hglib.fromunicode(str(time.time()))
+ENDTAG = b'\032' + hglib.fromunicode(str(time.time()))
 
-class htmlui(ui.ui):
+class htmlui(ui_mod.ui):
     def __init__(self, src=None):
         super(htmlui, self).__init__(src)
-        self.setconfig('ui', 'interactive', 'off')
-        self.setconfig('progress', 'disable', 'True')
+        self.setconfig(b'ui', b'interactive', b'off')
+        self.setconfig(b'progress', b'disable', b'True')
         self.output, self.error = [], []
 
     def write(self, *args, **opts):
-        label = opts.get('label', '')
         if self._buffers:
-            self._buffers[-1].extend([(str(a), label) for a in args])
-        else:
-            self.output.extend(self.smartlabel(''.join(args), label))
+            return super(htmlui, self).write(*args, **opts)
+        label = opts.get('label', b'')
+        self.output.extend(self.smartlabel(b''.join(args), label))
 
     def write_err(self, *args, **opts):
-        label = opts.get('label', 'ui.error')
-        self.error.extend(self.smartlabel(''.join(args), label))
+        if self._buffers:
+            return super(htmlui, self).write_err(*args, **opts)
+        label = opts.get('label', b'ui.error')
+        self.error.extend(self.smartlabel(b''.join(args), label))
 
     def label(self, msg, label):
         '''
         Called by Mercurial to apply styling (formatting) to a piece of
         text.  Our implementation wraps tags around the data so we can
-        find it later when it is passed to ui.write()
+        find it later when it is passed to smartlabel through ui.write()
         '''
+        if self._buffers:
+            return super(htmlui, self).label(msg, label)
         return BEGINTAG + self.style(msg, label) + ENDTAG
 
     def style(self, msg, label):
-        'Escape message for safe HTML, then apply specified style'
-        msg = cgi.escape(msg).replace('\n', '<br />')
-        style = qtlib.geteffect(label)
-        return '<span style="%s">%s</span>' % (style, msg)
+        '''htmlui specific method.
+        Escape message as safe HTML with style from specified label.
+        '''
+        msg = url_mod.escape(msg).replace(b'\n', b'<br />')
+        style = hglib.fromunicode(qtlib.geteffect(hglib.tounicode(label)))
+        return b'<span style="%s">%s</span>' % (style, msg)
 
     def smartlabel(self, text, label):
-        '''
-        Escape and apply style, excluding any text between BEGINTAG and
-        ENDTAG.  That text has already been escaped and styled.
+        '''htmlui specific method.
+        Escape and apply style from label on text, excluding any text between
+        BEGINTAG and ENDTAG which already must have been escaped and styled.
         '''
         parts = []
         try:
@@ -73,23 +83,21 @@ class htmlui(ui.ui):
             parts.append(self.style(text, label))
         return parts
 
-    def popbuffer(self, labeled=False):
-        b = self._buffers.pop()
-        if labeled:
-            return ''.join(self.style(a, label) for a, label in b)
-        return ''.join(a for a, label in b)
-
     def plain(self, feature=None):
+        '''Always pretend HGPLAIN when using htmlui'''
         return True
 
     def getdata(self):
-        d, e = ''.join(self.output), ''.join(self.error)
+        '''htmlui specific method.
+        Retrieve buffered htmlui output and error as unicode html.
+        '''
+        d, e = b''.join(self.output), b''.join(self.error)
         self.output, self.error = [], []
         return d, e
 
 if __name__ == "__main__":
     from mercurial import hg
-    u = htmlui()
+    u = htmlui.load()
     repo = hg.repository(u)
     repo.status()
-    print u.getdata()[0]
+    print(u.getdata()[0])
