@@ -7,9 +7,10 @@
 
 import os
 
-from mercurial import match
+from mercurial import error, match
 from tortoisehg.hgqt import qtlib
-from tortoisehg.hgqt.i18n import _
+from tortoisehg.util import hglib
+from tortoisehg.util.i18n import _
 
 def _createPrompt(parent, files):
     return qtlib.CustomPrompt(
@@ -25,15 +26,22 @@ def _createPrompt(parent, files):
 
 def promptForLfiles(parent, ui, repo, files):
     lfiles = []
-    section = 'largefiles'
+    section = b'largefiles'
     try:
-        minsize = int(ui.config(section, 'minsize', default='10'))
+        minsize = float(ui.config(section, b'minsize', default=10))
     except ValueError:
         minsize = 10
-    patterns = ui.config(section, 'patterns', default=())
+    patterns = ui.config(section, b'patterns')
     if patterns:
-        patterns = patterns.split(' ')
-        matcher = match.match(repo.root, '', list(patterns))
+        patterns = patterns.split(b' ')
+        try:
+            matcher = match.match(repo.root, b'', list(patterns))
+        except error.Abort as e:
+            qtlib.WarningMsgBox(_('Invalid Patterns'),
+                                _('Failed to process largefiles.patterns.'),
+                                hglib.tounicode(str(e)),
+                                parent=parent)
+            return None
     else:
         matcher = None
     for wfile in files:
@@ -42,9 +50,12 @@ def promptForLfiles(parent, ui, repo, files):
             lfiles.append(wfile)
         else:
             # check for minimal size
-            filesize = os.path.getsize(repo.wjoin(wfile))
-            if filesize > minsize*1024*1024:
-                lfiles.append(wfile)
+            try:
+                filesize = os.path.getsize(repo.wjoin(wfile))
+                if filesize > minsize * 1024 * 1024:
+                    lfiles.append(wfile)
+            except OSError:
+                pass  # file not exist or inaccessible
     if lfiles:
         ret = _createPrompt(parent, files).run()
         if ret == 0:
