@@ -5,14 +5,38 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2, incorporated herein by reference.
 
-from mercurial import error
+from __future__ import absolute_import
 
-from tortoisehg.util import hglib
-from tortoisehg.util.i18n import _
-from tortoisehg.hgqt import csinfo, qtlib
+from .qtcore import (
+    QSettings,
+    Qt,
+    pyqtSlot,
+)
+from .qtgui import (
+    QAbstractButton,
+    QButtonGroup,
+    QCheckBox,
+    QComboBox,
+    QDialog,
+    QDialogButtonBox,
+    QGridLayout,
+    QLabel,
+    QLayout,
+    QSizePolicy,
+    QVBoxLayout,
+)
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui import *
+from mercurial import (
+    error,
+    scmutil,
+)
+
+from ..util import hglib
+from ..util.i18n import _
+from . import (
+    csinfo,
+    qtlib,
+)
 
 class MatchDialog(QDialog):
 
@@ -51,7 +75,7 @@ class MatchDialog(QDialog):
         # make it easy to match the workding directory parent revision
         combo.addItem(hglib.tounicode('.'))
 
-        tags = list(self.repo.tags()) + self.repo._bookmarks.keys()
+        tags = list(self.repo.tags()) + list(self.repo._bookmarks.keys())
         tags.sort(reverse=True)
         for tag in tags:
             combo.addItem(hglib.tounicode(tag))
@@ -116,18 +140,20 @@ class MatchDialog(QDialog):
         s = QSettings()
 
         #### Persisted Options
-        self.summary_chk.setChecked(s.value('matching/summary', False).toBool())
+        self.summary_chk.setChecked(
+            qtlib.readBool(s, 'matching/summary', False))
         self.description_chk.setChecked(
-            s.value('matching/description', True).toBool())
-        self.author_chk.setChecked(s.value('matching/author', True).toBool())
-        self.branch_chk.setChecked(s.value('matching/branch', False).toBool())
-        self.date_chk.setChecked(s.value('matching/date', True).toBool())
-        self.files_chk.setChecked(s.value('matching/files', False).toBool())
-        self.diff_chk.setChecked(s.value('matching/diff', False).toBool())
-        self.parents_chk.setChecked(s.value('matching/parents', False).toBool())
-        self.phase_chk.setChecked(s.value('matching/phase', False).toBool())
+            qtlib.readBool(s, 'matching/description', True))
+        self.author_chk.setChecked(qtlib.readBool(s, 'matching/author', True))
+        self.branch_chk.setChecked(qtlib.readBool(s, 'matching/branch', False))
+        self.date_chk.setChecked(qtlib.readBool(s, 'matching/date', True))
+        self.files_chk.setChecked(qtlib.readBool(s, 'matching/files', False))
+        self.diff_chk.setChecked(qtlib.readBool(s, 'matching/diff', False))
+        self.parents_chk.setChecked(
+            qtlib.readBool(s, 'matching/parents', False))
+        self.phase_chk.setChecked(qtlib.readBool(s, 'matching/phase', False))
         self.substate_chk.setChecked(
-            s.value('matching/substate', False).toBool())
+            qtlib.readBool(s, 'matching/substate', False))
 
         ## bottom buttons
         buttons = QDialogButtonBox()
@@ -203,11 +229,11 @@ class MatchDialog(QDialog):
 
         self.rev_to_match_info_lbl.setText(_('Revision to Match:'))
         new_rev = hglib.fromunicode(self.rev_combo.currentText())
-        if new_rev.lower() == 'null':
+        if new_rev.lower() == b'null':
             self.match_btn.setEnabled(True)
             return
         try:
-            csinfo_update(self.repo[new_rev])
+            csinfo_update(scmutil.revsymbol(self.repo, new_rev))
             return
         except (error.LookupError, error.RepoLookupError, error.RepoError):
             pass
@@ -222,7 +248,7 @@ class MatchDialog(QDialog):
                     % len(rset))
             else:
                 self.rev_to_match_info_lbl.setText(_('Revision to Match:'))
-                csinfo_update(rset[0])
+                csinfo_update(rset.first())
             validrevset = True
         except (error.LookupError, error.RepoLookupError):
             csinfo_set_text(_('<b>Unknown revision!</b>'))
