@@ -8,6 +8,10 @@
 
 from __future__ import absolute_import
 
+from mercurial import (
+    pycompat,
+)
+
 from .qtcore import (
     QSettings,
     QSize,
@@ -51,7 +55,7 @@ class MergeDialog(QWizard):
 
     def __init__(self, repoagent, otherrev, parent=None):
         super(MergeDialog, self).__init__(parent)
-        assert isinstance(otherrev, int)
+        assert isinstance(otherrev, int), repr(otherrev)
         self._repoagent = repoagent
         f = self.windowFlags()
         self.setWindowFlags(f & ~Qt.WindowContextHelpButtonHint)
@@ -87,7 +91,8 @@ class MergeDialog(QWizard):
             self.setField(n, qs.value(n, False))
         repo = self._repoagent.rawRepo()
         n = 'autoresolve'
-        self.setField(n, repo.ui.configbool('tortoisehg', n,
+
+        self.setField(n, repo.ui.configbool(b'tortoisehg', pycompat.sysbytes(n),
                                             qtlib.readBool(qs, n, True)))
         qs.endGroup()
 
@@ -178,11 +183,11 @@ class SummaryPage(BasePage):
         self.setLayout(QVBoxLayout())
 
         repo = self.repo
-        contents = ('ishead',) + csinfo.PANEL_DEFAULT
+        contents = ('createsnewhead',) + csinfo.PANEL_DEFAULT
         style = csinfo.panelstyle(contents=contents)
         def markup_func(widget, item, value):
-            if item == 'ishead' and value is False:
-                text = _('Not a head revision!')
+            if item == 'createsnewhead' and value is True:
+                text = _('Creates new head!')
                 return qtlib.markup(text, fg='red', weight='bold')
             raise csinfo.UnknownItem(item)
         custom = csinfo.custom(markup=markup_func)
@@ -198,7 +203,7 @@ class SummaryPage(BasePage):
         ## current revision
         local_sep = qtlib.LabeledSeparator(_('Merge to (working directory)'))
         self.layout().addWidget(local_sep)
-        localCsInfo = create(repo['.'].rev())
+        localCsInfo = create(repo[b'.'].rev())
         self.layout().addWidget(localCsInfo)
         self.localCsInfo = localCsInfo
 
@@ -287,7 +292,7 @@ class SummaryPage(BasePage):
 
     def repositoryChanged(self):
         'repository has detected a change to changelog or parents'
-        pctx = self.repo['.']
+        pctx = self.repo[b'.']
         self.localCsInfo.update(pctx)
 
     def canExit(self):
@@ -397,9 +402,9 @@ class MergePage(BasePage):
         ucount = 0
         rcount = 0
         for root, path, status in thgrepo.recursiveMergeStatus(self.repo):
-            if status == 'u':
+            if status == b'u':
                 ucount += 1
-            if status == 'r':
+            if status == b'r':
                 rcount += 1
         if ucount:
             if self.field('autoresolve'):
@@ -473,7 +478,8 @@ class CommitPage(BasePage):
                     branch = None
                     if hasattr(pctx, 'branch') and pctx.branch() != cbranch:
                         branch = pctx.branch()
-                    parents.append((str(pctx.rev()), str(pctx), branch, pctx))
+                    parents.append((str(pctx.rev()), str(pctx),
+                                    hglib.tounicode(branch), pctx))
                 return parents
             raise csinfo.UnknownItem()
         def markup_func(widget, item, value):
@@ -543,7 +549,7 @@ class CommitPage(BasePage):
         self.layout().addWidget(skiplast)
 
         hblayout = QHBoxLayout()
-        self.opts = commit.readopts(self.repo.ui)
+        self.opts = commit.readopts(self._repoagent)
         self.optionsbtn = QPushButton(_('Commit Options'))
         self.optionsbtn.clicked.connect(self.details)
         hblayout.addWidget(self.optionsbtn)
@@ -601,7 +607,7 @@ class CommitPage(BasePage):
 
         # username will be prompted as necessary by hg if ui.askusername
         user = self.opts.get('user')
-        if not self.repo.ui.configbool('ui', 'askusername'):
+        if not self.repo.ui.configbool(b'ui', b'askusername'):
             user = hglib.tounicode(qtlib.getCurrentUsername(self, self.repo,
                                                             self.opts))
             if not user:
@@ -617,7 +623,7 @@ class CommitPage(BasePage):
                 'date': hglib.tounicode(self.opts.get('date')),
                 }
         commandlines = [hglib.buildcmdargs('commit', **opts)]
-        pushafter = self.repo.ui.config('tortoisehg', 'cipushafter')
+        pushafter = self.repo.ui.config(b'tortoisehg', b'cipushafter')
         if pushafter:
             cmd = ['push', hglib.tounicode(pushafter)]
             commandlines.append(cmd)
@@ -679,5 +685,5 @@ class ResultPage(BasePage):
 
     def currentPage(self):
         super(ResultPage, self).currentPage()
-        self.mergeCsInfo.update(self.repo['tip'])
+        self.mergeCsInfo.update(self.repo[b'tip'])
         self.wizard().setOption(QWizard.NoCancelButton, True)
