@@ -5,15 +5,35 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2, incorporated herein by reference.
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui import *
-from PyQt4.Qsci import QsciScintilla, QsciLexerMakefile
-
-from tortoisehg.util.i18n import _
-from tortoisehg.hgqt import qtlib, qscilib
+from __future__ import absolute_import
 
 import re
 
+from .qsci import (
+    QsciScintilla,
+    QsciLexerMakefile,
+)
+from .qtcore import (
+    QPoint,
+    QSettings,
+    Qt,
+    pyqtSlot,
+)
+from .qtgui import (
+    QColor,
+    QFontMetrics,
+)
+
+from mercurial import (
+    pycompat,
+)
+
+from ..util.i18n import _
+from . import (
+    qscilib,
+    qtlib,
+)
+
 class MessageEntry(qscilib.Scintilla):
 
     def __init__(self, parent, getCheckedFunc=None):
@@ -32,7 +52,7 @@ class MessageEntry(qscilib.Scintilla):
         self.setMatchedBraceBackgroundColor(Qt.yellow)
         self.setIndentationsUseTabs(False)
         self.setBraceMatching(QsciScintilla.SloppyBraceMatch)
-        # http://www.riverbankcomputing.com/pipermail/qscintilla/2009-February/000461.html
+        # https://www.riverbankcomputing.com/pipermail/qscintilla/2009-February/000461.html
         self.setHorizontalScrollBarPolicy(Qt.ScrollBarAsNeeded)
         self.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)
         # default message entry widgets to word wrap, user may override
@@ -53,7 +73,7 @@ class MessageEntry(qscilib.Scintilla):
     def applylexer(self):
         font = qtlib.getfont('fontcomment').font()
         self.fontHeight = QFontMetrics(font).height()
-        if QSettings().value('msgentry/lexer', True).toBool():
+        if qtlib.readBool(QSettings(), 'msgentry/lexer', True):
             self.setLexer(QsciLexerMakefile(self))
             self.lexer().setColor(QColor(Qt.red), QsciLexerMakefile.Error)
             self.lexer().setFont(font)
@@ -120,7 +140,7 @@ class MessageEntry(qscilib.Scintilla):
         self.summarylen = repo.summarylen
 
     def reflowBlock(self, line):
-        lines = unicode(self.text()).splitlines()
+        lines = pycompat.unicode(self.text()).splitlines()
         if line >= len(lines):
             return None
         if not len(lines[line]) > 1:
@@ -132,7 +152,7 @@ class MessageEntry(qscilib.Scintilla):
             # or are empty
             if not linetext:
                 return True
-            if (linetext[0] in '#-*+'):
+            if linetext[0] in '#-*+':
                 return True
             if len(linetext) >= 2:
                 if linetext[:2] in ('> ', '| '):
@@ -179,7 +199,7 @@ class MessageEntry(qscilib.Scintilla):
         if b == e == 0:
             return line + 1
 
-        group = [lines[l] for l in xrange(b, e+1)]
+        group = [lines[l] for l in pycompat.xrange(b, e+1)]
         MARKER = u'\033\033\033\033\033'
         curlinenum, curcol = self.getCursorPosition()
         if b <= curlinenum <= e:
@@ -254,5 +274,5 @@ class MessageEntry(qscilib.Scintilla):
 
     def minimumSizeHint(self):
         size = super(MessageEntry, self).minimumSizeHint()
-        size.setHeight(self.fontHeight * 3 / 2)
+        size.setHeight(self.fontHeight * 3 // 2)
         return size
