@@ -7,24 +7,47 @@
 
 """A dialog to allow users to post a review to reviewboard
 
-http:///www.reviewboard.org
+https://www.reviewboard.org
 
 This dialog requires a fork of the review board mercurial plugin, maintained
 by mdelagra, that can be downloaded from:
 
-https://bitbucket.org/mdelagra/mercurial-reviewboard/
+https://foss.heptapod.net/mercurial/tortoisehg/thg-build-deps/mercurial-reviewboard
 
 More information can be found at http://www.mikeyd.com.au/tortoisehg-reviewboard
 """
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui import *
-from mercurial import extensions, scmutil
-from tortoisehg.util import hglib
-from tortoisehg.util.i18n import _
-from tortoisehg.hgqt import cmdcore, qtlib
-from tortoisehg.hgqt.postreview_ui import Ui_PostReviewDialog
-from tortoisehg.hgqt.hgemail import _ChangesetsModel
+from __future__ import absolute_import
+
+from .qtcore import (
+    QSettings,
+    QThread,
+    QUrl,
+    Qt,
+    pyqtSlot,
+)
+from .qtgui import (
+    QDesktopServices,
+    QDialog,
+    QKeySequence,
+    QLineEdit,
+    QShortcut,
+)
+
+from mercurial import (
+    extensions,
+    pycompat,
+    scmutil,
+)
+
+from ..util import hglib
+from ..util.i18n import _
+from . import (
+    cmdcore,
+    qtlib,
+)
+from .hgemail import _ChangesetsModel
+from .postreview_ui import Ui_PostReviewDialog
 
 class LoadReviewDataThread(QThread):
     def __init__ (self, dialog):
@@ -52,13 +75,13 @@ class LoadReviewDataThread(QThread):
                                                     pwd)
                 self.loadCombos()
 
-            except rb.ReviewBoardError, e:
+            except rb.ReviewBoardError as e:
                 msg = e.msg
             except TypeError:
                 msg = _("Invalid reviewboard plugin. Please download the "
                         "Mercurial reviewboard plugin version 3.5 or higher "
                         "from the website below.\n\n %s") % \
-                        u'http://bitbucket.org/mdelagra/mercurial-reviewboard/'
+                        u'https://foss.heptapod.net/mercurial/tortoisehg/thg-build-deps/mercurial-reviewboard'
 
         self.dialog.error_message = msg
 
@@ -153,31 +176,31 @@ class PostReviewDialog(QDialog):
     def readSettings(self):
         s = QSettings()
 
-        self.restoreGeometry(s.value('reviewboard/geom').toByteArray())
+        self.restoreGeometry(qtlib.readByteArray(s, 'reviewboard/geom'))
 
         self.qui.publish_immediately_check.setChecked(
-                s.value('reviewboard/publish_immediately_check').toBool())
+            qtlib.readBool(s, 'reviewboard/publish_immediately_check'))
         self.qui.outgoing_changes_check.setChecked(
-                s.value('reviewboard/outgoing_changes_check').toBool())
+            qtlib.readBool(s, 'reviewboard/outgoing_changes_check'))
         self.qui.branch_check.setChecked(
-                s.value('reviewboard/branch_check').toBool())
+            qtlib.readBool(s, 'reviewboard/branch_check'))
         self.qui.update_fields.setChecked(
-                s.value('reviewboard/update_fields').toBool())
+            qtlib.readBool(s, 'reviewboard/update_fields'))
         self.qui.summary_edit.addItems(
-                s.value('reviewboard/summary_edit_history').toStringList())
+            qtlib.readStringList(s, 'reviewboard/summary_edit_history'))
 
         try:
-            self.repo_id = int(self.repo.ui.config('reviewboard', 'repoid'))
+            self.repo_id = int(self.repo.ui.config(b'reviewboard', b'repoid'))
         except Exception:
             self.repo_id = None
 
         if not self.repo_id:
-            self.repo_id = s.value('reviewboard/repo_id').toInt()[0]
+            self.repo_id = qtlib.readInt(s, 'reviewboard/repo_id')
 
-        self.server = self.repo.ui.config('reviewboard', 'server')
-        self.user = self.repo.ui.config('reviewboard', 'user')
-        self.password = self.repo.ui.config('reviewboard', 'password')
-        self.browser = self.repo.ui.config('reviewboard', 'browser')
+        self.server = self.repo.ui.config(b'reviewboard', b'server')
+        self.user = self.repo.ui.config(b'reviewboard', b'user')
+        self.password = self.repo.ui.config(b'reviewboard', b'password')
+        self.browser = self.repo.ui.config(b'reviewboard', b'browser')
 
     def writeSettings(self):
         s = QSettings()
@@ -195,7 +218,7 @@ class PostReviewDialog(QDialog):
         def itercombo(w):
             if w.currentText():
                 yield w.currentText()
-            for i in xrange(w.count()):
+            for i in pycompat.xrange(w.count()):
                 if w.itemText(i) != w.currentText():
                     yield w.itemText(i)
 
@@ -254,7 +277,7 @@ class PostReviewDialog(QDialog):
             opts['repoid'] = self.getRepoId()
             opts['summary'] = hglib.fromunicode(self.qui.summary_edit.currentText())
 
-        if (len(self.selectedRevs) > 1):
+        if len(self.selectedRevs) > 1:
             #Set the parent to the revision below the last one on the list
             #so all checked revisions are included in the request
             ctx = self.repo[self.selectedRevs[0]]
@@ -326,12 +349,12 @@ class PostReviewDialog(QDialog):
 
         def cmdargs(opts):
             args = []
-            for k, v in opts.iteritems():
+            for k, v in opts.items():
                 if isinstance(v, bool):
                     if v:
                         args.append('--%s' % k.replace('_', '-'))
                 else:
-                    for e in isinstance(v, basestring) and [v] or v:
+                    for e in hglib.isbasestring(v) and [v] or v:
                         args += ['--%s' % k.replace('_', '-'), e]
 
             return args
@@ -343,8 +366,8 @@ class PostReviewDialog(QDialog):
         self.qui.post_review_button.setEnabled(False)
         self.qui.close_button.setEnabled(False)
 
-        cmdline = map(hglib.tounicode,
-                      ['postreview'] + cmdargs(opts) + [revstr])
+        cmdline = pycompat.maplist(hglib.tounicode,
+                                   ['postreview'] + cmdargs(opts) + [revstr])
         self._cmdsession = sess = self._repoagent.runCommand(cmdline, self)
         del self._cmdoutputs[:]
         sess.commandFinished.connect(self.onCompletion)
@@ -357,14 +380,15 @@ class PostReviewDialog(QDialog):
 
         output = hglib.fromunicode(''.join(self._cmdoutputs), 'replace')
 
-        saved = 'saved:' in output
-        published = 'published:' in output
-        if (saved or published):
+        saved = b'saved:' in output
+        published = b'published:' in output
+        if saved or published:
             if saved:
-                url = output.split('saved: ').pop().strip()
+                url = hglib.tounicode(output.split(b'saved: ').pop().strip())
                 msg = _('Review draft posted to %s\n') % url
             else:
-                url = output.split('published: ').pop().strip()
+                url = output.split(b'published: ').pop().strip()
+                url = hglib.tounicode(url)
                 msg = _('Review published to %s\n') % url
 
             QDesktopServices.openUrl(QUrl(url))
@@ -372,8 +396,8 @@ class PostReviewDialog(QDialog):
             qtlib.InfoMsgBox(_('Review Board'), _('Success'),
                                msg, parent=self)
         else:
-            error = output.split('abort: ').pop().strip()
-            if error[:29] == "HTTP Error: basic auth failed":
+            error = output.split(b'abort: ').pop().strip()
+            if error[:29] == b"HTTP Error: basic auth failed":
                 if self.passwordPrompt():
                     self.accept()
                 else:
@@ -382,7 +406,7 @@ class PostReviewDialog(QDialog):
                     return
             else:
                 qtlib.ErrorMsgBox(_('Review Board'),
-                                  _('Error'), error)
+                                  _('Error'), hglib.tounicode(error))
 
         self.writeSettings()
         super(PostReviewDialog, self).accept()
@@ -390,7 +414,7 @@ class PostReviewDialog(QDialog):
     @pyqtSlot(str, str)
     def _captureOutput(self, msg, label):
         if label != 'control':
-            self._cmdoutputs.append(unicode(msg))
+            self._cmdoutputs.append(pycompat.unicode(msg))
 
     @pyqtSlot()
     def onSettingsButtonClicked(self):
