@@ -27,6 +27,10 @@ from .qtgui import (
     qApp,
 )
 
+from hgext.largefiles import (
+    lfutil,
+)
+
 from mercurial import (
     hg,
     scmutil,
@@ -126,16 +130,15 @@ class PurgeDialog(QDialog):
 
             def run(self):
                 try:
-                    repo.lfstatus = True
-                    stat = repo.status(ignored=True, unknown=True)
-                    repo.lfstatus = False
-                    trashcan = repo.vfs.join('Trashcan')
+                    with lfutil.lfstatus(repo):
+                        stat = repo.status(ignored=True, unknown=True)
+                    trashcan = repo.vfs.join(b'Trashcan')
                     if os.path.isdir(trashcan):
                         trash = os.listdir(trashcan)
                     else:
                         trash = []
-                    self.files = stat[4], stat[5], trash
-                except Exception, e:
+                    self.files = stat.unknown, stat.ignored, trash
+                except Exception as e:
                     self.error = str(e)
 
         self.th = CheckThread(self)
@@ -228,7 +231,7 @@ class PurgeThread(QThread):
     def run(self):
         try:
             self.failures = self.purge(self.root, self.opts)
-        except Exception, e:
+        except Exception as e:
             self.failures = None
             self.showMessage.emit(hglib.tounicode(str(e)))
 
@@ -240,7 +243,7 @@ class PurgeThread(QThread):
 
         if opts['trash']:
             self.showMessage.emit(_('Deleting trash folder...'))
-            trashcan = repo.vfs.join('Trashcan')
+            trashcan = repo.vfs.join(b'Trashcan')
             try:
                 shutil.rmtree(trashcan)
             except EnvironmentError:
@@ -249,14 +252,14 @@ class PurgeThread(QThread):
         self.showMessage.emit('')
         match = scmutil.matchall(repo)
         match.explicitdir = match.traversedir = directories.append
-        repo.lfstatus = True
-        status = repo.status(match=match, ignored=opts['ignored'],
-                             unknown=opts['unknown'], clean=False)
-        repo.lfstatus = False
+        with lfutil.lfstatus(repo):
+            status = repo.status(match=match, ignored=opts['ignored'],
+                                 unknown=opts['unknown'], clean=False)
         files = []
-        for k, i in [('unknown', 4), ('ignored', 5)]:
-            if opts[k]:
-                files.extend(status[i])
+        if opts['unknown']:
+            files.extend(status.unknown)
+        if opts['ignored']:
+            files.extend(status.ignored)
 
         def remove(remove_func, name):
             try:
