@@ -44,8 +44,13 @@ from .qtnetwork import (
 )
 
 from mercurial import (
+    encoding,
     error,
-    util,
+    pycompat,
+)
+from mercurial.utils import (
+    procutil,
+    stringutil,
 )
 
 from ..util import (
@@ -56,6 +61,7 @@ from ..util import (
 from ..util.i18n import _
 from . import (
     bugreport,
+    hgconfig,
     qtlib,
     thgrepo,
     workbench,
@@ -76,15 +82,15 @@ if PYQT_VERSION < 0x40705 or QT_VERSION < 0x40600:
 
 if getattr(sys, 'frozen', False) and os.name == 'nt':
     # load icons and translations
-    from . import icons_rc, translations_rc
+    from . import icons_rc, translations_rc  # pytype: disable=import-error
 
 try:
-    from thginithook import thginithook
+    from thginithook import thginithook  # pytype: disable=import-error
 except ImportError:
     thginithook = None
 
 def _ugetuser():
-    return hglib.tounicode(util.getuser())
+    return hglib.tounicode(procutil.getuser())
 
 # {exception class: message}
 # It doesn't check the hierarchy of exception classes for simplicity.
@@ -101,11 +107,10 @@ _recoverableexc = {
 
 def earlyExceptionMsgBox(e):
     """Show message for recoverable error before the QApplication is started"""
-    opts = {}
-    opts['cmd'] = ' '.join(sys.argv[1:])
-    opts['values'] = e
-    opts['error'] = traceback.format_exc()
-    opts['nofork'] = True
+    opts = {'cmd': ' '.join(sys.argv[1:]),
+            'values': e,
+            'error': traceback.format_exc(),
+            'nofork': True}
     errstring = _recoverableexc[e.__class__]
     if not QApplication.instance():
         main = QApplication(sys.argv)
@@ -114,9 +119,8 @@ def earlyExceptionMsgBox(e):
 
 def earlyBugReport(e):
     """Show generic errors before the QApplication is started"""
-    opts = {}
-    opts['cmd'] = ' '.join(sys.argv[1:])
-    opts['error'] = traceback.format_exc()
+    opts = {'cmd': ' '.join(sys.argv[1:]),
+            'error': traceback.format_exc()}
     if not QApplication.instance():
         main = QApplication(sys.argv)
     dlg = bugreport.BugReport(opts)
@@ -138,27 +142,30 @@ class ExceptionCatcher(QObject):
         self._exceptionOccured.connect(self.putexception,
                                        Qt.QueuedConnection)
 
-        self._ui.debug('setting up excepthook\n')
-        self._origexcepthook = sys.excepthook
-        sys.excepthook = self.ehook
+        self._origexcepthook = None
+        if not self._ui.configbool(b'tortoisehg', b'traceback'):
+            self._ui.debug(b'setting up excepthook\n')
+            self._origexcepthook = sys.excepthook
+            sys.excepthook = self.ehook
+
         self._originthandler = signal.signal(signal.SIGINT, self._inthandler)
         self._initWakeup()
 
     def release(self):
-        if not self._origexcepthook:
-            return
-        self._ui.debug('restoring excepthook\n')
-        sys.excepthook = self._origexcepthook
-        self._origexcepthook = None
-        signal.signal(signal.SIGINT, self._originthandler)
-        self._originthandler = None
-        self._releaseWakeup()
+        if self._origexcepthook:
+            self._ui.debug(b'restoring excepthook\n')
+            sys.excepthook = self._origexcepthook
+            self._origexcepthook = None
+        if self._originthandler:
+            signal.signal(signal.SIGINT, self._originthandler)
+            self._originthandler = None
+            self._releaseWakeup()
 
     def ehook(self, etype, evalue, tracebackobj):
         'Will be called by any thread, on any unhandled exception'
         if self._ui.debugflag:
             elist = traceback.format_exception(etype, evalue, tracebackobj)
-            self._ui.debug(''.join(elist))
+            self._ui.debug(encoding.strtolocal(''.join(elist)))
         self._exceptionOccured.emit(etype, evalue, tracebackobj)
         # not thread-safe to touch self.errors here
 
@@ -183,10 +190,9 @@ class ExceptionCatcher(QObject):
             self.errors = []
 
     def _showexceptiondialog(self):
-        opts = {}
-        opts['cmd'] = ' '.join(sys.argv[1:])
-        opts['error'] = ''.join(''.join(traceback.format_exception(*args))
-                                for args in self.errors)
+        opts = {'cmd': ' '.join(sys.argv[1:]),
+                'error': ''.join(''.join(traceback.format_exception(*args))
+                                 for args in self.errors)}
         etype, evalue = self.errors[0][:2]
         parent = self._mainapp.activeWindow()
         if (len(set(e[0] for e in self.errors)) == 1
@@ -243,8 +249,9 @@ class ExceptionCatcher(QObject):
             rfd = self._wakeupsn.socket()
             try:
                 os.read(rfd, 1)
-            except OSError, inst:
-                self._ui.debug('failed to read wakeup fd: %s\n' % inst)
+            except OSError as inst:
+                self._ui.debug(b'failed to read wakeup fd: %s\n'
+                               % stringutil.forcebytestr(inst))
             self._wakeupsn.setEnabled(True)
 
     else:
@@ -294,20 +301,20 @@ class GarbageCollector(QObject):
         l0, l1, l2 = gc.get_count()
         if l0 > self.threshold[0]:
             num = gc.collect(0)
-            self._ui.debug('GarbageCollector.check: %d %d %d\n' % (l0, l1, l2))
-            self._ui.debug('collected gen 0, found %d unreachable\n' % num)
+            self._ui.debug(b'GarbageCollector.check: %d %d %d\n' % (l0, l1, l2))
+            self._ui.debug(b'collected gen 0, found %d unreachable\n' % num)
             if l1 > self.threshold[1]:
                 num = gc.collect(1)
-                self._ui.debug('collected gen 1, found %d unreachable\n' % num)
+                self._ui.debug(b'collected gen 1, found %d unreachable\n' % num)
                 if l2 > self.threshold[2]:
                     num = gc.collect(2)
-                    self._ui.debug('collected gen 2, found %d unreachable\n'
+                    self._ui.debug(b'collected gen 2, found %d unreachable\n'
                                    % num)
 
     def debug_cycles(self):
         gc.collect()
         for obj in gc.garbage:
-            self._ui.debug('%s, %r, %s\n' % (obj, obj, type(obj)))
+            self._ui.debug(b'%s, %r, %s\n' % (obj, obj, type(obj)))
 
 
 def allowSetForegroundWindow(processid=-1):
@@ -317,7 +324,7 @@ def allowSetForegroundWindow(processid=-1):
         # on windows we must explicitly allow bringing the main window to
         # the foreground. To do so we must use ctypes
         try:
-            from ctypes import windll
+            from ctypes import windll  # pytype: disable=import-error
             windll.user32.AllowSetForegroundWindow(processid)
         except ImportError:
             pass
@@ -334,7 +341,7 @@ def connectToExistingWorkbench(root, revset=None):
     server)
     """
     if revset:
-        data = '\0'.join([root, revset])
+        data = b'\0'.join([root, revset])
     else:
         data = root
     servername = QApplication.applicationName() + '-' + _ugetuser()
@@ -361,7 +368,7 @@ def _fixapplicationfont():
     if os.name != 'nt':
         return
     try:
-        import win32gui, win32con
+        import win32gui, win32con  # pytype: disable=import-error
     except ImportError:
         return
 
@@ -373,7 +380,7 @@ def _fixapplicationfont():
     if lf.lfWeight != win32con.FW_DONTCARE:
         weights = [(0, QFont.Light), (400, QFont.Normal), (600, QFont.DemiBold),
                    (700, QFont.Bold), (800, QFont.Black)]
-        n, w = filter(lambda e: e[0] <= lf.lfWeight, weights)[-1]
+        n, w = [e for e in weights if e[0] <= lf.lfWeight][-1]
         f.setWeight(w)
     f.setPixelSize(abs(lf.lfHeight))
     QApplication.setFont(f, 'QWidget')
@@ -396,6 +403,7 @@ class QtRunner(QObject):
     def __init__(self):
         super(QtRunner, self).__init__()
         self._ui = None
+        self._config = None
         self._mainapp = None
         self._exccatcher = None
         self._server = None
@@ -422,6 +430,7 @@ class QtRunner(QObject):
                 QFont.insertSubstitution('.Lucida Grande UI', 'Lucida Grande')
 
         self._ui = ui
+        self._config = hgconfig.HgConfig(ui)
         self._mainapp = QApplication(sys.argv)
 
         if QT_VERSION >= 0x50000:
@@ -447,7 +456,7 @@ class QtRunner(QObject):
 
         self._repomanager = thgrepo.RepoManager(ui, self)
         self._reporeleaser = releaser = QSignalMapper(self)
-        releaser.mapped[unicode].connect(self._repomanager.releaseRepoAgent)
+        releaser.mapped[pycompat.unicode].connect(self._repomanager.releaseRepoAgent)
 
         # stop services after control returns to the main event loop
         self._mainapp.setQuitOnLastWindowClosed(False)
@@ -472,7 +481,7 @@ class QtRunner(QObject):
             return self._mainapp.exec_()
         finally:
             self._exccatcher.release()
-            self._mainapp = self._ui = None
+            self._mainapp = self._ui = self._config = None
 
     @pyqtSlot()
     def _quitGracefully(self):
@@ -484,7 +493,7 @@ class QtRunner(QObject):
             self._server.close()
         if self._tryQuit():
             return
-        self._ui.debug('repositories are closing asynchronously\n')
+        self._ui.debug(b'repositories are closing asynchronously\n')
         self._repomanager.repositoryClosed.connect(self._tryQuit)
         QTimer.singleShot(5000, self._mainapp.quit)  # in case of bug
 
@@ -519,12 +528,12 @@ class QtRunner(QObject):
                 reporoot = repoagent.rootPath()
                 args.insert(0, repoagent)
             return dlgfunc(self._ui, *args, **opts), reporoot
-        except error.RepoError, inst:
+        except error.RepoError as inst:
             qtlib.WarningMsgBox(_('Repository Error'),
-                                hglib.tounicode(str(inst)))
-        except error.Abort, inst:
+                                hglib.tounicode(stringutil.forcebytestr(inst)))
+        except error.Abort as inst:
             qtlib.WarningMsgBox(_('Abort'),
-                                hglib.tounicode(str(inst)),
+                                hglib.tounicode(stringutil.forcebytestr(inst)),
                                 hglib.tounicode(inst.hint or ''))
         if reporoot:
             self._repomanager.releaseRepoAgent(reporoot)
@@ -548,17 +557,18 @@ class QtRunner(QObject):
 
     def createWorkbench(self):
         """Create Workbench window and keep single reference"""
-        assert self._ui and self._mainapp and self._repomanager
+        assert self._ui and self._config and self._mainapp and self._repomanager
         assert not self._workbench
-        self._workbench = workbench.Workbench(self._ui, self._repomanager)
+        self._workbench = workbench.Workbench(
+            self._ui, self._config, self._repomanager)
         return self._workbench
 
     @pyqtSlot(str)
     def openRepoInWorkbench(self, uroot):
         """Show the specified repository in Workbench; reuses the existing
         Workbench process"""
-        assert self._ui
-        singlewb = self._ui.configbool('tortoisehg', 'workbench.single', True)
+        assert self._config
+        singlewb = self._config.configBool('tortoisehg', 'workbench.single')
         # only if the server is another process; otherwise it would deadlock
         if (singlewb and not self._server
             and connectToExistingWorkbench(hglib.fromunicode(uroot))):
@@ -592,9 +602,9 @@ class QtRunner(QObject):
         socket = self._server.nextPendingConnection()
         if socket:
             socket.waitForReadyRead(10000)
-            data = str(socket.readAll())
-            if data and data != '[echo]':
-                args = data.split('\0', 1)
+            data = bytes(socket.readAll())
+            if data and data != b'[echo]':
+                args = data.split(b'\0', 1)
                 if len(args) > 1:
                     uroot, urevset = map(hglib.tounicode, args)
                 else:
