@@ -27,8 +27,13 @@ from .qtgui import (
     QVBoxLayout,
 )
 
+from hgext.largefiles import (
+    lfutil,
+)
+
 from mercurial import (
     error,
+    pycompat,
     util,
 )
 
@@ -106,7 +111,7 @@ class QuickOpDialog(QDialog):
         defcheck = checktypes[self.command]
 
         opts = {}
-        for s, val in status.statusTypes.iteritems():
+        for s, val in status.statusTypes.items():
             opts[val.name] = s in filetypes
 
         opts['checkall'] = True # pre-check all matching files
@@ -142,7 +147,7 @@ class QuickOpDialog(QDialog):
         self.bb = bb
 
         if self.command == 'add':
-            if 'largefiles' in self.repo.extensions():
+            if b'largefiles' in self.repo.extensions():
                 self.addLfilesButton = QPushButton(_('Add &Largefiles'))
             else:
                 self.addLfilesButton = None
@@ -179,13 +184,13 @@ class QuickOpDialog(QDialog):
         if files:
             cmdlines.append(hglib.buildcmdargs(self.command, *files, **opts))
         if lfiles:
-            assert self.command == 'add'
+            assert self.command == 'add', self.command
             lopts = opts.copy()
             lopts['large'] = True
             cmdlines.append(hglib.buildcmdargs(self.command, *lfiles, **lopts))
         self.files = files + lfiles
 
-        ucmdlines = [map(hglib.tounicode, xs) for xs in cmdlines]
+        ucmdlines = [pycompat.maplist(hglib.tounicode, xs) for xs in cmdlines]
         self._cmdsession = sess = self._repoagent.runCommandSequence(ucmdlines,
                                                                      self)
         sess.commandFinished.connect(self.commandFinished)
@@ -216,20 +221,18 @@ class QuickOpDialog(QDialog):
                                 parent=self)
             return
         if self.command == 'remove':
-            self.repo.lfstatus = True
-            try:
-                repostate = self.repo.status()
-            except (EnvironmentError, error.Abort), e:
-                qtlib.WarningMsgBox(_('Unable to read repository status'),
-                                    hglib.tounicode(str(e)), parent=self)
-                return
-            finally:
-                self.repo.lfstatus = False
+            with lfutil.lfstatus(self.repo):
+                try:
+                    repostate = self.repo.status()
+                except (EnvironmentError, error.Abort) as e:
+                    qtlib.WarningMsgBox(_('Unable to read repository status'),
+                                        hglib.tounicode(str(e)), parent=self)
+                    return
+
             if not self.chk.isChecked():
-                modified = repostate[0]
                 selmodified = []
                 for wfile in files:
-                    if wfile in modified:
+                    if wfile in repostate.modified:
                         selmodified.append(wfile)
                 if selmodified:
                     prompt = qtlib.CustomPrompt(
@@ -247,16 +250,15 @@ class QuickOpDialog(QDialog):
                         cmdopts['force'] = True
                     elif ret == 2:
                         return
-            unknown, ignored = repostate[4:6]
             for wfile in files:
-                if wfile in unknown or wfile in ignored:
+                if wfile in repostate.unknown or wfile in repostate.ignored:
                     try:
                         util.unlink(wfile)
                     except EnvironmentError:
                         pass
                     files.remove(wfile)
         elif self.command == 'add':
-            if 'largefiles' in self.repo.extensions():
+            if b'largefiles' in self.repo.extensions():
                 self.addWithPrompt(files)
                 return
         if files:
@@ -301,7 +303,7 @@ class HeadlessQuickop(QObject):
         QObject.__init__(self)
         self.files = cmdline[1:]
         self._cmddialog = cmdui.CmdSessionDialog()
-        sess = repoagent.runCommand(map(hglib.tounicode, cmdline))
+        sess = repoagent.runCommand(pycompat.maplist(hglib.tounicode, cmdline))
         sess.commandFinished.connect(self.commandFinished)
         self._cmddialog.setSession(sess)
 
@@ -322,7 +324,7 @@ def run(ui, repoagent, *pats, **opts):
     repo = repoagent.rawRepo()
     pats = hglib.canonpaths(pats)
     command = opts['alias']
-    imm = repo.ui.config('tortoisehg', 'immediate', '')
+    imm = repo.ui.config(b'tortoisehg', b'immediate')
     if opts.get('headless') or command in imm.lower():
         cmdline = [command] + pats
         return HeadlessQuickop(repoagent, cmdline)
