@@ -5,18 +5,49 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2, incorporated herein by reference.
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui import *
+from __future__ import absolute_import
 
 import os
 
-from tortoisehg.util import hglib
-from tortoisehg.util.i18n import _
-from tortoisehg.hgqt import qtlib, csinfo, resolve, thgrepo, wctxcleaner
-from tortoisehg.hgqt import cmdcore, cmdui
+from .qtcore import (
+    QSettings,
+    QTimer,
+    Qt,
+    pyqtSlot,
+)
+from .qtgui import (
+    QButtonGroup,
+    QCheckBox,
+    QDialog,
+    QDialogButtonBox,
+    QGroupBox,
+    QLabel,
+    QMessageBox,
+    QRadioButton,
+    QVBoxLayout,
+)
+
+from ..util import hglib
+from ..util.i18n import _
+from . import (
+    cmdcore,
+    cmdui,
+    csinfo,
+    qtlib,
+    resolve,
+    thgrepo,
+    wctxcleaner,
+)
 
 BB = QDialogButtonBox
 
+_SOURCE_SPECS = [
+    # name, title, option label
+    ('source', _('Rebase source and descendants'), '-s/--source'),
+    ('base', _('Rebase entire source branch'), '-b/--base'),
+    ('rev', _('Rebase exact revision'), '-r/--rev'),
+]
+
 class RebaseDialog(QDialog):
 
     def __init__(self, repoagent, parent, **opts):
@@ -36,14 +67,14 @@ class RebaseDialog(QDialog):
 
         style = csinfo.panelstyle(selectable=True)
 
-        srcb = QGroupBox(_('Rebase changeset and descendants'))
+        self._sourcebox = srcb = QGroupBox(self)
         srcb.setLayout(QVBoxLayout())
         srcb.layout().setContentsMargins(*(2,)*4)
         s = opts.get('source', '.')
         source = csinfo.create(self.repo, s, style, withupdate=True)
         srcb.layout().addWidget(source)
         self.sourcecsinfo = source
-        self.layout().addWidget(srcb)
+        box.addWidget(srcb)
 
         destb = QGroupBox(_('To rebase destination'))
         destb.setLayout(QVBoxLayout())
@@ -52,49 +83,54 @@ class RebaseDialog(QDialog):
         dest = csinfo.create(self.repo, d, style, withupdate=True)
         destb.layout().addWidget(dest)
         self.destcsinfo = dest
-        self.layout().addWidget(destb)
+        box.addWidget(destb)
 
         self.swaplabel = QLabel('<a href="X">%s</a>'  # don't care href
                                 % _('Swap source and destination'))
         self.swaplabel.linkActivated.connect(self.swap)
-        self.layout().addWidget(self.swaplabel)
+        box.addWidget(self.swaplabel)
 
         sep = qtlib.LabeledSeparator(_('Options'))
-        self.layout().addWidget(sep)
+        box.addWidget(sep)
+
+        self._sourcegroup = QButtonGroup(self)
+        self._sourcegroup.buttonClicked.connect(self._updateSourceSelector)
+        for i, (name, title, oplabel) in enumerate(_SOURCE_SPECS):
+            w = QRadioButton('%s (%s)' % (title, oplabel), self)
+            w.setChecked(name == 'source')
+            self._sourcegroup.addButton(w, i)
+            box.addWidget(w)
 
         self.keepchk = QCheckBox(_('Keep original changesets (--keep)'))
         self.keepchk.setChecked(opts.get('keep', False))
-        self.layout().addWidget(self.keepchk)
+        box.addWidget(self.keepchk)
 
         self.keepbrancheschk = QCheckBox(_('Keep original branch names '
                                            '(--keepbranches)'))
         self.keepbrancheschk.setChecked(opts.get('keepbranches', False))
-        self.layout().addWidget(self.keepbrancheschk)
+        box.addWidget(self.keepbrancheschk)
 
         self.collapsechk = QCheckBox(_('Collapse the rebased changesets '
                                        '(--collapse)'))
         self.collapsechk.setChecked(opts.get('collapse', False))
-        self.layout().addWidget(self.collapsechk)
-
-        self.basechk = QCheckBox(_('Rebase entire source branch (-b/--base)'))
-        self.layout().addWidget(self.basechk)
+        box.addWidget(self.collapsechk)
 
         self.autoresolvechk = QCheckBox(_('Automatically resolve merge '
                                           'conflicts where possible'))
-        self.layout().addWidget(self.autoresolvechk)
+        box.addWidget(self.autoresolvechk)
 
         self.svnchk = QCheckBox(_('Rebase unpublished onto Subversion head '
                                   '(override source, destination)'))
-        self.svnchk.setVisible('hgsubversion' in repo.extensions())
-        self.layout().addWidget(self.svnchk)
+        self.svnchk.setVisible(b'hgsubversion' in repo.extensions())
+        box.addWidget(self.svnchk)
 
         self._cmdlog = cmdui.LogWidget(self)
         self._cmdlog.hide()
-        self.layout().addWidget(self._cmdlog, 2)
+        box.addWidget(self._cmdlog, 2)
         self._stbar = cmdui.ThgStatusBar(self)
         self._stbar.setSizeGripEnabled(False)
         self._stbar.linkActivated.connect(self.linkActivated)
-        self.layout().addWidget(self._stbar)
+        box.addWidget(self._stbar)
 
         bbox = QDialogButtonBox()
         self.cancelbtn = bbox.addButton(QDialogButtonBox.Cancel)
@@ -105,7 +141,7 @@ class RebaseDialog(QDialog):
         self.abortbtn = bbox.addButton(_('Abort'),
                                        QDialogButtonBox.ActionRole)
         self.abortbtn.clicked.connect(self.abort)
-        self.layout().addWidget(bbox)
+        box.addWidget(bbox)
         self.bbox = bbox
 
         self._wctxcleaner = wctxcleaner.WctxCleaner(repoagent, self)
@@ -125,6 +161,7 @@ class RebaseDialog(QDialog):
         self.setMaximumHeight(800)
         self.resize(0, 340)
         self.setWindowTitle(_('Rebase - %s') % repoagent.displayName())
+        self._updateSourceSelector()
         self._readSettings()
 
     @property
@@ -135,8 +172,8 @@ class RebaseDialog(QDialog):
         qs = QSettings()
         qs.beginGroup('rebase')
         self.autoresolvechk.setChecked(
-            self.repo.ui.configbool('tortoisehg', 'autoresolve',
-                                    qs.value('autoresolve', True).toBool()))
+            self._repoagent.configBool('tortoisehg', 'autoresolve',
+                                       qtlib.readBool(qs, 'autoresolve', True)))
         qs.endGroup()
 
     def _writeSettings(self):
@@ -158,18 +195,24 @@ class RebaseDialog(QDialog):
             txt = _('You may continue the rebase')
         self._stbar.showMessage(txt)
 
+    @pyqtSlot()
+    def _updateSourceSelector(self):
+        _name, title, _oplabel = _SOURCE_SPECS[self._sourcegroup.checkedId()]
+        self._sourcebox.setTitle(title)
+
     def rebase(self):
         self.rebasebtn.setEnabled(False)
         self.cancelbtn.setVisible(False)
         self.keepchk.setEnabled(False)
         self.keepbrancheschk.setEnabled(False)
-        self.basechk.setEnabled(False)
+        for w in self._sourcegroup.buttons():
+            w.setEnabled(False)
         self.collapsechk.setEnabled(False)
         self.swaplabel.setVisible(False)
 
         itool = self.autoresolvechk.isChecked() and 'merge' or 'fail'
         opts = {'config': 'ui.merge=internal:%s' % itool}
-        if os.path.exists(self.repo.join('rebasestate')):
+        if os.path.exists(self.repo.vfs.join(b'rebasestate')):
             opts['continue'] = True
         else:
             opts.update({
@@ -180,9 +223,7 @@ class RebaseDialog(QDialog):
             if self.svnchk.isChecked():
                 opts['svn'] = True
             else:
-                sourcearg = 'source'
-                if self.basechk.isChecked():
-                    sourcearg = 'base'
+                sourcearg = _SOURCE_SPECS[self._sourcegroup.checkedId()][0]
                 opts[sourcearg] = hglib.tounicode(str(self.opts.get('source')))
                 opts['dest'] = hglib.tounicode(str(self.opts.get('dest')))
         cmdline = hglib.buildcmdargs('rebase', **opts)
@@ -239,7 +280,7 @@ class RebaseDialog(QDialog):
 
     def checkResolve(self):
         for root, path, status in thgrepo.recursiveMergeStatus(self.repo):
-            if status == 'u':
+            if status == b'u':
                 txt = _('Rebase generated merge <b>conflicts</b> that must '
                         'be <a href="resolve"><b>resolved</b></a>')
                 self.rebasebtn.setEnabled(False)
@@ -249,7 +290,7 @@ class RebaseDialog(QDialog):
             txt = _('You may continue the rebase')
         self._stbar.showMessage(txt)
 
-        if os.path.exists(self.repo.join('rebasestate')):
+        if os.path.exists(self.repo.vfs.join(b'rebasestate')):
             self.swaplabel.setVisible(False)
             self.abortbtn.setEnabled(True)
             self.rebasebtn.setText('Continue')
@@ -267,7 +308,7 @@ class RebaseDialog(QDialog):
             self._wctxcleaner.runCleaner(cmd)
 
     def reject(self):
-        if os.path.exists(self.repo.join('rebasestate')):
+        if os.path.exists(self.repo.vfs.join(b'rebasestate')):
             main = _('Exiting with an unfinished rebase is not recommended.')
             text = _('Consider aborting the rebase first.')
             labels = ((QMessageBox.Yes, _('&Exit')),
