@@ -6,16 +6,39 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2, incorporated herein by reference.
 
-import os, sys
+from __future__ import absolute_import
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui import *
+import os
+import sys
 
-from mercurial import util
+from .qtcore import (
+    pyqtSlot,
+)
+from .qtgui import (
+    QCheckBox,
+    QFileDialog,
+    QFormLayout,
+    QHBoxLayout,
+    QLabel,
+    QLineEdit,
+    QMessageBox,
+    QPushButton,
+    QSizePolicy,
+)
 
-from tortoisehg.hgqt import cmdcore, cmdui, qtlib, manifestmodel
-from tortoisehg.util import hglib
-from tortoisehg.util.i18n import _
+from mercurial import (
+    pycompat,
+    util,
+)
+
+from ..util import hglib
+from ..util.i18n import _
+from . import (
+    cmdcore,
+    cmdui,
+    manifestmodel,
+    qtlib,
+)
 
 class RenameWidget(cmdui.AbstractCmdWidget):
 
@@ -84,10 +107,10 @@ class RenameWidget(cmdui.AbstractCmdWidget):
         return self._repoagent.rawRepo()
 
     def source(self):
-        return unicode(self.src_txt.text())
+        return pycompat.unicode(self.src_txt.text())
 
     def destination(self):
-        return unicode(self.dest_txt.text())
+        return pycompat.unicode(self.dest_txt.text())
 
     def _sourceFile(self):
         root = self._repoagent.rootPath()
@@ -106,7 +129,8 @@ class RenameWidget(cmdui.AbstractCmdWidget):
         FD = QFileDialog
         if os.path.isfile(self._sourceFile()):
             caption = _('Select Source File')
-            path = FD.getOpenFileName(self, caption, '', '', None, FD.ReadOnly)
+            path, _filter = FD.getOpenFileName(self, caption, '', '', '',
+                                               FD.ReadOnly)
         else:
             caption = _('Select Source Folder')
             path = FD.getExistingDirectory(self, caption, '',
@@ -123,7 +147,7 @@ class RenameWidget(cmdui.AbstractCmdWidget):
             caption = _('Select Destination File')
         else:
             caption = _('Select Destination Folder')
-        path = FD.getSaveFileName(self, caption)
+        path, _filter = FD.getSaveFileName(self, caption)
         relpath = self.to_relative_path(path)
         if not relpath:
             return
@@ -132,7 +156,7 @@ class RenameWidget(cmdui.AbstractCmdWidget):
     def to_relative_path(self, fullpath):  # unicode or QString
         if not fullpath:
             return
-        fullpath = util.normpath(unicode(fullpath))
+        fullpath = util.normpath(pycompat.unicode(fullpath))
         pathprefix = util.normpath(hglib.tounicode(self.repo.root)) + '/'
         if not os.path.normcase(fullpath).startswith(os.path.normcase(pathprefix)):
             return
@@ -148,7 +172,7 @@ class RenameWidget(cmdui.AbstractCmdWidget):
 
     def isCaseFoldingOnWin(self):
         fullsrc, fulldest = self._sourceFile(), self._destinationFile()
-        return (fullsrc.upper() == fulldest.upper() and sys.platform == 'win32')
+        return fullsrc.upper() == fulldest.upper() and sys.platform == 'win32'
 
     def compose_command(self):
         name = self.isCopyCommand() and 'copy' or 'rename'
