@@ -5,7 +5,7 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function
 
 import os
 
@@ -26,6 +26,10 @@ from .qtgui import (
     QFont,
 )
 
+from mercurial import (
+    pycompat,
+)
+
 from ..util import hglib
 from ..util.i18n import _
 from . import repotreeitem
@@ -53,16 +57,16 @@ def readXml(source, rootElementName):
     if xr.readNextStartElement():
         ele = str(xr.name())
         if ele != rootElementName:
-            print "unexpected xml element '%s' "\
-                  "(was looking for %s)" % (ele, rootElementName)
+            print("unexpected xml element '%s' "
+                  "(was looking for %s)" % (ele, rootElementName))
             return
     if xr.hasError():
-        print hglib.fromunicode(xr.errorString(), 'replace')
+        print(hglib.fromunicode(xr.errorString(), 'replace'))
     if xr.readNextStartElement():
         itemread = repotreeitem.undumpObject(xr)
         xr.skipCurrentElement()
     if xr.hasError():
-        print hglib.fromunicode(xr.errorString(), 'replace')
+        print(hglib.fromunicode(xr.errorString(), 'replace'))
     return itemread
 
 def iterRepoItemFromXml(source):
@@ -72,7 +76,9 @@ def iterRepoItemFromXml(source):
         t = xr.readNext()
         if (t == QXmlStreamReader.StartElement
             and xr.name() in ('repo', 'subrepo')):
-            yield repotreeitem.undumpObject(xr)
+            rti = repotreeitem.undumpObject(xr)
+            assert isinstance(rti, repotreeitem.RepoItem)
+            yield rti
 
 def getRepoItemList(root, standalone=False):
     if standalone:
@@ -124,7 +130,7 @@ class RepoTreeModel(QAbstractItemModel):
     def index(self, row, column, parent=QModelIndex()):
         if not self.hasIndex(row, column, parent):
             return QModelIndex()
-        if (not parent.isValid()):
+        if not parent.isValid():
             parentItem = self.rootItem
         else:
             parentItem = parent.internalPointer()
@@ -219,7 +225,7 @@ class RepoTreeModel(QAbstractItemModel):
 
     def dropMimeData(self, data, action, row, column, parent):
         group = parent.internalPointer()
-        d = str(data.data(repoRegMimeType))
+        d = bytes(data.data(repoRegMimeType))
         if not data.hasUrls():
             # The source is a group
             if row < 0:
@@ -338,7 +344,8 @@ class RepoTreeModel(QAbstractItemModel):
     def indexFromItemPath(self, path, column=0):
         """Model index for the item specified by the given virtual path"""
         try:
-            item = repotreeitem.findbyitempath(self.rootItem, unicode(path))
+            item = repotreeitem.findbyitempath(self.rootItem,
+                                               pycompat.unicode(path))
         except ValueError:
             return QModelIndex()
         return self._indexFromItem(item, column)
@@ -390,7 +397,7 @@ class RepoTreeModel(QAbstractItemModel):
             item._sharedpath = tmpitem._sharedpath
             item._valid = tmpitem._valid
             self._emitItemDataChanged(item)
-        return map(hglib.tounicode, invalidpaths)
+        return pycompat.maplist(hglib.tounicode, invalidpaths)
 
     def updateCommonPaths(self, showShortPaths=None):
         if showShortPaths is not None:
@@ -404,7 +411,7 @@ class RepoTreeModel(QAbstractItemModel):
 
     @pyqtSlot(str)
     def _updateShortName(self, uroot):
-        uroot = unicode(uroot)
+        uroot = pycompat.unicode(uroot)
         repoagent = self._repomanager.repoAgent(uroot)
         it = self.getRepoItem(uroot)
         if it:
@@ -413,7 +420,7 @@ class RepoTreeModel(QAbstractItemModel):
 
     @pyqtSlot(str)
     def _updateBaseNode(self, uroot):
-        uroot = unicode(uroot)
+        uroot = pycompat.unicode(uroot)
         repo = self._repomanager.repoAgent(uroot).rawRepo()
         it = self.getRepoItem(uroot)
         if it:
