@@ -36,6 +36,11 @@ from .qtgui import (
     QVBoxLayout,
 )
 
+from mercurial import (
+    encoding,
+    pycompat,
+)
+
 from ..util import hglib
 from ..util.i18n import _
 from . import (
@@ -249,8 +254,10 @@ class ResolveDialog(QDialog):
         paths = []
         if not tree.selectionModel():
             return paths
+        model = tree.model()
+        assert model is not None
         for idx in tree.selectionModel().selectedRows():
-            root, wfile = tree.model().getPathForIndex(idx)
+            root, wfile = model.getPathForIndex(idx)
             paths.append((root, wfile))
         return paths
 
@@ -263,7 +270,7 @@ class ResolveDialog(QDialog):
             for root, wfile in selected:
                 if root == curroot:
                     cmd.append(os.path.normpath(os.path.join(root, wfile)))
-            cmdlines.append(map(hglib.tounicode, cmd))
+            cmdlines.append(pycompat.maplist(hglib.tounicode, cmd))
             selected = [(r, w) for r, w in selected if r != curroot]
         if cmdlines:
             sess = self._repoagent.runCommandSequence(cmdlines, self)
@@ -275,9 +282,11 @@ class ResolveDialog(QDialog):
     def merge(self, tool=False):
         if not tool:
             tool = self.tcombo.readValue()
+            if tool:
+                tool = encoding.strfromlocal(tool)
         cmd = ['resolve']
         if tool:
-            cmd += ['--tool='+tool]
+            cmd += ['--tool=' + tool]
         self.runCommand(self.utree, cmd)
 
     @pyqtSlot(QAction)
@@ -315,9 +324,8 @@ class ResolveDialog(QDialog):
     def v3way(self):
         paths = self.getVdiffFiles(self.rtree)
         if paths:
-            opts = {}
-            opts['rev'] = []
-            opts['tool'] = self.tcombo.readValue()
+            opts = {'rev': [],
+                    'tool': self.tcombo.readValue()}
             dlg = visdiff.visualdiff(self.repo.ui, self.repo, paths, opts)
             if dlg:
                 dlg.exec_()
@@ -325,9 +333,8 @@ class ResolveDialog(QDialog):
     def vp0(self):
         paths = self.getVdiffFiles(self.rtree)
         if paths:
-            opts = {}
-            opts['rev'] = ['p1()']
-            opts['tool'] = self.tcombo.readValue()
+            opts = {'rev': ['p1()'],
+                    'tool': self.tcombo.readValue()}
             dlg = visdiff.visualdiff(self.repo.ui, self.repo, paths, opts)
             if dlg:
                 dlg.exec_()
@@ -335,9 +342,8 @@ class ResolveDialog(QDialog):
     def vp1(self):
         paths = self.getVdiffFiles(self.rtree)
         if paths:
-            opts = {}
-            opts['rev'] = ['p2()']
-            opts['tool'] = self.tcombo.readValue()
+            opts = {'rev': ['p2()'],
+                    'tool': self.tcombo.readValue()}
             dlg = visdiff.visualdiff(self.repo.ui, self.repo, paths, opts)
             if dlg:
                 dlg.exec_()
@@ -345,9 +351,8 @@ class ResolveDialog(QDialog):
     def diffLocToAnc(self):
         paths = self.getVdiffFiles(self.utree)
         if paths:
-            opts = {}
-            opts['rev'] = ['ancestor(p1(),p2())..p1()']
-            opts['tool'] = self.tcombo.readValue()
+            opts = {'rev': ['ancestor(p1(),p2())..p1()'],
+                    'tool': self.tcombo.readValue()}
             dlg = visdiff.visualdiff(self.repo.ui, self.repo, paths, opts)
             if dlg:
                 dlg.exec_()
@@ -355,9 +360,8 @@ class ResolveDialog(QDialog):
     def diffOthToAnc(self):
         paths = self.getVdiffFiles(self.utree)
         if paths:
-            opts = {}
-            opts['rev'] = ['ancestor(p1(),p2())..p2()']
-            opts['tool'] = self.tcombo.readValue()
+            opts = {'rev': ['ancestor(p1(),p2())..p2()'],
+                    'tool': self.tcombo.readValue()}
             dlg = visdiff.visualdiff(self.repo.ui, self.repo, paths, opts)
             if dlg:
                 dlg.exec_()
@@ -366,7 +370,7 @@ class ResolveDialog(QDialog):
     def refresh(self):
         u, r = [], []
         for root, path, status in thgrepo.recursiveMergeStatus(self.repo):
-            if status == 'u':
+            if status == b'u':
                 u.append((root, path))
             else:
                 r.append((root, path))
@@ -380,6 +384,7 @@ class ResolveDialog(QDialog):
 
         model = self.utree.model()
         smodel = self.utree.selectionModel()
+        assert model is not None
         sflags = QItemSelectionModel.Select | QItemSelectionModel.Rows
         for i, path in enumerate(u):
             if path in paths:
@@ -398,6 +403,7 @@ class ResolveDialog(QDialog):
 
         model = self.rtree.model()
         smodel = self.rtree.selectionModel()
+        assert model is not None
         for i, path in enumerate(r):
             if path in paths:
                 smodel.select(model.index(i, 0), sflags)
@@ -416,7 +422,9 @@ class ResolveDialog(QDialog):
     def reject(self):
         s = QSettings()
         s.setValue('resolve/geom', self.saveGeometry())
-        if self.utree.model().rowCount() > 0:
+        model = self.utree.model()
+        assert model is not None
+        if model.rowCount() > 0:
             main = _('Exit without finishing resolve?')
             text = _('Unresolved conflicts remain. Are you sure?')
             labels = ((QMessageBox.Yes, _('E&xit')),
@@ -456,7 +464,7 @@ class ResolveDialog(QDialog):
         self.rtreecmenu.popup(self.rtree.viewport().mapToGlobal(point))
 
     def utreeDoubleClicked(self):
-        if self.repo.ui.configbool('tortoisehg', 'autoresolve', True):
+        if self._repoagent.configBool('tortoisehg', 'autoresolve', True):
             self.merge()
         else:
             self.merge('internal:merge')
