@@ -5,38 +5,62 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2, incorporated herein by reference.
 
-import os, tempfile
-from PyQt4.QtCore import *
-from PyQt4.QtGui import *
-from mercurial import util, error
-from tortoisehg.util import paths, wconfig, hglib
-from tortoisehg.hgqt import cmdcore, cmdui, qtlib
-from tortoisehg.hgqt.i18n import _
-from tortoisehg.hgqt.serve_ui import Ui_ServeDialog
-from tortoisehg.hgqt.webconf import WebconfForm
+from __future__ import absolute_import
+
+import os
+import tempfile
+
+from .qtcore import (
+    Qt,
+    pyqtSlot,
+)
+from .qtgui import (
+    QDialog,
+    QSystemTrayIcon,
+)
+
+from mercurial import (
+    error,
+    pycompat,
+    util,
+)
+
+from ..util import (
+    hglib,
+    paths,
+    wconfig,
+)
+from ..util.i18n import _
+from . import (
+    cmdcore,
+    cmdui,
+    qtlib,
+)
+from .serve_ui import Ui_ServeDialog
+from .webconf import WebconfForm
 
 class ServeDialog(QDialog):
     """Dialog for serving repositories via web"""
-    def __init__(self, webconf, parent=None):
+    def __init__(self, ui, webconf, parent=None):
         super(ServeDialog, self).__init__(parent)
         self.setWindowFlags((self.windowFlags() | Qt.WindowMinimizeButtonHint)
                             & ~Qt.WindowContextHelpButtonHint)
-        # TODO: choose appropriate icon
-        self.setWindowIcon(qtlib.geticon('proxy'))
+        self.setWindowIcon(qtlib.geticon('hg-serve'))
 
         self._qui = Ui_ServeDialog()
         self._qui.setupUi(self)
 
         self._initwebconf(webconf)
-        self._initcmd()
+        self._initcmd(ui)
         self._initactions()
         self._updateform()
 
-    def _initcmd(self):
+    def _initcmd(self, ui):
         # TODO: forget old logs?
         self._log_edit = cmdui.LogWidget(self)
         self._qui.details_tabs.addTab(self._log_edit, _('Log'))
-        self._agent = cmdcore.CmdAgent(self)
+        # as of hg 3.0, hgweb does not cooperate with command-server channel
+        self._agent = cmdcore.CmdAgent(ui, self, worker='proc')
         self._agent.outputReceived.connect(self._log_edit.appendLog)
         self._agent.busyChanged.connect(self._updateform)
 
@@ -74,12 +98,12 @@ class ServeDialog(QDialog):
         if self.isstarted():
             return
 
-        self._agent.runCommand(map(hglib.tounicode, self._cmdargs()),
-                               worker='proc')
+        self._agent.runCommand(pycompat.maplist(hglib.tounicode,
+                                                self._cmdargs()))
 
     def _cmdargs(self):
         """Build command args to run server"""
-        a = ['serve', '--port', str(self.port), '--debug']
+        a = ['serve', '--port', str(self.port), '-v']
         if self._singlerepo:
             a += ['-R', self._singlerepo]
         else:
@@ -91,7 +115,8 @@ class ServeDialog(QDialog):
         if not hasattr(self._webconf, 'write'):
             return self._webconf.path
 
-        fd, fname = tempfile.mkstemp(prefix='webconf_', dir=qtlib.gettempdir())
+        fd, fname = tempfile.mkstemp(prefix=b'webconf_',
+                                     dir=qtlib.gettempdir())
         f = os.fdopen(fd, 'w')
         try:
             self._webconf.write(f)
@@ -186,18 +211,18 @@ class ServeDialog(QDialog):
 
 def _asconfigliststr(value):
     r"""
-    >>> _asconfigliststr('foo')
-    'foo'
-    >>> _asconfigliststr('foo bar')
-    '"foo bar"'
-    >>> _asconfigliststr('foo,bar')
-    '"foo,bar"'
-    >>> _asconfigliststr('foo "bar"')
-    '"foo \\"bar\\""'
+    >>> _asconfigliststr(b'foo')
+    b'foo'
+    >>> _asconfigliststr(b'foo bar')
+    b'"foo bar"'
+    >>> _asconfigliststr(b'foo,bar')
+    b'"foo,bar"'
+    >>> _asconfigliststr(b'foo "bar"')
+    b'"foo \\"bar\\""'
     """
     # ui.configlist() uses isspace(), which is locale-dependent
-    if util.any(c.isspace() or c == ',' for c in value):
-        return '"' + value.replace('"', '\\"') + '"'
+    if any(c.isspace() or c == b',' for c in pycompat.iterbytestr(value)):
+        return b'"' + value.replace(b'"', b'\\"') + b'"'
     else:
         return value
 
@@ -211,20 +236,20 @@ def _readconfig(ui, repopath, webconfpath):
         c.path = os.path.abspath(webconfpath)
         return lui, c
     elif repopath:  # imitate webconf for single repo
-        lui.readconfig(os.path.join(repopath, '.hg', 'hgrc'), repopath)
+        lui.readconfig(os.path.join(repopath, b'.hg', b'hgrc'), repopath)
         c = wconfig.config()
         try:
-            if not os.path.exists(os.path.join(repopath, '.hgsub')):
+            if not os.path.exists(os.path.join(repopath, b'.hgsub')):
                 # no _asconfigliststr(repopath) for now, because ServeDialog
                 # cannot parse it as a list in single-repo mode.
-                c.set('paths', '/', repopath)
+                c.set(b'paths', b'/', repopath)
             else:
                 # since hg 8cbb59124e67, path entry is parsed as a list
-                base = lui.config('web', 'name') or os.path.basename(repopath)
-                c.set('paths', base,
-                      _asconfigliststr(os.path.join(repopath, '**')))
+                base = hglib.shortreponame(lui) or os.path.basename(repopath)
+                c.set(b'paths', base,
+                      _asconfigliststr(os.path.join(repopath, b'**')))
         except (EnvironmentError, error.Abort, error.RepoError):
-            c.set('paths', '/', repopath)
+            c.set(b'paths', b'/', repopath)
         return lui, c
     else:
         return lui, None
@@ -234,9 +259,9 @@ def run(ui, *pats, **opts):
     webconfpath = opts.get('web_conf') or opts.get('webdir_conf')
 
     lui, webconf = _readconfig(ui, repopath, webconfpath)
-    dlg = ServeDialog(webconf=webconf)
+    dlg = ServeDialog(lui, webconf=webconf)
     try:
-        dlg.setport(int(lui.config('web', 'port', '8000')))
+        dlg.setport(int(lui.config(b'web', b'port')))
     except ValueError:
         pass
 
