@@ -6,15 +6,39 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
-from tortoisehg.hgqt import qtlib
-from tortoisehg.util.i18n import _
-from tortoisehg.util import menuthg
+from __future__ import absolute_import
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui import *
+from mercurial.windows import winreg
 
-from _winreg import (HKEY_CURRENT_USER, REG_SZ, REG_DWORD,
-                     OpenKey, CreateKey, QueryValueEx, SetValueEx)
+CreateKey = winreg.CreateKey
+HKEY_CURRENT_USER = winreg.HKEY_CURRENT_USER
+OpenKey = winreg.OpenKey
+QueryValueEx = winreg.QueryValueEx
+REG_DWORD = winreg.REG_DWORD
+REG_SZ = winreg.REG_SZ
+SetValueEx = winreg.SetValueEx
+
+from .qtgui import (
+    QApplication,
+    QCheckBox,
+    QDialog,
+    QDialogButtonBox,
+    QGridLayout,
+    QGroupBox,
+    QHBoxLayout,
+    QLabel,
+    QListWidget,
+    QListWidgetItem,
+    QPushButton,
+    QStyle,
+    QTabWidget,
+    QVBoxLayout,
+    QWidget,
+)
+
+from ..util import menuthg
+from ..util.i18n import _
+from . import qtlib
 
 THGKEY = 'TortoiseHg'
 OVLKEY = 'TortoiseOverlays'
@@ -200,7 +224,7 @@ class ShellConfigWindow(QDialog):
         self.load_shell_configs()
 
     def load_shell_configs(self):
-        for name, info in vars.iteritems():
+        for name, info in vars.items():
             default, regkey, regtype, evalfunc, wrfunc, cbattr = info
             try:
                 hkey = OpenKey(HKEY_CURRENT_USER, 'Software\\' + regkey)
@@ -208,7 +232,7 @@ class ShellConfigWindow(QDialog):
                 vars[name][0] = evalfunc(v)
             except (WindowsError, EnvironmentError):
                 pass
-            if cbattr != None:
+            if cbattr is not None:
                 checkbutton = getattr(self, cbattr)
                 checkbutton.setChecked(vars[name][0])
 
@@ -246,9 +270,9 @@ class ShellConfigWindow(QDialog):
         hkey = CreateKey(HKEY_CURRENT_USER, "Software\\" + THGKEY)
         SetValueEx(hkey, PROMOTEDITEMS, 0, REG_SZ, ','.join(promoted))
 
-        for name, info in vars.iteritems():
+        for name, info in vars.items():
             default, regkey, regtype, evalfunc, wrfunc, cbattr = info
-            if cbattr == None:
+            if cbattr is None:
                 continue
             checkbutton = getattr(self, cbattr)
             v = wrfunc(checkbutton.isChecked())
