@@ -54,8 +54,10 @@ from .qtgui import (
 )
 
 from mercurial import (
+    encoding,
     hg,
     httpconnection,
+    pycompat,
     util,
 )
 
@@ -77,22 +79,41 @@ from . import (
     thgrepo,
 )
 
+if hglib.TYPE_CHECKING:
+    from typing import (
+        Dict,
+        Iterable,
+        List,
+        Set,
+        Tuple,
+        Union,
+        Optional,
+    )
+    from mercurial import (
+        localrepo,
+    )
+
+
 def parseurl(url):
-    assert type(url) == unicode
+    # type: (pycompat.unicode) -> util.url
+    assert isinstance(url, pycompat.unicode), repr(url)
     return util.url(hglib.fromunicode(url))
 
 def linkify(url):
-    assert type(url) == unicode
+    # type: (pycompat.unicode) -> pycompat.unicode
+    assert isinstance(url, pycompat.unicode), repr(url)
     u = util.url(hglib.fromunicode(url))
-    if u.scheme in ('local', 'http', 'https'):
+    if u.scheme in (b'http', b'https'):
         safe = util.hidepassword(hglib.fromunicode(url))
         return u'<a href="%s">%s</a>' % (url, hglib.tounicode(safe))
+    elif u.scheme is None or u.scheme == b'file':
+        return u'<a href="%s">%s</a>' % (url, hglib.tounicode(u.path))
     else:
         return url
 
 # ignore preceding white spaces because ui.prompt() for username/password
 # writes extra " "s to the output channel. (hg 3.1)
-_extractnodeids = re.compile(r'^\s*([0-9a-f]{40})$', re.MULTILINE).findall
+_extractnodeids = re.compile(br'^\s*([0-9a-f]{40})$', re.MULTILINE).findall
 
 class SyncWidget(QWidget, qtlib.TaskWidget):
     newCommand = pyqtSignal(cmdcore.CmdSession)
@@ -115,16 +136,17 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
 
         self._repoagent = repoagent
         self._cmdsession = cmdcore.nullCmdSession()
-        self._lasturl = None  # peer repository of last command
+        # peer repository of last command
+        self._lasturl = None  # type: Optional[pycompat.unicode]
         self._lastbfile = None  # output bundle of last incoming command
-        self.opts = {}
+        self.opts = {}  # type: Dict[str, Union[bool, bytes]]
         self.cmenu = None
 
         s = QSettings()
         for opt in ('force', 'new-branch', 'noproxy', 'debug', 'mq'):
             val = qtlib.readBool(s, 'sync/' + opt)
             if val:
-                if opt != 'mq' or 'mq' in self.repo.extensions():
+                if opt != 'mq' or b'mq' in self.repo.extensions():
                     self.opts[opt] = val
         for opt in ('remotecmd', 'branch'):
             val = hglib.fromunicode(qtlib.readString(s, 'sync/' + opt))
@@ -165,7 +187,7 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
         newaction(_('Email outgoing changesets for remote repository'),
              'mail-forward', self.emailclicked)
 
-        if 'perfarce' in self.repo.extensions():
+        if b'perfarce' in self.repo.extensions():
             a = QAction(self)
             a.setToolTip(_('Manage pending perforce changelists'))
             a.setText('P4')
@@ -283,9 +305,9 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
         self._dialogs = qtlib.DialogKeeper(
             lambda self, dlgmeth, *args: dlgmeth(self, *args), parent=self)
 
-        self.curalias = None
+        self.curalias = None  # type: Optional[bytes]
         self.reload()
-        if 'default' in self.paths:
+        if b'default' in self.paths:
             self.setUrl('default')
         else:
             self.setEditUrl('')
@@ -325,7 +347,7 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
         return 0
 
     def refreshTargets(self, rev):
-        if type(rev) is not int:
+        if not isinstance(rev, int):
             return
 
         if rev >= len(self.repo):
@@ -355,39 +377,39 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
             self.refreshUrl()
 
             s = QSettings()
-            for opt, val in self.opts.iteritems():
-                if isinstance(val, str):
+            for opt, val in self.opts.items():
+                if isinstance(val, bytes):
                     val = hglib.tounicode(val)
                 s.setValue('sync/' + opt, val)
 
     @pyqtSlot()
     def reload(self):
         # Refresh configured paths
-        self.paths = {}
-        fn = self.repo.vfs.join('hgrc')
+        self.paths = {}  # type: Dict[bytes, bytes]
+        fn = self.repo.vfs.join(b'hgrc')
         fn, cfg = hgrcutil.loadIniFile([fn], self)
-        if 'paths' in cfg:
-            for alias in cfg['paths']:
-                self.paths[ alias ] = cfg['paths'][ alias ]
+        if b'paths' in cfg:
+            for alias in cfg[b'paths']:
+                self.paths[ alias ] = cfg[b'paths'][ alias ]
         tm = PathsModel(self.paths.items(), self)
         self.hgrctv.setModel(tm)
         sm = self.hgrctv.selectionModel()
         sm.currentRowChanged.connect(self.pathSelected)
 
         # Refresh post-pull
-        self.cachedpp = self.repo.postpull
-        name = _('Post Pull: ') + self.repo.postpull.title()
+        self.cachedpp = hglib.tounicode(self.repo.postpull)
+        name = _('Post Pull: ') + self.cachedpp.title()
         self.postpullbutton.setText(name)
 
         # Refresh related paths
-        known = set()
+        known = set()  # type: Set[bytes]
         known.add(os.path.abspath(self.repo.root).lower())
         for path in self.paths.values():
             if not util.hasscheme(path):
                 known.add(os.path.abspath(util.localpath(path)).lower())
             else:
                 known.add(path)
-        related = {}
+        related = {}  # type: Dict[bytes, bytes]
         repoid = hglib.repoidnode(self.repo)
         for root, shortname in thgrepo.relatedRepositories(repoid):
             if root == self.repo.root:
@@ -403,11 +425,11 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
             elif paths.is_on_fixed_drive(root):
                 # directly read the repository's configuration file
                 tempui = self.repo.ui.copy()
-                tempui.readconfig(os.path.join(root, '.hg', 'hgrc'))
+                tempui.readconfig(os.path.join(root, b'.hg', b'hgrc'))
                 ui = tempui
             else:
                 continue
-            for alias, path in ui.configitems('paths'):
+            for alias, path in ui.configitems(b'paths'):
                 if not util.hasscheme(path):
                     abs = os.path.abspath(util.localpath(path)).lower()
                 else:
@@ -422,7 +444,8 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
         sm.currentRowChanged.connect(self.pathSelected)
 
     def currentUrl(self):
-        return unicode(self.urlentry.text())
+        # type: () -> pycompat.unicode
+        return pycompat.unicode(self.urlentry.text())
 
     def urlChanged(self):
         self.securebutton.setEnabled('https://' in self.currentUrl())
@@ -432,12 +455,12 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
         self.urlChanged()
 
         opts = []
-        for opt, value in self.opts.iteritems():
+        for opt, value in self.opts.items():
             if value is True:
                 opts.append('--'+opt)
             elif value:
-                opts.append('--'+opt+'='+value)
-        self.optionslabel.setText(hglib.tounicode(' '.join(opts)))
+                opts.append('--'+opt+'='+hglib.tounicode(value))
+        self.optionslabel.setText(' '.join(opts))
         self.optionslabel.setVisible(bool(opts))
         self.optionshdrlabel.setVisible(bool(opts))
 
@@ -449,13 +472,16 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
         self.setEditUrl(hglib.tounicode(path))
 
     def setEditUrl(self, newurl):
+        # type: (pycompat.unicode) -> None
         'Set the current URL without changing the alias [unicode]'
         self.urlentry.setText(newurl)
         self.refreshUrl()
 
     def setUrl(self, newurl):
+        # type: (pycompat.unicode) -> None
         'Set the current URL to the given alias or URL [unicode]'
         model = self.hgrctv.model()
+        assert model is not None
         for col in (0, 1):  # search known (alias, url)
             ixs = model.match(model.index(0, col), Qt.DisplayRole, newurl, 1,
                               Qt.MatchFixedString | Qt.MatchCaseSensitive)
@@ -481,11 +507,11 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
     def dropEvent(self, event):
         data = event.mimeData()
         if data.hasUrls():
-            url = unicode(data.urls()[0].toString())
+            url = pycompat.unicode(data.urls()[0].toString())
             event.setDropAction(Qt.CopyAction)
             event.accept()
         elif data.hasText():
-            url = unicode(data.text())
+            url = pycompat.unicode(data.text())
             event.setDropAction(Qt.CopyAction)
             event.accept()
         else:
@@ -499,6 +525,7 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
 
     @pyqtSlot(QPoint, str, str, bool)
     def menuRequest(self, point, url, alias, editable):
+        # type: (QPoint, str, str, bool) -> None
         'menu event emitted by one of the two URL lists'
         if not self.cmenu:
             separator = (None, None, None)
@@ -530,25 +557,25 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
         self.cmenu.exec_(point)
 
     def exploreurl(self):
-        url = unicode(self.menuurl)
+        url = pycompat.unicode(self.menuurl)
         u = parseurl(url)
-        if not u.scheme or u.scheme == 'file':
+        if not u.scheme or u.scheme == b'file':
             qtlib.openlocalurl(u.path)
         else:
             QDesktopServices.openUrl(QUrl(url))
 
     def terminalurl(self):
-        url = unicode(self.menuurl)
+        url = pycompat.unicode(self.menuurl)
         u = parseurl(url)
-        if u.scheme and u.scheme != 'file':
+        if u.scheme and u.scheme != b'file':
             qtlib.InfoMsgBox(_('Repository not local'),
                         _('A terminal shell cannot be opened for remote'))
             return
-        qtlib.openshell(u.path, 'repo ' + u.path)
+        qtlib.openshell(u.path, b'repo ' + u.path)
 
     def editurl(self):
         alias = hglib.fromunicode(self.menualias)
-        urlu = unicode(self.menuurl)
+        urlu = pycompat.unicode(self.menuurl)
         dlg = SaveDialog(self._repoagent, alias, urlu, self, edit=True)
         dlg.setWindowFlags(Qt.Sheet)
         dlg.setWindowModality(Qt.WindowModal)
@@ -581,10 +608,10 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
     def saveclicked(self):
         if self.curalias:
             alias = self.curalias
-        elif 'default' not in self.paths:
-            alias = 'default'
+        elif b'default' not in self.paths:
+            alias = b'default'
         else:
-            alias = 'new'
+            alias = b'new'
         dlg = SaveDialog(self._repoagent, alias, self.currentUrl(), self)
         dlg.setWindowFlags(Qt.Sheet)
         dlg.setWindowModality(Qt.WindowModal)
@@ -619,6 +646,7 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
         self.stopAction.setEnabled(not sess.isFinished())
 
     def run(self, cmdline, details):
+        # type: (List[Union[str, bytes]], Iterable[str]) -> cmdcore.CmdSession
         if not self._cmdsession.isFinished():
             return cmdcore.nullCmdSession()
         self.lastcmdline = list(cmdline)
@@ -659,8 +687,8 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
                     parent=self)
             return cmdcore.nullCmdSession()
 
-        if u.scheme == 'https':
-            if self.repo.ui.configbool('insecurehosts', u.host):
+        if u.scheme == b'https':
+            if self.repo.ui.configbool(b'insecurehosts', u.host):
                 cmdline.append('--insecure')
             if u.user:
                 cleanurl = util.removeauth(lurl)
@@ -668,7 +696,7 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
                                                     u.user)
                 if res:
                     group, auth = res
-                    if auth.get('username'):
+                    if auth.get(b'username'):
                         if qtlib.QuestionMsgBox(
                             _('Redundant authentication info'),
                             _('You have authentication info configured for '
@@ -680,7 +708,7 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
 
         if not self.opts.get('mq'):
             cmdline.append(lurl)
-        ucmdline = map(hglib.tounicode, cmdline)
+        ucmdline = pycompat.maplist(hglib.tounicode, cmdline)
         # bypass overlay of incoming bundle to pull changes
         overlay = ucmdline[0] not in ('fetch', 'incoming', 'pull')
         self._cmdsession = sess = self._repoagent.runCommand(ucmdline, self,
@@ -720,6 +748,7 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
             self.pushclicked(confirm, **kwargs)
 
     def pullBundle(self, bundle, rev, bsource=None):
+        # type: (pycompat.unicode, Optional[bytes], Optional[pycompat.unicode]) -> None
         'accept bundle changesets'
         if not self._cmdsession.isFinished():
             self.showMessage.emit(_('sync command already running'))
@@ -731,7 +760,7 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
         # go on default branch.
         self.setEditUrl(bsource or bundle)
         if rev is not None:
-            self.opts['rev'] = str(rev)
+            self.opts['rev'] = rev
         self.pullclicked(bsource)
         self.setEditUrl(save)
         self.opts['rev'] = orev
@@ -741,6 +770,7 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
     ##
 
     def linkifyWithTarget(self, url):
+        # type: (pycompat.unicode) -> pycompat.unicode
         link = linkify(url)
         if self.targetcheckbox.isChecked():
             link += u" (%s)" % self.targetcombo.currentText()
@@ -750,11 +780,14 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
         url = self.currentUrl()
         link = self.linkifyWithTarget(url)
         if not url.startswith('p4://'):
-            bfile = hglib.fromunicode(url)
+            bfile = url.replace('/', '_')
             for badchar in (':', '*', '\\', '?', '#'):
                 bfile = bfile.replace(badchar, '')
-            bfile = bfile.replace('/', '_')
-            bfile = tempfile.mktemp('.hg', bfile+'_', qtlib.gettempdir())
+            fd, bfile = tempfile.mkstemp(
+                suffix=b'.hg',
+                prefix=hglib.fromunicode(bfile) + b'_',
+                dir=qtlib.gettempdir())
+            os.close(fd)
             self._lastbfile = hglib.tounicode(bfile)
             cmdline = ['incoming', '--quiet', '--bundle', bfile]
             sess = self.run(cmdline, ('force', 'branch', 'rev'))
@@ -773,7 +806,9 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
         link = self.linkifyWithTarget(self._lasturl)
         if ret == 0:
             self.showMessage.emit(_('Found incoming changesets from %s') % link)
-            if self._lastbfile and os.path.exists(self._lastbfile):
+            # hg-git 'incoming' gives no data for bundle
+            if (self._lastbfile and os.path.exists(self._lastbfile)
+                and os.path.getsize(self._lastbfile) > 0):
                 self.incomingBundle.emit(self._lastbfile, self._lasturl)
         elif ret == 1:
             self.showMessage.emit(_('No incoming changesets from %s') % link)
@@ -782,11 +817,14 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
                                   % (link, ret))
 
     def pullclicked(self, url=None):
+        # type: (Optional[pycompat.unicode]) -> None
         link = self.linkifyWithTarget(url or self.currentUrl())
 
         cmdline = ['pull', '--verbose']
-        uimerge = self.repo.ui.configbool('tortoisehg', 'autoresolve', True) \
-            and 'ui.merge=internal:merge' or 'ui.merge=internal:fail'
+        if self._repoagent.configBool('tortoisehg', 'autoresolve'):
+            uimerge = 'ui.merge=internal:merge'
+        else:
+            uimerge = 'ui.merge=internal:fail'
         if self.cachedpp == 'rebase':
             cmdline += ['--rebase', '--config', uimerge]
         elif self.cachedpp == 'update':
@@ -815,13 +853,13 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
         self.pullCompleted.emit()
         # handle file conflicts during rebase
         if self.cachedpp in ('rebase', 'updateorrebase'):
-            if os.path.exists(self.repo.vfs.join('rebasestate')):
+            if os.path.exists(self.repo.vfs.join(b'rebasestate')):
                 dlg = rebase.RebaseDialog(self._repoagent, self)
                 dlg.exec_()
                 return
         # handle file conflicts during update
         for root, path, status in thgrepo.recursiveMergeStatus(self.repo):
-            if status == 'u':
+            if status == b'u':
                 qtlib.InfoMsgBox(_('Merge caused file conflicts'),
                                 _('File conflicts need to be resolved'))
                 dlg = resolve.ResolveDialog(self._repoagent, self)
@@ -840,8 +878,8 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
     def _onOutgoingFinished(self, ret):
         link = self.linkifyWithTarget(self._lasturl)
         if ret == 0:
-            data = str(self._cmdsession.readAll())
-            nodes = _extractnodeids(data)
+            data = bytes(self._cmdsession.readAll())
+            nodes = [pycompat.sysstr(node) for node in _extractnodeids(data)]
             self.showMessage.emit(_('%d outgoing changesets to %s') %
                                   (len(nodes), link))
             self.outgoingNodes.emit(nodes)
@@ -861,7 +899,7 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
     def _onP4pendingFinished(self, ret):
         pending = {}
         if ret == 0:
-            output = str(self._cmdsession.readAll())
+            output = str(self._cmdsession.readAll())  # decode?
             for line in output.splitlines():
                 try:
                     hashes = line.split(' ')
@@ -877,6 +915,7 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
                             raise ValueError
                         pending[changelist] = hashes
                 except (ValueError, IndexError):
+                    # XXX: This is unconditionally overwritten before use
                     text = _('Unable to parse p4pending output')
             if pending:
                 text = _('%d pending changelists found') % len(pending)
@@ -896,7 +935,7 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
 
     def pushclicked(self, confirm, rev=None, branch=None, pushall=False):
         if confirm is None:
-            confirm = self.repo.ui.configbool('tortoisehg', 'confirmpush', True)
+            confirm = self._repoagent.configBool('tortoisehg', 'confirmpush')
         if rev == '':
             rev = None
         if branch == '':
@@ -933,8 +972,8 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
         if branch is None:
             branch = self.opts.get('branch')
         if not pushall and rev is None and branch is None:
-            defaultpush = self.repo.ui.config('tortoisehg', 'defaultpush',
-                                              'all')
+            defaultpush = self._repoagent.configString(
+                'tortoisehg', 'defaultpush')
             if self.targetcheckbox.isChecked():
                 pass
             elif defaultpush == 'all':
@@ -1012,7 +1051,7 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
     def _onOutgoingEmailFinished(self, ret):
         if ret == 0:
             cmdline = self.lastcmdline
-            data = str(self._cmdsession.readAll())
+            data = bytes(self._cmdsession.readAll())
             revs = tuple(self.repo[n].rev() for n in _extractnodeids(data))
             self.showMessage.emit(_('%d outgoing changesets') % len(revs))
             try:
@@ -1046,7 +1085,7 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
     @pyqtSlot(str)
     def removeAlias(self, alias):
         alias = hglib.fromunicode(alias)
-        fn = self.repo.vfs.join('hgrc')
+        fn = self.repo.vfs.join(b'hgrc')
         fn, cfg = hgrcutil.loadIniFile([fn], self)
         if not hasattr(cfg, 'write'):
             qtlib.WarningMsgBox(_('Unable to remove URL'),
@@ -1054,12 +1093,12 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
             return
         if fn is None:
             return
-        if alias in cfg['paths']:
-            del cfg['paths'][alias]
+        if alias in cfg[b'paths']:
+            del cfg[b'paths'][alias]
         try:
             wconfig.writefile(cfg, fn)
             self._repoagent.pollStatus()
-        except EnvironmentError, e:
+        except EnvironmentError as e:
             qtlib.WarningMsgBox(_('Unable to write configuration file'),
                                 hglib.tounicode(str(e)), parent=self)
         self.reload()
@@ -1086,15 +1125,15 @@ class PostPullDialog(QDialog):
         layout.addWidget(self._opchecks['none'])
         layout.addWidget(self._opchecks['update'])
 
-        if 'fetch' in repo.extensions():
+        if b'fetch' in repo.extensions():
             btntxt = _('Fetch - use fetch (auto merge pulled changes)')
         else:
             btntxt = _('Fetch - use fetch extension (fetch is not active!)')
         self._opchecks['fetch'] = chk = QRadioButton(btntxt)
         layout.addWidget(chk)
-        chk.setVisible('fetch' in repo.extensions())
+        chk.setVisible(b'fetch' in repo.extensions())
 
-        if 'rebase' in repo.extensions():
+        if b'rebase' in repo.extensions():
             rebasetxt = _('Rebase - rebase local commits above pulled changes')
             updateorrebasetxt = _('UpdateOrRebase - pull, then try to update '
                                   'or rebase')
@@ -1105,19 +1144,19 @@ class PostPullDialog(QDialog):
                                   '(rebase is not active!)')
         self._opchecks['rebase'] = chk = QRadioButton(rebasetxt)
         layout.addWidget(chk)
-        chk.setVisible('rebase' in repo.extensions())
+        chk.setVisible(b'rebase' in repo.extensions())
         self._opchecks['updateorrebase'] = chk = QRadioButton(updateorrebasetxt)
         layout.addWidget(chk)
-        chk.setVisible('rebase' in repo.extensions())
+        chk.setVisible(b'rebase' in repo.extensions())
 
-        chk = self._opchecks[repo.postpull]
+        chk = self._opchecks[hglib.tounicode(repo.postpull)]
         chk.setChecked(True)
         chk.show()
 
         self.autoresolve_chk = QCheckBox(_('Automatically resolve merge '
                                            'conflicts where possible'))
         self.autoresolve_chk.setChecked(
-            repo.ui.configbool('tortoisehg', 'autoresolve', True))
+            repoagent.configBool('tortoisehg', 'autoresolve'))
         layout.addWidget(self.autoresolve_chk)
 
         cfglabel = QLabel(_('<a href="config">Launch settings tool...</a>'))
@@ -1137,6 +1176,7 @@ class PostPullDialog(QDialog):
         return self._repoagent.rawRepo()
 
     def linkactivated(self, command):
+        # type: (pycompat.unicode) -> None
         if command == 'config':
             from tortoisehg.hgqt.settings import SettingsDialog
             sd = SettingsDialog(configrepo=False, focus='tortoisehg.postpull',
@@ -1144,11 +1184,12 @@ class PostPullDialog(QDialog):
             sd.exec_()
 
     def getValue(self):
-        return iter(op for op, chk in self._opchecks.iteritems()
-                    if chk.isChecked()).next()
+        # type: () -> str
+        return next(iter(op for op, chk in self._opchecks.items()
+                    if chk.isChecked()))
 
     def accept(self):
-        path = self.repo.vfs.join('hgrc')
+        path = self.repo.vfs.join(b'hgrc')
         fn, cfg = hgrcutil.loadIniFile([path], self)
         if not hasattr(cfg, 'write'):
             qtlib.WarningMsgBox(_('Unable to save post pull operation'),
@@ -1157,12 +1198,13 @@ class PostPullDialog(QDialog):
         if fn is None:
             return
         try:
-            cfg.set('tortoisehg', 'postpull', self.getValue())
-            cfg.set('tortoisehg', 'autoresolve',
-                    self.autoresolve_chk.isChecked())
+            cfg.set(b'tortoisehg', b'postpull',
+                    encoding.strtolocal(self.getValue()))
+            cfg.set(b'tortoisehg', b'autoresolve',
+                    self.autoresolve_chk.isChecked() and b'True' or b'False')
             wconfig.writefile(cfg, fn)
             self._repoagent.pollStatus()
-        except EnvironmentError, e:
+        except EnvironmentError as e:
             qtlib.WarningMsgBox(_('Unable to write configuration file'),
                                 hglib.tounicode(str(e)), parent=self)
         super(PostPullDialog, self).accept()
@@ -1199,7 +1241,7 @@ class SaveDialog(QDialog):
 
         u = parseurl(urlu)
         clearable = bool(not edit and (u.user or u.passwd)
-                         and u.scheme in ('http', 'https'))
+                         and u.scheme in (b'http', b'https'))
         self.clearcb = QCheckBox(_('Remove authentication data from URL'))
         self.clearcb.setToolTip(
             _('User authentication data should be associated with the '
@@ -1232,7 +1274,8 @@ class SaveDialog(QDialog):
         self._updateUi()
 
     def savePath(self, repo, alias, path, confirm=True):
-        fn = repo.vfs.join('hgrc')
+        # type: (localrepo.localrepository, bytes, bytes, bool) -> None
+        fn = repo.vfs.join(b'hgrc')
         fn, cfg = hgrcutil.loadIniFile([fn], self)
         if not hasattr(cfg, 'write'):
             qtlib.WarningMsgBox(_('Unable to save an URL'),
@@ -1241,30 +1284,30 @@ class SaveDialog(QDialog):
         if fn is None:
             return
         if (confirm and (not self.edit or path != self.origurl)
-            and alias in cfg['paths']):
+            and alias in cfg[b'paths']):
             if not qtlib.QuestionMsgBox(_('Confirm URL replace'),
                 _('%s already exists, replace URL?') % hglib.tounicode(alias),
                 parent=self):
                 return
-        cfg.set('paths', alias, path)
+        cfg.set(b'paths', alias, path)
         if self.edit and alias != self.origalias:
-            cfg.remove('paths', self.origalias)
+            cfg.remove(b'paths', self.origalias)
         try:
             wconfig.writefile(cfg, fn)
-        except EnvironmentError, e:
+        except EnvironmentError as e:
             qtlib.WarningMsgBox(_('Unable to write configuration file'),
                                 hglib.tounicode(str(e)), parent=self)
         if self.updatesubpaths.isChecked():
-            ctx = repo['.']
-            for subname in ctx.substate:
-                if ctx.substate[subname][2] != 'hg':
+            ctx = repo[b'.']
+            for subname in ctx.substate:  # type: bytes
+                if ctx.substate[subname][2] != b'hg':
                     continue
                 if not os.path.exists(repo.wjoin(subname)):
                     continue
                 defaultsubpath = ctx.substate[subname][0]
                 pathurl = util.url(path)
                 if pathurl.scheme:
-                    subpath = str(pathurl).rstrip('/') + '/' + subname
+                    subpath = bytes(pathurl).rstrip(b'/') + b'/' + subname
                 else:
                     subpath = os.path.normpath(os.path.join(path, subname))
                 if defaultsubpath != subname:
@@ -1322,13 +1365,14 @@ def _addBrowseButton(edit, slot):
 
 class SecureDialog(QDialog):
     def __init__(self, repoagent, urlu, parent):
+        # type: (thgrepo.RepoAgent, pycompat.unicode, QWidget) -> None
         super(SecureDialog, self).__init__(parent)
         self._repoagent = repoagent
         self._querysess = cmdcore.nullCmdSession()
         repo = repoagent.rawRepo()
         self._url = urlu
         u = parseurl(urlu)
-        assert u.host
+        assert u.host, repr(u)
         uhost = hglib.tounicode(u.host)
         self.setWindowTitle(_('Security: ') + uhost)
         self.setWindowFlags(self.windowFlags() & \
@@ -1336,14 +1380,15 @@ class SecureDialog(QDialog):
 
         # if the already user has an [auth] configuration for this URL, use it
         cleanurl = util.removeauth(hglib.fromunicode(urlu))
-        res = httpconnection.readauthforuri(repo.ui, cleanurl, u.user)
+        res = httpconnection.readauthforuri(repo.ui, cleanurl, u.user)  # type: Optional[Tuple[bytes, Dict[bytes, bytes]]]
+        self.schemes = None  # type: Optional[bytes]
         if res:
             self.alias, auth = res
         else:
             self.alias, auth = u.host, {}
-        self.host = u.host
-        if cleanurl.startswith('svn+https://'):
-            self.schemes = 'svn+https'
+        self.host = u.host  # type: bytes
+        if cleanurl.startswith(b'svn+https://'):
+            self.schemes = b'svn+https'
         else:
             self.schemes = None
 
@@ -1363,12 +1408,12 @@ class SecureDialog(QDialog):
         self.insecureradio = QRadioButton(
             _('No host validation, but still encrypted (bad)'))
         hbox = QHBoxLayout()
-        fprint = repo.ui.config('hostsecurity', u.host + ':fingerprints', '')
+        fprint = repo.ui.config(b'hostsecurity', u.host + b':fingerprints')
         if not fprint:
-            fprint = repo.ui.config('hostfingerprints', u.host, '')
+            fprint = repo.ui.config(b'hostfingerprints', u.host, b'')
             if fprint:
-                fprint = 'sha1:' + fprint
-        self.fprintentry = le = QLineEdit(fprint)
+                fprint = b'sha1:' + fprint
+        self.fprintentry = le = QLineEdit(hglib.tounicode(fprint))
         self.fprintradio.toggled.connect(self.fprintentry.setEnabled)
         self.fprintentry.setEnabled(False)
         if hasattr(le, 'setPlaceholderText'): # Qt >= 4.7
@@ -1385,7 +1430,7 @@ class SecureDialog(QDialog):
         self.cacertradio.setChecked(True) # default
         if fprint:
             self.fprintradio.setChecked(True)
-        elif repo.ui.config('insecurehosts', u.host):
+        elif repo.ui.config(b'insecurehosts', u.host):
             self.insecureradio.setChecked(True)
 
         self.fprintradio.toggled.connect(self._updateUi)
@@ -1396,21 +1441,23 @@ class SecureDialog(QDialog):
         e.addItem(_('TLS 1.0'), 'tls1.0')
         e.addItem(_('TLS 1.1'), 'tls1.1')
         e.addItem(_('TLS 1.2'), 'tls1.2')
-        protocol = repo.ui.config('hostsecurity', '%s:minimumprotocol' % u.host)
-        e.setCurrentIndex(e.findData(hglib.tounicode(protocol or '')))
+        protocol = repo.ui.config(b'hostsecurity',
+                                  b'%s:minimumprotocol' % u.host)
+        e.setCurrentIndex(e.findData(hglib.tounicode(protocol or b'')))
         hbox = QHBoxLayout()
         hbox.addWidget(QLabel(_('Minimum Protocol')))
         hbox.addWidget(self._protocolcombo)
         vbox.addLayout(hbox)
 
-        self._authentries = {}  # key: QLineEdit
+        self._authentries = {}  # type: Dict[bytes, QLineEdit]
         authbox = QGroupBox(_('User Authentication'))
         form = QFormLayout()
         authbox.setLayout(form)
         self.layout().addWidget(authbox)
 
-        k = 'username'
-        self._authentries[k] = e = QLineEdit(u.user or auth.get(k, ''))
+        k = b'username'
+        c = hglib.tounicode(u.user or auth.get(k, b''))
+        self._authentries[k] = e = QLineEdit(c)
         e.setToolTip(
 _('''Optional. Username to authenticate with. If not given, and the remote
 site requires basic or digest authentication, the user will be prompted for
@@ -1418,31 +1465,34 @@ it. Environment variables are expanded in the username letting you do
 foo.username = $USER.'''))
         form.addRow(_('Username'), e)
 
-        k = 'password'
-        self._authentries[k] = e = QLineEdit(u.passwd or auth.get(k, ''))
+        k = b'password'
+        c = hglib.tounicode(u.passwd or auth.get(k, b''))
+        self._authentries[k] = e = QLineEdit(c)
         e.setEchoMode(QLineEdit.Password)
         e.setToolTip(
 _('''Optional. Password to authenticate with. If not given, and the remote
 site requires basic or digest authentication, the user will be prompted for
 it.'''))
         form.addRow(_('Password'), e)
-        if 'mercurial_keyring' in repo.extensions():
+        if b'mercurial_keyring' in repo.extensions():
             e.clear()
             e.setEnabled(False)
             e.setToolTip(_('Mercurial keyring extension is enabled. '
                            'Passwords will be stored in a platform-native '
                            'secure method.'))
 
-        k = 'key'
-        self._authentries[k] = e = QLineEdit(auth.get(k, ''))
+        k = b'key'
+        c = hglib.tounicode(auth.get(k, b''))
+        self._authentries[k] = e = QLineEdit(c)
         e.setToolTip(
 _('''Optional. PEM encoded client certificate key file. Environment variables
 are expanded in the filename.'''))
         form.addRow(_('User Certificate Key'),
                     _addBrowseButton(e, self._browseClientKey))
 
-        k = 'cert'
-        self._authentries[k] = e = QLineEdit(auth.get(k, ''))
+        k = b'cert'
+        c = hglib.tounicode(auth.get(k, b''))
+        self._authentries[k] = e = QLineEdit(c)
         e.setToolTip(
 _('''Optional. PEM encoded client certificate chain file. Environment variables
 are expanded in the filename.'''))
@@ -1458,7 +1508,7 @@ are expanded in the filename.'''))
         self.layout().addWidget(bb)
 
         self._updateUi()
-        e = self._authentries['username']
+        e = self._authentries[b'username']
         e.selectAll()
         QTimer.singleShot(0, e.setFocus)
 
@@ -1475,7 +1525,7 @@ are expanded in the filename.'''))
     def _onQueryFingerprintFinished(self, ret):
         sess = self._querysess
         if ret == 0:
-            data = str(sess.readAll())
+            data = bytes(sess.readAll())
             self.fprintentry.setText(hglib.tounicode(data).strip())
         else:
             cmdui.errorMessageBox(sess, self, _('Certificate Query Error'))
@@ -1486,7 +1536,7 @@ are expanded in the filename.'''))
 
     @pyqtSlot()
     def _browseClientKey(self):
-        e = self._authentries['key']
+        e = self._authentries[b'key']
         n, _f = QFileDialog.getOpenFileName(
             self, _('Select User Certificate Key File'), e.text(),
             ';;'.join([_('PEM files (*.pem *.key)'), _('All files (*)')]))
@@ -1495,7 +1545,7 @@ are expanded in the filename.'''))
 
     @pyqtSlot()
     def _browseClientCert(self):
-        e = self._authentries['cert']
+        e = self._authentries[b'cert']
         n, _f = QFileDialog.getOpenFileName(
             self, _('Select User Certificate Chain File'), e.text(),
             ';;'.join([_('PEM files (*.pem *.crt *.cer)'), _('All files (*)')]))
@@ -1513,6 +1563,7 @@ are expanded in the filename.'''))
             return
 
         def setorclear(section, item, value):
+            # type: (bytes, bytes, Optional[bytes]) -> None
             if value:
                 cfg.set(section, item, value)
             elif not value and item in cfg[section]:
@@ -1526,24 +1577,24 @@ are expanded in the filename.'''))
             insecure = None
         else:
             fprint = None
-            insecure = '1'
-        setorclear('hostsecurity', '%s:fingerprints' % self.host, fprint)
-        setorclear('insecurehosts', self.host, insecure)
+            insecure = b'1'
+        setorclear(b'hostsecurity', b'%s:fingerprints' % self.host, fprint)
+        setorclear(b'insecurehosts', self.host, insecure)
 
         e = self._protocolcombo
         protocol = hglib.fromunicode(e.itemData(e.currentIndex()))
-        setorclear('hostsecurity', '%s:minimumprotocol' % self.host, protocol)
+        setorclear(b'hostsecurity', b'%s:minimumprotocol' % self.host, protocol)
 
-        cfg.set('auth', self.alias+'.prefix', self.host)
-        for k in ['username', 'password', 'key', 'cert']:
-            setorclear('auth', '%s.%s' % (self.alias, k),
+        cfg.set(b'auth', b'%s.prefix' % self.alias, self.host)
+        for k in [b'username', b'password', b'key', b'cert']:
+            setorclear(b'auth', b'%s.%s' % (self.alias, k),
                        hglib.fromunicode(self._authentries[k].text()))
-        setorclear('auth', self.alias+'.schemes', self.schemes)
+        setorclear(b'auth', b'%s.schemes' % self.alias, self.schemes)
 
         try:
             wconfig.writefile(cfg, fn)
             self._repoagent.pollStatus()
-        except EnvironmentError, e:
+        except EnvironmentError as e:
             qtlib.WarningMsgBox(_('Unable to write configuration file'),
                                 hglib.tounicode(str(e)), parent=self)
         super(SecureDialog, self).accept()
@@ -1592,6 +1643,7 @@ class PathsTree(QTreeView):
 
 class PathsModel(QAbstractTableModel):
     def __init__(self, pathlist, parent=None):
+        # type: (Iterable[Tuple[bytes, bytes]], Optional[QWidget]) -> None
         QAbstractTableModel.__init__(self, parent)
         self.headers = (_('Alias'), _('URL'))
         self.rows = []
@@ -1650,6 +1702,7 @@ class PathsModel(QAbstractTableModel):
 class OptionsDialog(QDialog):
     'Utility dialog for configuring uncommon options'
     def __init__(self, repoagent, opts, parent):
+        # type: (thgrepo.RepoAgent, Dict[str, bytes], QWidget) -> None
         QDialog.__init__(self, parent)
         self.setWindowTitle(_('%s - sync options') % repoagent.displayName())
 
@@ -1671,7 +1724,7 @@ class OptionsDialog(QDialog):
             _('Temporarily disable configured HTTP proxy'))
         self.noproxycb.setChecked(opts.get('noproxy', False))
         layout.addWidget(self.noproxycb)
-        proxy = repo.ui.config('http_proxy', 'host')
+        proxy = repoagent.configString('http_proxy', 'host')
         self.noproxycb.setEnabled(bool(proxy))
 
         self.debugcb = QCheckBox(
@@ -1681,7 +1734,7 @@ class OptionsDialog(QDialog):
 
         self.mqcb = QCheckBox(_('Work on patch queue (--mq)'))
         self.mqcb.setChecked(opts.get('mq', False))
-        self.mqcb.setVisible('mq' in repo.extensions())
+        self.mqcb.setVisible(b'mq' in repo.extensions())
         layout.addWidget(self.mqcb)
 
         form = QFormLayout()
@@ -1707,7 +1760,7 @@ class OptionsDialog(QDialog):
         layout.addWidget(bb)
 
     def accept(self):
-        outopts = {}
+        outopts = {}  # type: Dict[str, Union[bool, bytes]]
         for name, le in (('remotecmd', self.remotele),
                          ('branch', self.branchle)):
             outopts[name] = hglib.fromunicode(le.text()).strip()
