@@ -27,6 +27,10 @@ from .qtgui import (
     QWidget,
 )
 
+from mercurial import (
+    scmutil,
+)
+
 from ..util import (
     hglib,
     i18n,
@@ -41,8 +45,10 @@ keep = i18n.keepgettext()
 
 class TagDialog(QDialog):
 
-    def __init__(self, repoagent, tag='', rev='tip', parent=None, opts={}):
+    def __init__(self, repoagent, tag='', rev='tip', parent=None, opts=None):
         super(TagDialog, self).__init__(parent)
+        if opts is None:
+            opts = {}
         self.setWindowFlags(self.windowFlags() &
                             ~Qt.WindowContextHelpButtonHint)
 
@@ -65,7 +71,7 @@ class TagDialog(QDialog):
         base.addWidget(formwidget)
 
         repo = repoagent.rawRepo()
-        ctx = repo[rev]
+        ctx = scmutil.revsymbol(repo, hglib.fromunicode(rev))
         form.addRow(_('Revision:'), QLabel('%d (%s)' % (ctx.rev(), ctx)))
         self.rev = ctx.rev()
 
@@ -101,7 +107,7 @@ class TagDialog(QDialog):
         optbox.addWidget(self.replaceCheckBox)
 
         self.englishCheckBox = QCheckBox(_('Use English commit message'))
-        engmsg = repo.ui.configbool('tortoisehg', 'engmsg', False)
+        engmsg = repoagent.configBool('tortoisehg', 'engmsg')
         self.englishCheckBox.setChecked(engmsg)
         optbox.addWidget(self.englishCheckBox)
 
@@ -156,11 +162,9 @@ class TagDialog(QDialog):
         """ update display on dialog with recent repo data """
         cur = self.tagCombo.currentText()
 
-        tags = list(self.repo.tags())
-        tags.sort(reverse=True)
         self.tagCombo.clear()
-        for tag in tags:
-            if tag in ('tip', 'qbase', 'qtip', 'qparent'):
+        for tag in sorted(self.repo.tags(), reverse=True):
+            if tag in (b'tip', b'qbase', b'qtip', b'qparent'):
                 continue
             self.tagCombo.addItem(hglib.tounicode(tag))
         if cur:
