@@ -8,6 +8,8 @@
 # See mercurial/extensions.py, comments to wrapfunction, for this approach
 # to extending repositories and change contexts.
 
+from __future__ import absolute_import
+
 import os
 import sys
 import shutil
@@ -15,50 +17,83 @@ import tempfile
 import re
 import time
 
-from PyQt4.QtCore import *
+from .qtcore import (
+    QFile,
+    QFileSystemWatcher,
+    QIODevice,
+    QObject,
+    QSignalMapper,
+    pyqtSignal,
+    pyqtSlot,
+)
 
-from mercurial import hg, error, bundlerepo, extensions, filemerge, node
-from mercurial import localrepo, subrepo
-from mercurial import ui as uimod
 from hgext import mq
+from mercurial import (
+    bundlerepo,
+    error,
+    extensions,
+    filemerge,
+    hg,
+    localrepo,
+    node,
+    pycompat,
+    subrepo,
+)
 
-from tortoisehg.util import hglib, paths
-from tortoisehg.util.patchctx import patchctx
-from tortoisehg.hgqt import cmdcore, qtlib
+from ..util import (
+    hglib,
+    paths,
+)
+from ..util.patchctx import patchctx
+from . import (
+    cmdcore,
+    hgconfig,
+)
+
+if hglib.TYPE_CHECKING:
+    from typing import (
+        List,
+        Optional,
+        Text,
+        Union,
+    )
+    from .qtgui import (
+        QWidget,
+    )
 
 _repocache = {}
-_kbfregex = re.compile(r'^\.kbf/')
-_lfregex = re.compile(r'^\.hglf/')
+_kbfregex = re.compile(br'^\.kbf/')
+_lfregex = re.compile(br'^\.hglf/')
 
 # thgrepo.repository() will be deprecated
-def repository(_ui=None, path=''):
+def repository(_ui=None, path=b''):
     '''Returns a subclassed Mercurial repository to which new
     THG-specific methods have been added. The repository object
     is obtained using mercurial.hg.repository()'''
+    assert isinstance(path, bytes), repr(path)
     if path not in _repocache:
         if _ui is None:
-            _ui = uimod.ui()
+            _ui = hglib.loadui()
         try:
             repo = hg.repository(_ui, path)
             repo = repo.unfiltered()
             repo.__class__ = _extendrepo(repo)
-            repo = repo.filtered('visible')
+            repo = repo.filtered(b'visible')
             agent = RepoAgent(repo)
             _repocache[path] = agent.rawRepo()
             return agent.rawRepo()
         except EnvironmentError:
-            raise error.RepoError('Cannot open repository at %s' % path)
-    if not os.path.exists(os.path.join(path, '.hg/')):
+            raise error.RepoError(b'Cannot open repository at %s' % path)
+    if not os.path.exists(os.path.join(path, b'.hg/')):
         del _repocache[path]
-        # this error must be in local encoding
-        raise error.RepoError('%s is not a valid repository' % path)
+        raise error.RepoError(b'%s is not a valid repository' % path)
     return _repocache[path]
 
 def _filteredrepo(repo, hiddenincluded):
     if hiddenincluded:
         return repo.unfiltered()
     else:
-        return repo.filtered('visible')
+        return repo.filtered(b'visible')
 
 
 # flags describing changes that could occur in repository
@@ -123,7 +158,7 @@ class RepoWatcher(QObject):
         # ".hg/store" from the list, but actually they are not watched.
         # Thus, they cannot be watched again by the same fswatcher instance.
         if self._fswatcher.directories() or self._fswatcher.files():
-            self._ui.debug('failed to remove paths - destroying watcher\n')
+            self._ui.debug(b'failed to remove paths - destroying watcher\n')
             self._fswatcher.setParent(None)
             self._fswatcher = None
 
@@ -154,7 +189,7 @@ class RepoWatcher(QObject):
     @pyqtSlot()
     def _onFsChanged(self):
         if self._deferredpoll:
-            self._ui.debug('filesystem change detected, but poll deferred\n')
+            self._ui.debug(b'filesystem change detected, but poll deferred\n')
             self._deferredpoll |= _PollFsChangesPending
             return
         self._pollFsChanges()
@@ -169,16 +204,16 @@ class RepoWatcher(QObject):
 
     def _addMissingPaths(self):
         'Add files to watcher that may have been added or replaced'
-        existing = [f for f, (_flag, watched) in self._filesmap.iteritems()
+        existing = [f for f, (_flag, watched) in self._filesmap.items()
                     if watched and f in self._laststats]
-        files = [unicode(f) for f in self._fswatcher.files()]
+        files = [pycompat.unicode(f) for f in self._fswatcher.files()]
         for f in existing:
             if hglib.tounicode(f) not in files:
-                self._ui.debug('add file to watcher: %s\n' % f)
+                self._ui.debug(b'add file to watcher: %s\n' % f)
                 self._fswatcher.addPath(hglib.tounicode(f))
         for f in self._repo.uifiles():
             if f and os.path.exists(f) and hglib.tounicode(f) not in files:
-                self._ui.debug('add ui file to watcher: %s\n' % f)
+                self._ui.debug(b'add ui file to watcher: %s\n' % f)
                 self._fswatcher.addPath(hglib.tounicode(f))
 
     def clearStatus(self):
@@ -187,36 +222,36 @@ class RepoWatcher(QObject):
 
     def pollStatus(self):
         if self._deferredpoll:
-            self._ui.debug('poll request deferred\n')
+            self._ui.debug(b'poll request deferred\n')
             self._deferredpoll |= _PollStatusPending
             return
         self._pollStatus()
 
     def _pollStatus(self):
         if not os.path.exists(self._repo.path):
-            self._ui.debug('repository destroyed: %s\n' % self._repo.root)
+            self._ui.debug(b'repository destroyed: %s\n' % self._repo.root)
             self.repositoryDestroyed.emit()
             return
         if self._locked():
-            self._ui.debug('locked, aborting\n')
+            self._ui.debug(b'locked, aborting\n')
             return
         curstats, curdata = self._readState()
         changeflags = self._calculateChangeFlags(curstats, curdata)
         if self._locked():
-            self._ui.debug('lock still held - ignoring for now\n')
+            self._ui.debug(b'lock still held - ignoring for now\n')
             return
         self._laststats = curstats
         self._lastdata = curdata
         if changeflags:
-            self._ui.debug('change found (flags = 0x%x)\n' % changeflags)
+            self._ui.debug(b'change found (flags = 0x%x)\n' % changeflags)
             self.repositoryChanged.emit(changeflags)  # may update repo paths
             self._fixState()
         self._checkuimtime()
 
     def _locked(self):
-        if os.path.lexists(self._repo.join('wlock')):
+        if os.path.lexists(self._repo.vfs.join(b'wlock')):
             return True
-        if os.path.lexists(self._repo.sjoin('lock')):
+        if os.path.lexists(self._repo.svfs.join(b'lock')):
             return True
         return False
 
@@ -225,22 +260,23 @@ class RepoWatcher(QObject):
         repo = self._repo
         q = getattr(repo, 'mq', None)
         newfilesmap = {
-            repo.join('bookmarks'): (LogChanged, False),
-            repo.join('bookmarks.current'): (LogChanged, False),
-            repo.join('branch'): (0, False),
-            repo.join('dirstate'): (WorkingStateChanged, False),
-            repo.join('localtags'): (LogChanged, False),
-            repo.sjoin('00changelog.i'): (LogChanged, False),
-            repo.sjoin('obsstore'): (LogChanged, False),
-            repo.sjoin('phaseroots'): (LogChanged, False),
+            repo.vfs.join(b'bookmarks'): (LogChanged, False),
+            repo.vfs.join(b'bookmarks.current'): (LogChanged, False),
+            repo.vfs.join(b'branch'): (0, False),
+            repo.vfs.join(b'topic'): (LogChanged, False),
+            repo.vfs.join(b'dirstate'): (WorkingStateChanged, False),
+            repo.vfs.join(b'localtags'): (LogChanged, False),
+            repo.svfs.join(b'00changelog.i'): (LogChanged, False),
+            repo.svfs.join(b'obsstore'): (LogChanged, False),
+            repo.svfs.join(b'phaseroots'): (LogChanged, False),
             }
         if q:
             newfilesmap.update({
-                q.join('guards'): (LogChanged, True),
-                q.join('series'): (LogChanged, True),
-                q.join('status'): (LogChanged, True),
-                repo.join('patches.queue'): (LogChanged, True),
-                repo.join('patches.queues'): (LogChanged, True),
+                q.join(b'guards'): (LogChanged, True),
+                q.join(b'series'): (LogChanged, True),
+                q.join(b'status'): (LogChanged, True),
+                repo.vfs.join(b'patches.queue'): (LogChanged, True),
+                repo.vfs.join(b'patches.queues'): (LogChanged, True),
                 })
         newpaths = set(newfilesmap) - set(self._filesmap)
         if not newpaths:
@@ -248,9 +284,9 @@ class RepoWatcher(QObject):
         self._filesmap = newfilesmap
         self._datamap = {
             RepoWatcher._readbranch: (WorkingBranchChanged,
-                                      repo.join('branch')),
+                                      repo.vfs.join(b'branch')),
             RepoWatcher._readparents: (WorkingParentChanged,
-                                       repo.join('dirstate')),
+                                       repo.vfs.join(b'dirstate')),
             }
         newstats, newdata = self._readState(newpaths)
         self._laststats.update(newstats)
@@ -271,43 +307,44 @@ class RepoWatcher(QObject):
                 pass
 
         curdata = {}
-        for readmeth, (_flag, path) in self._datamap.iteritems():
+        for readmeth, (_flag, path) in self._datamap.items():
             if path not in targetpaths:
                 continue
-            last = self._laststats.get(path, -1)
-            cur = curstats.get(path, -1)
+            last = self._laststats.get(path)
+            cur = curstats.get(path)
             if last != cur:
                 try:
                     curdata[readmeth] = readmeth(self)
                 except EnvironmentError:
                     pass
-            elif cur >= 0 and readmeth in self._lastdata:
+            elif cur is not None and readmeth in self._lastdata:
                 curdata[readmeth] = self._lastdata[readmeth]
 
         return curstats, curdata
 
     def _calculateChangeFlags(self, curstats, curdata):
         changeflags = 0
-        for path, (flag, _watched) in self._filesmap.iteritems():
-            last = self._laststats.get(path, -1)
-            cur = curstats.get(path, -1)
+        for path, (flag, _watched) in self._filesmap.items():
+            last = self._laststats.get(path)
+            cur = curstats.get(path)
             if last != cur:
-                self._ui.debug(' stat: %s (%r -> %r)\n' % (path, last, cur))
+                self._ui.debug(b' stat: %s (%r -> %r)\n' % (path, last, cur))
                 changeflags |= flag
-        for readmeth, (flag, _path) in self._datamap.iteritems():
+        for readmeth, (flag, _path) in self._datamap.items():
             last = self._lastdata.get(readmeth)
             cur = curdata.get(readmeth)
             if last != cur:
-                self._ui.debug(' data: %s (%r -> %r)\n'
-                               % (readmeth.__name__, last, cur))
+                self._ui.debug(b' data: %s (%r -> %r)\n'
+                               % (pycompat.sysbytes(readmeth.__name__), last,
+                                  cur))
                 changeflags |= flag
         return changeflags
 
     def _readparents(self):
-        return self._repo.opener('dirstate').read(40)
+        return self._repo.vfs(b'dirstate').read(40)
 
     def _readbranch(self):
-        return self._repo.opener('branch').read()
+        return self._repo.vfs(b'branch').read()
 
     def _checkuimtime(self):
         'Check for modified config files, or a new .hg/hgrc file'
@@ -315,7 +352,7 @@ class RepoWatcher(QObject):
             files = self._repo.uifiles()
             mtime = max(os.path.getmtime(f) for f in files if os.path.isfile(f))
             if mtime > self._uimtime:
-                self._ui.debug('config change detected\n')
+                self._ui.debug(b'config change detected\n')
                 self._uimtime = mtime
                 self.configChanged.emit()
         except (EnvironmentError, ValueError):
@@ -345,7 +382,8 @@ class RepoAgent(QObject):
         # RepoAgent instead of thgrepository.
         repo._pyqtobj = self
         # base repository for bundle or union (set in dispatch._dispatch)
-        repo.ui.setconfig('bundle', 'mainreporoot', repo.root)
+        repo.ui.setconfig(b'bundle', b'mainreporoot', repo.root)
+        self._config = hgconfig.HgConfig(repo.ui)
         # keep url separately from repo.url() because it is abbreviated to
         # relative path to cwd in bundle or union repo
         self._overlayurl = ''
@@ -370,12 +408,12 @@ class RepoAgent(QObject):
         """Start filesystem monitoring on repository open by RepoManager"""
         repo = self._repo
         ui = repo.ui
-        monitorrepo = repo.ui.config('tortoisehg', 'monitorrepo', 'localonly')
+        monitorrepo = self.configString('tortoisehg', 'monitorrepo')
         if monitorrepo == 'never':
-            ui.debug('watching of F/S events is disabled by configuration\n')
+            ui.debug(b'watching of F/S events is disabled by configuration\n')
         elif (monitorrepo == 'localonly'
               and not paths.is_on_fixed_drive(repo.path)):
-            ui.debug('not watching F/S events for network drive\n')
+            ui.debug(b'not watching F/S events for network drive\n')
         else:
             self._watcher.startMonitoring()
 
@@ -411,9 +449,27 @@ class RepoAgent(QObject):
     def rootPath(self):
         return hglib.tounicode(self._repo.root)
 
+    def configBool(self, section, name, default=hgconfig.UNSET_DEFAULT):
+        return self._config.configBool(section, name, default)
+
+    def configInt(self, section, name, default=hgconfig.UNSET_DEFAULT):
+        return self._config.configInt(section, name, default)
+
+    def configString(self, section, name, default=hgconfig.UNSET_DEFAULT):
+        return self._config.configString(section, name, default)
+
+    def configStringList(self, section, name, default=hgconfig.UNSET_DEFAULT):
+        return self._config.configStringList(section, name, default)
+
+    def configStringItems(self, section):
+        return self._config.configStringItems(section)
+
+    def hasConfig(self, section, name):
+        return self._config.hasConfig(section, name)
+
     def displayName(self):
         """Name for window titles and similar"""
-        if self._repo.ui.configbool('tortoisehg', 'fullpath'):
+        if self.configBool('tortoisehg', 'fullpath'):
             return self.rootPath()
         else:
             return self.shortName()
@@ -427,7 +483,7 @@ class RepoAgent(QObject):
             return os.path.basename(self.rootPath())
 
     def hiddenRevsIncluded(self):
-        return self._repo.filtername != 'visible'
+        return self._repo.filtername != b'visible'
 
     def setHiddenRevsIncluded(self, included):
         """Switch visibility of hidden (i.e. pruned) changesets"""
@@ -441,7 +497,7 @@ class RepoAgent(QObject):
 
     def setOverlay(self, url):
         """Switch to bundle or union repository overlaying this"""
-        url = unicode(url)
+        url = pycompat.unicode(url)
         if self._overlayurl == url:
             return
         repo = hg.repository(self._baserepo.ui, hglib.fromunicode(url))
@@ -450,7 +506,7 @@ class RepoAgent(QObject):
         repo = repo.unfiltered()
         repo.__class__ = _extendrepo(repo)
         repo._pyqtobj = self  # TODO: remove repo-to-agent references
-        repo = repo.filtered('visible')
+        repo = repo.filtered(b'visible')
         self._changeRepo(_filteredrepo(repo, self.hiddenRevsIncluded()))
         self._overlayurl = url
         self._watcher.suspendStatusPolling()
@@ -495,6 +551,7 @@ class RepoAgent(QObject):
     @pyqtSlot()
     def _onConfigChanged(self):
         self._repo.invalidateui()
+        self._config = hgconfig.HgConfig(self._repo.ui)
         assert not self._cmdagent.isBusy()
         self._cmdagent.stopService()  # to reload config
         self.configChanged.emit()
@@ -515,6 +572,7 @@ class RepoAgent(QObject):
         self.repositoryDestroyed.emit()
 
     def isBusy(self):
+        # type: () -> bool
         return self._cmdagent.isBusy()
 
     def _preinvalidateCache(self):
@@ -538,16 +596,19 @@ class RepoAgent(QObject):
         self.busyChanged.emit(busy)
 
     def runCommand(self, cmdline, uihandler=None, overlay=True):
+        # type: (List[Text], Optional[Union[QWidget, cmdcore.UiHandler]], bool) -> cmdcore.CmdSession
         """Executes a single command asynchronously in this repository"""
         cmdline = self._extendCmdline(cmdline, overlay)
         return self._cmdagent.runCommand(cmdline, uihandler)
 
     def runCommandSequence(self, cmdlines, uihandler=None, overlay=True):
+        # type: (List[List[Text]], Optional[Union[QWidget, cmdcore.UiHandler]], bool) -> cmdcore.CmdSession
         """Executes a series of commands asynchronously in this repository"""
         cmdlines = [self._extendCmdline(l, overlay) for l in cmdlines]
         return self._cmdagent.runCommandSequence(cmdlines, uihandler)
 
     def _extendCmdline(self, cmdline, overlay):
+        # type: (List[Text], bool) -> List[Text]
         if self.hiddenRevsIncluded():
             cmdline = ['--hidden'] + cmdline
         if overlay and self._overlayurl:
@@ -555,6 +616,7 @@ class RepoAgent(QObject):
         return cmdline
 
     def abortCommands(self):
+        # type: () -> None
         """Abort running and queued commands"""
         self._cmdagent.abortCommands()
 
@@ -578,7 +640,7 @@ class RepoAgent(QObject):
         manager = self.parent()
         if not manager:
             raise RuntimeError('cannot open sub agent of unmanaged repo')
-        assert isinstance(manager, RepoManager)
+        assert isinstance(manager, RepoManager), manager
         self._subrepoagents[path] = agent = manager.openRepoAgent(path)
         return agent
 
@@ -590,7 +652,7 @@ class RepoAgent(QObject):
         manager = self.parent()
         if not manager:
             raise RuntimeError('cannot release sub agents of unmanaged repo')
-        assert isinstance(manager, RepoManager)
+        assert isinstance(manager, RepoManager), manager
         for path in self._subrepoagents:
             manager.releaseRepoAgent(path)
         self._subrepoagents.clear()
@@ -611,14 +673,10 @@ class RepoManager(QObject):
 
     _SIGNALMAP = [
         # source, dest
-        (SIGNAL('configChanged()'),
-         SIGNAL('configChanged(QString)')),
-        (SIGNAL('repositoryDestroyed()'),
-         SIGNAL('repositoryDestroyed(QString)')),
-        (SIGNAL('serviceStopped()'),
-         SLOT('_tryCloseRepoAgent(QString)')),
-        (SIGNAL('busyChanged(bool)'),
-         SLOT('_mapBusyChanged(QString)')),
+        ('configChanged', 'configChanged'),
+        ('repositoryDestroyed', 'repositoryDestroyed'),
+        ('serviceStopped', '_tryCloseRepoAgent'),
+        ('busyChanged', '_mapBusyChanged'),
         ]
 
     def __init__(self, ui, parent=None):
@@ -631,7 +689,7 @@ class RepoManager(QObject):
         for _sig, slot in self._SIGNALMAP:
             mapper = QSignalMapper(self)
             self._sigmappers.append(mapper)
-            QObject.connect(mapper, SIGNAL('mapped(QString)'), self, slot)
+            mapper.mapped[str].connect(getattr(self, slot))
 
     def openRepoAgent(self, path):
         """Return RepoAgent for the specified path and increment refcount"""
@@ -642,12 +700,12 @@ class RepoManager(QObject):
             return agent
 
         # TODO: move repository creation from thgrepo.repository()
-        self._ui.debug('opening repo: %s\n' % hglib.fromunicode(path))
+        self._ui.debug(b'opening repo: %s\n' % hglib.fromunicode(path))
         agent = repository(self._ui, hglib.fromunicode(path))._pyqtobj
         assert agent.parent() is None
         agent.setParent(self)
         for (sig, _slot), mapper in zip(self._SIGNALMAP, self._sigmappers):
-            QObject.connect(agent, sig, mapper, SLOT('map()'))
+            getattr(agent, sig).connect(mapper.map)
             mapper.setMapping(agent, agent.rootPath())
         agent.repositoryChanged.connect(self._mapRepositoryChanged)
         agent.progressReceived.connect(self._mapProgressReceived)
@@ -671,23 +729,23 @@ class RepoManager(QObject):
         agent.releaseSubRepoAgents()
 
         if agent.isServiceRunning():
-            self._ui.debug('stopping service: %s\n' % hglib.fromunicode(path))
+            self._ui.debug(b'stopping service: %s\n' % hglib.fromunicode(path))
             agent.stopService()
         else:
             self._tryCloseRepoAgent(path)
 
     @pyqtSlot(str)
     def _tryCloseRepoAgent(self, path):
-        path = unicode(path)
+        path = pycompat.unicode(path)
         agent, refcount = self._openagents[path]
         if refcount > 0:
             # repo may be reopen before its services stopped
             return
-        self._ui.debug('closing repo: %s\n' % hglib.fromunicode(path))
+        self._ui.debug(b'closing repo: %s\n' % hglib.fromunicode(path))
         del self._openagents[path]
         # TODO: disconnected automatically if _repocache does not exist
         for (sig, _slot), mapper in zip(self._SIGNALMAP, self._sigmappers):
-            QObject.disconnect(agent, sig, mapper, SLOT('map()'))
+            getattr(agent, sig).disconnect(mapper.map)
             mapper.removeMappings(agent)
         agent.repositoryChanged.disconnect(self._mapRepositoryChanged)
         agent.progressReceived.disconnect(self._mapProgressReceived)
@@ -702,23 +760,23 @@ class RepoManager(QObject):
 
     def repoRootPaths(self):
         """Return list of root paths of open repositories"""
-        return self._openagents.keys()
+        return list(self._openagents.keys())
 
-    @qtlib.senderSafeSlot(int)
+    @pyqtSlot(int)
     def _mapRepositoryChanged(self, flags):
         agent = self.sender()
-        assert isinstance(agent, RepoAgent)
+        assert isinstance(agent, RepoAgent), agent
         self.repositoryChanged.emit(agent.rootPath(), flags)
 
     @pyqtSlot(str)
     def _mapBusyChanged(self, path):
-        agent, _refcount = self._openagents[unicode(path)]
+        agent, _refcount = self._openagents[pycompat.unicode(path)]
         self.busyChanged.emit(path, agent.isBusy())
 
-    @qtlib.senderSafeSlot(cmdcore.ProgressMessage)
+    @pyqtSlot(cmdcore.ProgressMessage)
     def _mapProgressReceived(self, progress):
         agent = self.sender()
-        assert isinstance(agent, RepoAgent)
+        assert isinstance(agent, RepoAgent), agent
         self.progressReceived.emit(agent.rootPath(), progress)
 
 
@@ -746,7 +804,7 @@ def _extendrepo(repo):
             if changeid in self.thgmqunappliedpatches:
                 q = self.mq # must have mq to pass the previous if
                 return genPatchContext(self, q.join(changeid), rev=changeid)
-            elif type(changeid) is str and '\0' not in changeid and \
+            elif isinstance(changeid, bytes) and b'\0' not in changeid and \
                     os.path.isabs(changeid) and os.path.isfile(changeid):
                 return genPatchContext(repo, changeid)
 
@@ -768,7 +826,7 @@ def _extendrepo(repo):
 
         @localrepo.unfilteredpropertycache
         def _thghiddentags(self):
-            ht = self.ui.config('tortoisehg', 'hidetags', '')
+            ht = self.ui.config(b'tortoisehg', b'hidetags', b'')
             return [t.strip() for t in ht.split()]
 
         @localrepo.unfilteredpropertycache
@@ -781,7 +839,7 @@ def _extendrepo(repo):
             q = self.mq
             applied = set([p.name for p in q.applied])
 
-            return [pname for pname in q.series if not pname in applied]
+            return [pname for pname in q.series if pname not in applied]
 
         @localrepo.unfilteredpropertycache
         def _thgmqpatchnames(self):
@@ -799,17 +857,17 @@ def _extendrepo(repo):
             cfg = self.ui._ucfg
             files = set()
             for line in cfg._source.values():
-                f = line.rsplit(':', 1)[0]
+                f = line.rsplit(b':', 1)[0]
                 files.add(f)
-            files.add(self.join('hgrc'))
+            files.add(self.vfs.join(b'hgrc'))
             return files
 
         @localrepo.unfilteredpropertycache
         def _exts(self):
             lclexts = []
             allexts = [n for n,m in extensions.extensions()]
-            for name, path in self.ui.configitems('extensions'):
-                if name.startswith('hgext.'):
+            for name, path in self.ui.configitems(b'extensions'):
+                if name.startswith(b'hgext.'):
                     name = name[6:]
                 if name in allexts:
                     lclexts.append(name)
@@ -817,14 +875,14 @@ def _extendrepo(repo):
 
         @localrepo.unfilteredpropertycache
         def postpull(self):
-            pp = self.ui.config('tortoisehg', 'postpull')
-            if pp in ('rebase', 'update', 'fetch', 'updateorrebase'):
+            pp = self.ui.config(b'tortoisehg', b'postpull')
+            if pp in (b'rebase', b'update', b'fetch', b'updateorrebase'):
                 return pp
-            return 'none'
+            return b'none'
 
         @localrepo.unfilteredpropertycache
         def tabwidth(self):
-            tw = self.ui.config('tortoisehg', 'tabwidth')
+            tw = self.ui.config(b'tortoisehg', b'tabwidth')
             try:
                 tw = int(tw)
                 tw = min(tw, 16)
@@ -834,18 +892,18 @@ def _extendrepo(repo):
 
         @localrepo.unfilteredpropertycache
         def maxdiff(self):
-            maxdiff = self.ui.config('tortoisehg', 'maxdiff')
+            maxdiff = self.ui.config(b'tortoisehg', b'maxdiff')
             try:
                 maxdiff = int(maxdiff)
                 if maxdiff < 1:
-                    return sys.maxint
+                    return sys.maxsize
             except (ValueError, TypeError):
                 maxdiff = 1024 # 1MB by default
             return maxdiff * 1024
 
         @localrepo.unfilteredpropertycache
         def summarylen(self):
-            slen = self.ui.config('tortoisehg', 'summarylen')
+            slen = self.ui.config(b'tortoisehg', b'summarylen')
             try:
                 slen = int(slen)
                 if slen < 10:
@@ -856,14 +914,14 @@ def _extendrepo(repo):
 
         @localrepo.unfilteredpropertycache
         def deadbranches(self):
-            db = self.ui.config('tortoisehg', 'deadbranch', '')
-            return [b.strip() for b in db.split(',')]
+            db = self.ui.config(b'tortoisehg', b'deadbranch', b'')
+            return [b.strip() for b in db.split(b',')]
 
         @localrepo.unfilteredpropertycache
         def mergetools(self):
             seen, installed = [], []
-            for key, value in self.ui.configitems('merge-tools'):
-                t = key.split('.')[0]
+            for key, value in self.ui.configitems(b'merge-tools'):
+                t = key.split(b'.')[0]
                 if t not in seen:
                     seen.append(t)
                     if filemerge._findtool(self.ui, t):
@@ -883,7 +941,7 @@ def _extendrepo(repo):
             return tag in self._thgmqpatchnames
 
         def thgshelves(self):
-            self.shelfdir = sdir = self.join('shelves')
+            self.shelfdir = sdir = self.vfs.join(b'shelves')
             if os.path.isdir(sdir):
                 def getModificationTime(x):
                     try:
@@ -904,7 +962,8 @@ def _extendrepo(repo):
 
         def thginvalidate(self):
             'Should be called when mtime of repo store/dirstate are changed'
-            self.dirstate.invalidate()
+            self.invalidatedirstate()
+
             if not isinstance(repo, bundlerepo.bundlerepository):
                 self.invalidate()
             # mq.queue.invalidate does not handle queue changes, so force
@@ -918,40 +977,45 @@ def _extendrepo(repo):
         def invalidateui(self):
             'Should be called when mtime of ui files are changed'
             origui = self.ui
-            self.ui = uimod.ui()
-            self.ui.readconfig(self.join('hgrc'))
+            self.ui = hglib.loadui()
+            self.ui.readconfig(self.vfs.join(b'hgrc'))
             hglib.copydynamicconfig(origui, self.ui)
             for a in _uiprops:
                 if localrepo.hasunfilteredcache(self, a):
                     delattr(self.unfiltered(), a)
 
         def thgbackup(self, path):
-            'Make a backup of the given file in the repository "trashcan"'
-            # The backup name will be the same as the orginal file plus '.bak'
-            trashcan = self.join('Trashcan')
+            '''Make a backup of the given file in the directory "Trashcan" if
+            it exists'''
+            # The backup name will be similar to the orginal file name plus
+            # '.bak' and characters to make it unique
+            trashcan = hglib.tounicode(self.vfs.join(b'Trashcan'))
             if not os.path.isdir(trashcan):
                 os.mkdir(trashcan)
+            path = hglib.tounicode(path)
             if not os.path.exists(path):
                 return
             name = os.path.basename(path)
             root, ext = os.path.splitext(name)
-            dest = tempfile.mktemp(ext+'.bak', root+'_', trashcan)
+            fd, dest = tempfile.mkstemp(suffix=ext + '.bak', prefix=root + '_',
+                                        dir=trashcan)
+            os.close(fd)
             shutil.copyfile(path, dest)
 
         def isStandin(self, path):
-            if 'largefiles' in self.extensions():
+            if b'largefiles' in self.extensions():
                 if _lfregex.match(path):
                     return True
-            if 'largefiles' in self.extensions() or 'kbfiles' in self.extensions():
+            if b'largefiles' in self.extensions() or b'kbfiles' in self.extensions():
                 if _kbfregex.match(path):
                     return True
             return False
 
         def bfStandin(self, path):
-            return '.kbf/' + path
+            return b'.kbf/' + path
 
         def lfStandin(self, path):
-            return '.hglf/' + path
+            return b'.hglf/' + path
 
     return thgrepository
 
@@ -977,7 +1041,7 @@ def _createchangectxcls(parentcls):
                 r = srepo._repo
                 r = r.unfiltered()
                 r.__class__ = _extendrepo(r)
-                srepo._repo = r.filtered('visible')
+                srepo._repo = r.filtered(b'visible')
             return srepo
 
         def thgtags(self):
@@ -1019,20 +1083,21 @@ def _createchangectxcls(parentcls):
 
         def changesToParent(self, whichparent):
             parent = self.parents()[whichparent]
-            return self._repo.status(parent.node(), self.node())[:3]
+            status = self._repo.status(parent.node(), self.node())
+            return status.modified, status.added, status.removed
 
         def longsummary(self):
-            if self._repo.ui.configbool('tortoisehg', 'longsummary'):
+            if self._repo.ui.configbool(b'tortoisehg', b'longsummary'):
                 limit = 80
             else:
                 limit = None
             return hglib.longsummary(self.description(), limit)
 
         def hasStandin(self, file):
-            if 'largefiles' in self._repo.extensions():
+            if b'largefiles' in self._repo.extensions():
                 if self._repo.lfStandin(file) in self.manifest():
                     return True
-            elif 'largefiles' in self._repo.extensions() or 'kbfiles' in self._repo.extensions():
+            elif b'largefiles' in self._repo.extensions() or b'kbfiles' in self._repo.extensions():
                 if self._repo.bfStandin(file) in self.manifest():
                     return True
             return False
@@ -1041,7 +1106,7 @@ def _createchangectxcls(parentcls):
             return self._repo.isStandin(path)
 
         def findStandin(self, file):
-            if 'largefiles' in self._repo.extensions():
+            if b'largefiles' in self._repo.extensions():
                 if self._repo.lfStandin(file) in self.manifest():
                     return self._repo.lfStandin(file)
             return self._repo.bfStandin(file)
