@@ -34,11 +34,13 @@ from mercurial import (
     copies,
     error,
     match,
+    pycompat,
     scmutil,
     util,
 )
 from mercurial.utils import (
     procutil,
+    stringutil,
 )
 
 from ..util import hglib
@@ -46,7 +48,7 @@ from ..util.i18n import _
 from . import qtlib
 
 # Match parent2 first, so 'parent1?' will match both parent1 and parent
-_regex = '\$(parent2|parent1?|child|plabel1|plabel2|clabel|repo|phash1|phash2|chash)'
+_regex = b'\$(parent2|parent1?|child|plabel1|plabel2|clabel|repo|phash1|phash2|chash)'
 
 _nonexistant = _('[non-existant]')
 
@@ -71,17 +73,17 @@ def snapshotset(repo, ctxs, sa, sb, copies, copyworkingdir = False):
     # Always make a copy of ctx1a
     files1a = sources | mod_a | rem_a | ((mod_b | add_b) - add_a)
     dir1a, fns_mtime1a = snapshot(repo, files1a, ctx1a)
-    label1a = '@%d:%s' % (ctx1a.rev(), ctx1a)
+    label1a = b'@%d:%s' % (ctx1a.rev(), ctx1a)
 
     # Make a copy of ctx1b if relevant
     if ctx1b:
         files1b = sources | mod_b | rem_b | ((mod_a | add_a) - add_b)
         dir1b, fns_mtime1b = snapshot(repo, files1b, ctx1b)
-        label1b = '@%d:%s' % (ctx1b.rev(), ctx1b)
+        label1b = b'@%d:%s' % (ctx1b.rev(), ctx1b)
     else:
         dir1b = None
         fns_mtime1b = []
-        label1b = ''
+        label1b = b''
 
     # Either make a copy of ctx2, or use working dir directly if relevant.
     files2 = mod_a | add_a | mod_b | add_b
@@ -92,10 +94,10 @@ def snapshotset(repo, ctxs, sa, sb, copies, copyworkingdir = False):
             dir2 = repo.root
             fns_mtime2 = []
         # If ctx2 is working copy, use empty label.
-        label2 = ''
+        label2 = b''
     else:
         dir2, fns_mtime2 = snapshot(repo, files2, ctx2)
-        label2 = '@%d:%s' % (ctx2.rev(), ctx2)
+        label2 = b'@%d:%s' % (ctx2.rev(), ctx2)
 
     dirs = [dir1a, dir1b, dir2]
     labels = [label1a, label1b, label2]
@@ -103,18 +105,21 @@ def snapshotset(repo, ctxs, sa, sb, copies, copyworkingdir = False):
     return dirs, labels, fns_and_mtimes
 
 def snapshot(repo, files, ctx):
-    '''snapshot files as of some revision'''
-    dirname = os.path.basename(repo.root) or 'root'
-    dirname += '.%d' % _diffCount
+    '''snapshot repo files as of some revision, returning a tuple with the
+    created temporary snapshot dir and tuples of file info if using working
+    copy.'''
+    dirname = os.path.basename(repo.root) or b'root'
+    dirname += b'.%d' % _diffCount
     if ctx.rev() is not None:
-        dirname += '.%d' % ctx.rev()
+        dirname += b'.%d' % ctx.rev()
     base = os.path.join(qtlib.gettempdir(), dirname)
     fns_and_mtime = []
     if not os.path.exists(base):
         os.makedirs(base)
     for fn in files:
+        assert isinstance(fn, bytes), repr(fn)
         wfn = util.pconvert(fn)
-        if not wfn in ctx:
+        if wfn not in ctx:
             # File doesn't exist; could be a bogus modify
             continue
         dest = os.path.join(base, wfn)
@@ -127,10 +132,9 @@ def snapshot(repo, files, ctx):
                 os.makedirs(destdir)
             fctx = ctx[wfn]
             data = repo.wwritedata(wfn, fctx.data())
-            f = open(dest, 'wb')
-            f.write(data)
-            f.close()
-            if 'x' in fctx.flags():
+            with open(dest, 'wb') as f:
+                f.write(data)
+            if b'x' in fctx.flags():
                 util.setflags(dest, False, True)
             if ctx.rev() is None:
                 fns_and_mtime.append((dest, repo.wjoin(fn), 
@@ -144,32 +148,31 @@ def snapshot(repo, files, ctx):
 
 def launchtool(cmd, opts, replace, block):
     def quote(match):
-        key = match.group()[1:]
+        key = pycompat.sysstr(match.group()[1:])
         return procutil.shellquote(replace[key])
-    if isinstance(cmd, unicode):
+    if isinstance(cmd, pycompat.unicode):
         cmd = hglib.fromunicode(cmd)
     lopts = []
     for opt in opts:
-        if isinstance(opt, unicode):
+        if isinstance(opt, pycompat.unicode):
             lopts.append(hglib.fromunicode(opt))
         else:
             lopts.append(opt)
-    args = ' '.join(lopts)
+    args = b' '.join(lopts)
     args = re.sub(_regex, quote, args)
-    cmdline = procutil.shellquote(cmd) + ' ' + args
-    cmdline = procutil.quotecommand(cmdline)
+    cmdline = procutil.shellquote(cmd) + b' ' + args
     try:
-        proc = subprocess.Popen(cmdline, shell=True,
+        proc = subprocess.Popen(procutil.tonativestr(cmdline), shell=True,
                                 creationflags=qtlib.openflags,
                                 stderr=subprocess.PIPE,
                                 stdout=subprocess.PIPE,
                                 stdin=subprocess.PIPE)
         if block:
             proc.communicate()
-    except (OSError, EnvironmentError), e:
+    except (OSError, EnvironmentError) as e:
         QMessageBox.warning(None,
                 _('Tool launch failure'),
-                _('%s : %s') % (cmd, str(e)))
+                _('%s : %s') % (hglib.tounicode(cmd), hglib.tounicode(str(e))))
 
 def filemerge(ui, fname, patchedfname):
     'Launch the preferred visual diff tool for two text files'
@@ -190,16 +193,16 @@ def filemerge(ui, fname, patchedfname):
 
 def besttool(ui, tools, force=None):
     'Select preferred or highest priority tool from dictionary'
-    preferred = force or ui.config('tortoisehg', 'vdiff') or \
-                         ui.config('ui', 'merge')
+    preferred = force or ui.config(b'tortoisehg', b'vdiff') or \
+                         ui.config(b'ui', b'merge')
     if preferred and preferred in tools:
         return preferred
     pris = []
     for t in tools.keys():
         try:
-            p = ui.configint('merge-tools', t + '.priority', 0)
-        except error.ConfigError, inst:
-            ui.warn('visdiff: %s\n' % inst)
+            p = ui.configint(b'merge-tools', t + b'.priority')
+        except error.ConfigError as inst:
+            ui.warn(b'visdiff: %s\n' % stringutil.forcebytestr(inst))
             p = 0
         pris.append((-p, t))
     tools = sorted(pris)
@@ -214,7 +217,9 @@ def visualdiff(ui, repo, pats, opts):
         ctx1b = None
         if change:
             # TODO: figure out what's the expect type
-            if isinstance(change,  str):
+            if isinstance(change, pycompat.unicode):
+                change = hglib.fromunicode(change)
+            if isinstance(change, bytes):
                 ctx2 = scmutil.revsymbol(repo, change)
             else:
                 ctx2 = repo[change]
@@ -224,7 +229,8 @@ def visualdiff(ui, repo, pats, opts):
             else:
                 ctx1a = p[0]
         else:
-            n1, n2 = scmutil.revpair(repo, revs)
+            n1, n2 = scmutil.revpair(repo, [hglib.fromunicode(rev)
+                                            for rev in revs])
             ctx1a, ctx2 = repo[n1], repo[n2]
             p = ctx2.parents()
             if not revs and len(p) > 1:
@@ -236,12 +242,19 @@ def visualdiff(ui, repo, pats, opts):
         return None
 
     pats = scmutil.expandpats(pats)
-    m = match.match(repo.root, '', pats, None, None, 'relpath')
+    m = match.match(repo.root, b'', pats, None, None, b'relpath')
     n2 = ctx2.node()
-    mod_a, add_a, rem_a = map(set, repo.status(ctx1a.node(), n2, m)[:3])
+
+    def _status(ctx):
+        status = repo.status(ctx.node(), n2, m)
+        return status.modified, status.added, status.removed
+
+    mod_a, add_a, rem_a = pycompat.maplist(set, _status(ctx1a))
     if ctx1b:
-        mod_b, add_b, rem_b = map(set, repo.status(ctx1b.node(), n2, m)[:3])
+        mod_b, add_b, rem_b = pycompat.maplist(set, _status(ctx1b))
         cpy = copies.mergecopies(repo, ctx1a, ctx1b, ctx1a.ancestor(ctx1b))[0]
+        if util.safehasattr(copies, "branch_copies"):
+            cpy = cpy.copy  # hg 5.4 >= 7f8bdee0034e
     else:
         cpy = copies.pathcopies(ctx1a, ctx2)
         mod_b, add_b, rem_b = set(), set(), set()
@@ -264,32 +277,32 @@ def visualdiff(ui, repo, pats, opts):
 
     # Build tool list based on diff-patterns matches
     toollist = set()
-    patterns = repo.ui.configitems('diff-patterns')
+    patterns = repo.ui.configitems(b'diff-patterns')
     patterns = [(p, t) for p,t in patterns if t in detectedtools]
     for path in MAR:
         for pat, tool in patterns:
-            mf = match.match(repo.root, '', [pat])
+            mf = match.match(repo.root, b'', [pat])
             if mf(path):
                 toollist.add(tool)
                 break
         else:
             toollist.add(preferred)
 
-    cto = cpy.keys()
+    cto = list(cpy.keys())
     for path in MAR:
         if path in cto:
             hascopies = True
             break
     else:
         hascopies = False
-    force = repo.ui.configbool('tortoisehg', 'forcevdiffwin')
+    force = repo.ui.configbool(b'tortoisehg', b'forcevdiffwin')
     if len(toollist) > 1 or (hascopies and len(MAR) > 1) or force:
         usewin = True
     else:
         preferred = toollist.pop()
-        dirdiff = repo.ui.configbool('merge-tools', preferred + '.dirdiff')
-        dir3diff = repo.ui.configbool('merge-tools', preferred + '.dir3diff')
-        usewin = repo.ui.configbool('merge-tools', preferred + '.usewin')
+        dirdiff = repo.ui.configbool(b'merge-tools', preferred + b'.dirdiff')
+        dir3diff = repo.ui.configbool(b'merge-tools', preferred + b'.dir3diff')
+        usewin = repo.ui.configbool(b'merge-tools', preferred + b'.usewin')
         if not usewin and len(MAR) > 1:
             if ctx1b is not None:
                 usewin = not dir3diff
@@ -334,15 +347,15 @@ def visualdiff(ui, repo, pats, opts):
         label1a, label1b, label2 = labels
         fns_and_mtime = fns_and_mtimes[2]
 
-        if len(MAR) > 1 and label2 == '':
-            label2 = 'working files'
+        if len(MAR) > 1 and label2 == b'':
+            label2 = b'working files'
 
         def getfile(fname, dir, label):
             file = os.path.join(qtlib.gettempdir(), dir, fname)
             if os.path.isfile(file):
                 return fname+label, file
-            nullfile = os.path.join(qtlib.gettempdir(), 'empty')
-            fp = open(nullfile, 'w')
+            nullfile = os.path.join(qtlib.gettempdir(), b'empty')
+            fp = open(nullfile, 'wb')
             fp.close()
             return (hglib.fromunicode(_nonexistant, 'replace') + label,
                     nullfile)
@@ -361,9 +374,9 @@ def visualdiff(ui, repo, pats, opts):
                 label1b, dir1b = getfile(file1, dir1b, label1b)
             label2, dir2 = getfile(file2local, dir2, label2)
         if do3way:
-            label1a += '[local]'
-            label1b += '[other]'
-            label2 += '[merged]'
+            label1a += b'[local]'
+            label1b += b'[other]'
+            label2 += b'[merged]'
 
         repoagent = repo._pyqtobj  # TODO
         replace = dict(parent=dir1a, parent1=dir1a, parent2=dir1b,
@@ -377,8 +390,9 @@ def visualdiff(ui, repo, pats, opts):
         for copy_fn, working_fn, mtime in fns_and_mtime:
             try:
                 if os.lstat(copy_fn).st_mtime != mtime:
-                    ui.debug('file changed while diffing. '
-                            'Overwriting: %s (src: %s)\n' % (working_fn, copy_fn))
+                    ui.debug(b'file changed while diffing. '
+                             b'Overwriting: %s (src: %s)\n'
+                             % (working_fn, copy_fn))
                     util.copyfile(copy_fn, working_fn)
             except EnvironmentError:
                 pass # Ignore I/O errors or missing files
@@ -388,7 +402,7 @@ def visualdiff(ui, repo, pats, opts):
             dodiff()
         finally:
             # cleanup happens atexit
-            ui.note('cleaning up temp directory\n')
+            ui.note(b'cleaning up temp directory\n')
 
     if opts.get('mainapp'):
         dodiffwrapper()
@@ -455,8 +469,8 @@ class FileSelectionDialog(QDialog):
             hbox.addWidget(lbl)
             hbox.addWidget(combo, 1)
             layout.addLayout(hbox)
-            for i, name in enumerate(tools.iterkeys()):
-                combo.addItem(name)
+            for i, name in enumerate(tools.keys()):
+                combo.addItem(hglib.tounicode(name))
                 if name == preferred:
                     defrow = i
             combo.setCurrentIndex(defrow)
@@ -514,7 +528,7 @@ class FileSelectionDialog(QDialog):
     def onToolSelected(self, tool):
         'user selected a tool from the tool combo'
         tool = hglib.fromunicode(tool)
-        assert tool in self.tools
+        assert tool in self.tools, tool
         self.diffpath, self.diffopts, self.mergeopts = self.tools[tool]
         self.updateDiffButtons(tool)
 
@@ -525,7 +539,7 @@ class FileSelectionDialog(QDialog):
             return
 
         repo = self.repo
-        patterns = repo.ui.configitems('diff-patterns')
+        patterns = repo.ui.configitems(b'diff-patterns')
         patterns = [(p, t) for p,t in patterns if t in self.tools]
 
         fname = self.list.item(row).text()[2:]
@@ -534,13 +548,13 @@ class FileSelectionDialog(QDialog):
             return
         self.curFile = fname
         for pat, tool in patterns:
-            mf = match.match(repo.root, '', [pat])
+            mf = match.match(repo.root, b'', [pat])
             if mf(fname):
                 selected = tool
                 break
         else:
             selected = self.preferred
-        for i, name in enumerate(self.tools.iterkeys()):
+        for i, name in enumerate(self.tools.keys()):
             if name == selected:
                 self.toolCombo.setCurrentIndex(i)
 
@@ -557,13 +571,13 @@ class FileSelectionDialog(QDialog):
     def updateDiffButtons(self, tool):
         # hg>=4.4: configbool() may return None as the default is set to None
         if hasattr(self, 'p1button'):
-            d2 = self.repo.ui.configbool('merge-tools', tool + '.dirdiff')
-            d3 = self.repo.ui.configbool('merge-tools', tool + '.dir3diff')
+            d2 = self.repo.ui.configbool(b'merge-tools', tool + b'.dirdiff')
+            d3 = self.repo.ui.configbool(b'merge-tools', tool + b'.dir3diff')
             self.p1button.setEnabled(bool(d2))
             self.p2button.setEnabled(bool(d2))
             self.p3button.setEnabled(bool(d3))
         elif hasattr(self, 'dbutton'):
-            d2 = self.repo.ui.configbool('merge-tools', tool + '.dirdiff')
+            d2 = self.repo.ui.configbool(b'merge-tools', tool + b'.dirdiff')
             self.dbutton.setEnabled(bool(d2))
 
     def launch(self, fname):
@@ -582,7 +596,7 @@ class FileSelectionDialog(QDialog):
                 path = os.path.join(dir, util.localpath(source))
                 return source, path
             else:
-                nullfile = os.path.join(qtlib.gettempdir(), 'empty')
+                nullfile = os.path.join(qtlib.gettempdir(), b'empty')
                 fp = open(nullfile, 'w')
                 fp.close()
                 return hglib.fromunicode(_nonexistant, 'replace'), nullfile
@@ -598,9 +612,9 @@ class FileSelectionDialog(QDialog):
         label1b = other+rev1b
         label2 = fname+rev2
         if ctx1b:
-            label1a += '[local]'
-            label1b += '[other]'
-            label2 += '[merged]'
+            label1a += b'[local]'
+            label1b += b'[other]'
+            label2 += b'[merged]'
 
         # Function to quote file/dir names in the argument string
         replace = dict(parent=file1a, parent1=file1a, plabel1=label1a,
