@@ -10,13 +10,13 @@
 
 from mercurial import error
 
-def precursorsmarkers(obsstore, node):
-    return obsstore.precursors.get(node, ())
+def predecessorsmarkers(obsstore, node):
+    return obsstore.predecessors.get(node, ())
 
 def successorsmarkers(obsstore, node):
     return obsstore.successors.get(node, ())
 
-def first_known_precursors_rev(repo, rev):
+def first_known_predecessors_rev(repo, rev):
     if rev is None or not isinstance(rev, int):
         return
 
@@ -25,9 +25,15 @@ def first_known_precursors_rev(repo, rev):
         return
 
     clog = repo.changelog
-    nm = clog.nodemap
+
+    try:
+        # Added in 5.3
+        get_rev = clog.index.get_rev
+    except AttributeError:
+        get_rev = clog.nodemap.get
+
     start = clog.node(rev)
-    markers = precursorsmarkers(obsstore, start)
+    markers = predecessorsmarkers(obsstore, start)
     candidates = set(mark[0] for mark in markers)
     seen = set(candidates)
     if start in candidates:
@@ -36,7 +42,7 @@ def first_known_precursors_rev(repo, rev):
         seen.add(start)
     while candidates:
         current = candidates.pop()
-        crev = nm.get(current)
+        crev = get_rev(current)
         if crev is not None:
             try:
                 repo[crev]  # filter out filtered revisions
@@ -44,22 +50,27 @@ def first_known_precursors_rev(repo, rev):
                 continue
             except error.RepoLookupError:
                 pass
-        for mark in precursorsmarkers(obsstore, current):
+        for mark in predecessorsmarkers(obsstore, current):
             if mark[0] not in seen:
                 candidates.add(mark[0])
                 seen.add(mark[0])
 
-def first_known_precursors(ctx):
-    for rev in first_known_precursors_rev(ctx._repo, ctx.rev()):
+def first_known_predecessors(ctx):
+    for rev in first_known_predecessors_rev(ctx._repo, ctx.rev()):
         yield ctx._repo[rev]
 
 def first_known_successors(ctx):
     obsstore = getattr(ctx._repo, 'obsstore', None)
     startnode = ctx.node()
-    nm = ctx._repo.changelog.nodemap
+    try:
+        # Added in 5.3
+        get_rev = ctx._repo.changelog.index.get_rev
+    except AttributeError:
+        get_rev = ctx._repo.changelog.nodemap.get
+
     if obsstore is not None:
         markers = successorsmarkers(obsstore, startnode)
-        # consider all precursors
+        # consider all predecessors
         candidates = set()
         for mark in markers:
             candidates.update(mark[1])
@@ -71,7 +82,7 @@ def first_known_successors(ctx):
         while candidates:
             current = candidates.pop()
             # is this changeset in the displayed set ?
-            crev = nm.get(current)
+            crev = get_rev(current)
             if crev is not None:
                 try:
                     yield ctx._repo[crev]
