@@ -5,14 +5,23 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
+from __future__ import print_function
+
 import os
 import binascii
-import cStringIO
 
-from mercurial import patch, util, error
-from mercurial import node
+from mercurial import (
+    encoding,
+    error,
+    node,
+    patch,
+    pycompat,
+    util,
+)
+
 from mercurial.utils import (
     dateutil,
+    stringutil,
 )
 from mercurial.util import propertycache
 from hgext import mq
@@ -31,22 +40,22 @@ class patchctx(object):
         """
         self._path = patchpath
         if rev:
-            assert isinstance(rev, str)
+            assert isinstance(rev, bytes), repr(rev)
             self._patchname = rev
         else:
             self._patchname = os.path.basename(patchpath)
         self._repo = repo
-        self._rev = rev or 'patch'
+        self._rev = rev or b'patch'
         self._status = [[], [], []]
         self._fileorder = []
-        self._user = ''
-        self._desc = ''
-        self._branch = ''
+        self._user = b''
+        self._desc = b''
+        self._branch = b''
         self._node = node.nullid
         self._mtime = None
         self._fsize = 0
         self._parseerror = None
-        self._phase = 'draft'
+        self._phase = b'draft'
 
         try:
             self._mtime = os.path.getmtime(patchpath)
@@ -58,23 +67,23 @@ class patchctx(object):
             return
 
         try:
-            self._branch = ph.branch or ''
+            self._branch = ph.branch or b''
             self._node = binascii.unhexlify(ph.nodeid)
-            if self._repo.ui.configbool('mq', 'secret'):
-                self._phase = 'secret'
+            if self._repo.ui.configbool(b'mq', b'secret'):
+                self._phase = b'secret'
         except TypeError:
             pass
         except AttributeError:
             # hacks to try to deal with older versions of mq.py
-            self._branch = ''
+            self._branch = b''
             ph.diffstartline = len(ph.comments)
             if ph.message:
                 ph.diffstartline += 1
         except error.ConfigError:
             pass
 
-        self._user = ph.user or ''
-        self._desc = ph.message and '\n'.join(ph.message).strip() or ''
+        self._user = ph.user or b''
+        self._desc = ph.message and b'\n'.join(ph.message).strip() or b''
         date = None
         if ph.date:
             try:
@@ -100,9 +109,13 @@ class patchctx(object):
     def __iter__(self):
         return iter(sorted(self._files))
 
-    def __str__(self):      return node.short(self.node())
+    def __bytes__(self):
+        return node.short(self.node())
+
+    __str__ = encoding.strmethod(__bytes__)
+
     def node(self):         return self._node
-    def files(self):        return self._files.keys()
+    def files(self):        return list(self._files.keys())
     def rev(self):          return self._rev
     def hex(self):          return node.hex(self.node())
     def user(self):         return self._user
@@ -110,6 +123,7 @@ class patchctx(object):
     def description(self):  return self._desc
     def branch(self):       return self._branch
     def parents(self):      return ()
+    def repo(self):         return self._repo
     def tags(self):         return ()
     def bookmarks(self):    return ()
     def children(self):     return ()
@@ -126,24 +140,24 @@ class patchctx(object):
 
     def flags(self, wfile):
         if wfile == self._parseErrorFileName:
-            return ''
+            return b''
         if wfile in self._files:
             for gp in patch.readgitpatch(self._files[wfile][0].header):
                 if gp.mode:
                     islink, isexec = gp.mode
                     if islink:
-                        return 'l'
+                        return b'l'
                     elif wfile in self._status[1]:
                         # Do not report exec mode change if file is added
-                        return ''
+                        return b''
                     elif isexec:
-                        return 'x'
+                        return b'x'
                     else:
                         # techincally, this case could mean the file has had its
                         # exec bit cleared OR its symlink state removed
                         # TODO: change readgitpatch() to differentiate
-                        return '-'
-        return ''
+                        return b'-'
+        return b''
 
     # TortoiseHg methods
     def thgtags(self):              return []
@@ -156,7 +170,7 @@ class patchctx(object):
     def isStandin(self, path):      return False
 
     def longsummary(self):
-        if self._repo.ui.configbool('tortoisehg', 'longsummary'):
+        if self._repo.ui.configbool(b'tortoisehg', b'longsummary'):
             limit = 80
         else:
             limit = None
@@ -178,13 +192,14 @@ class patchctx(object):
     def thgmqpatchdata(self, wfile):
         'called by fileview to get diff data'
         if wfile == self._parseErrorFileName:
-            return '\n\n\nErrors while parsing patch:\n'+str(self._parseerror)
+            return (b'\n\n\nErrors while parsing patch:\n'
+                    + stringutil.forcebytestr(self._parseerror))
         if wfile in self._files:
-            buf = cStringIO.StringIO()
+            buf = pycompat.bytesio()
             for chunk in self._files[wfile]:
                 chunk.write(buf)
             return buf.getvalue()
-        return ''
+        return b''
 
     def phasestr(self):
         return self._phase
@@ -199,12 +214,12 @@ class patchctx(object):
 
         M, A, R = 0, 1, 2
         def get_path(a, b):
-            type = (a == '/dev/null') and A or M
-            type = (b == '/dev/null') and R or type
-            rawpath = (b != '/dev/null') and b or a
-            if not (rawpath.startswith('a/') or rawpath.startswith('b/')):
+            type = (a == b'/dev/null') and A or M
+            type = (b == b'/dev/null') and R or type
+            rawpath = (b != b'/dev/null') and b or a
+            if not (rawpath.startswith(b'a/') or rawpath.startswith(b'b/')):
                 return type, rawpath
-            return type, rawpath.split('/', 1)[-1]
+            return type, rawpath.split(b'/', 1)[-1]
 
         files = {}
         pf = open(self._path, 'rb')
@@ -225,12 +240,12 @@ class patchctx(object):
                     files[path] = [chunk]
                     self._fileorder.append(path)
                 files[path].extend(chunk.hunks)
-        except (patch.PatchError, AttributeError), e:
+        except (patch.PatchError, AttributeError) as e:
             self._status[2].append(self._parseErrorFileName)
             files[self._parseErrorFileName] = []
             self._parseerror = e
             if 'THGDEBUG' in os.environ:
-                print e
+                print(e)
         finally:
             pf.close()
         return files
