@@ -5,20 +5,51 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2, incorporated herein by reference.
 
+from __future__ import absolute_import, print_function
+
+import os
+import shlex
+import sys
+
+import mercurial
+from mercurial import (
+    encoding,
+    pycompat,
+)
+
 try:
-    from tortoisehg.util.config import (icon_path, bin_path, license_path,
-                                        locale_path)
+    # TODO: These are generated a str on non Windows platforms, but can be
+    #       filesystem encoded on Windows.
+    from .config import (
+        bin_path,
+        icon_path,
+        license_path,
+        locale_path,
+    )
 except ImportError:
     icon_path, bin_path, license_path, locale_path = None, None, None, None
 
-import os, sys
-import mercurial
-
 _hg_command = None
 
-def find_root(path=None):
-    p = path or os.getcwd()
-    while not os.path.isdir(os.path.join(p, ".hg")):
+if pycompat.TYPE_CHECKING:
+    from typing import (
+        List,
+        Optional,
+        overload,
+    )
+
+    @overload
+    def _find_root(p, dn):
+        # type: (str, str) -> str
+        pass
+    @overload
+    def _find_root(p, dn):
+        # type: (bytes, bytes) -> bytes
+        pass
+
+
+def _find_root(p, dn):
+    while not os.path.isdir(os.path.join(p, dn)):
         oldp = p
         p = os.path.dirname(p)
         if p == oldp:
@@ -27,41 +58,63 @@ def find_root(path=None):
             return None
     return p
 
+def find_root(path=None):
+    # type: (Optional[str]) -> str
+    return _find_root(path or os.getcwd(), '.hg')
+
+def find_root_bytes(path=None):
+    # type: (Optional[bytes]) -> bytes
+    return _find_root(path or encoding.getcwd(), b'.hg')
+
 def get_tortoise_icon(icon):
+    # type: (str) -> Optional[str]
     "Find a tortoisehg icon"
     icopath = os.path.join(get_icon_path(), icon)
     if os.path.isfile(icopath):
         return icopath
     else:
-        print 'icon not found', icon
+        print('icon not found', icon)
         return None
 
 def get_icon_path():
+    # type: () -> str
     global icon_path
     return icon_path or os.path.join(get_prog_root(), 'icons')
 
 def get_license_path():
+    # type: () -> str
     global license_path
     return license_path or os.path.join(get_prog_root(), 'COPYING.txt')
 
 def get_locale_path():
+    # type: () -> str
     global locale_path
     return locale_path or os.path.join(get_prog_root(), 'locale')
 
 def _get_hg_path():
+    # type: () -> str
     return os.path.abspath(os.path.join(mercurial.__file__, '..', '..'))
 
 def get_hg_command():
+    # type: () -> List[str]
     """List of command to execute hg (equivalent to mercurial.util.hgcmd)"""
     global _hg_command
     if _hg_command is None:
-        _hg_command = _find_hg_command()
+        if 'HG' in os.environ:
+            try:
+                _hg_command = shlex.split(os.environ['HG'],
+                                          posix=(os.name != 'nt'))
+            except ValueError:
+                _hg_command = [os.environ['HG']]
+        else:
+            _hg_command = _find_hg_command()
     return _hg_command
 
 if os.name == 'nt':
     import win32file
 
     def find_in_path(pgmname):
+        # type: (str) -> Optional[str]
         "return first executable found in search path"
         global bin_path
         ospath = os.environ['PATH'].split(os.pathsep)
@@ -76,6 +129,7 @@ if os.name == 'nt':
         return None
 
     def _find_hg_command():
+        # type: () -> List[str]
         if hasattr(sys, 'frozen'):
             progdir = get_prog_root()
             exe = os.path.join(progdir, 'hg.exe')
@@ -101,25 +155,36 @@ if os.name == 'nt':
         return [exe]
 
     def get_prog_root():
+        # type: () -> str
         if getattr(sys, 'frozen', False):
             return os.path.dirname(sys.executable)
         return os.path.dirname(os.path.dirname(os.path.dirname(__file__)))
 
     def get_thg_command():
+        # type: () -> List[str]
         if getattr(sys, 'frozen', False):
             return [sys.executable]
         return [sys.executable] + sys.argv[:1]
 
     def is_unc_path(path):
-        unc, rest = os.path.splitunc(path)
-        return bool(unc)
+        # type: (bytes) -> bool
+        # splitdrive() on py2 didn't get UNC support until 2.7.8 in 2014, which
+        # means it may not be available on CentOS 7, for example.
+        if pycompat.ispy3:
+            unc, rest = os.path.splitdrive(path)
+            return len(unc) > 2
+        else:
+            unc, rest = os.path.splitunc(path)
+            return bool(unc)
 
     def is_on_fixed_drive(path):
+        # type: (bytes) -> bool
         if is_unc_path(path):
             # All UNC paths (\\host\mount) are considered not-fixed
             return False
         drive, remain = os.path.splitdrive(path)
         if drive:
+            drive = pycompat.fsdecode(drive)
             return win32file.GetDriveType(drive) == win32file.DRIVE_FIXED
         else:
             return False
@@ -127,6 +192,7 @@ if os.name == 'nt':
 else: # Not Windows
 
     def find_in_path(pgmname):
+        # type: (str) -> Optional[str]
         """ return first executable found in search path """
         global bin_path
         ospath = os.environ['PATH'].split(os.pathsep)
@@ -138,6 +204,7 @@ else: # Not Windows
         return None
 
     def _find_hg_command():
+        # type: () -> List[str]
         # look for in-place build, i.e. "make local"
         exe = os.path.join(_get_hg_path(), 'hg')
         if os.path.exists(exe):
@@ -149,15 +216,19 @@ else: # Not Windows
         return [exe]
 
     def get_prog_root():
+        # type: () -> str
         path = os.path.dirname(os.path.dirname(os.path.dirname(__file__)))
         return path
 
     def get_thg_command():
+        # type: () -> List[str]
         return sys.argv[:1]
 
     def is_unc_path(path):
+        # type: (bytes) -> bool
         return False
 
     def is_on_fixed_drive(path):
+        # type: (bytes) -> bool
         return True
 
