@@ -7,10 +7,23 @@
 # GNU General Public License version 2, incorporated herein by reference.
 
 import os
-import sys
 import time
-import threading
-from mercurial import hg
+
+from hgext.largefiles import (
+    lfutil,
+)
+
+from mercurial import (
+    hg,
+    pycompat,
+)
+
+if pycompat.TYPE_CHECKING:
+    from typing import (
+        Dict,
+        List,
+        Optional,
+    )
 
 def get_system_times():
     t = os.times()
@@ -19,19 +32,8 @@ def get_system_times():
     return t
 
 if os.name == 'nt':
-    def browse_url(url):
-        try:
-            import win32api
-        except ImportError:
-            return
-        def start_browser():
-            try:
-                win32api.ShellExecute(0, 'open', url, None, None, 0)
-            except Exception:
-                pass
-        threading.Thread(target=start_browser).start()
-
     def shell_notify(paths, noassoc=False):
+        # type: (List[bytes], bool) -> None
         try:
             from win32com.shell import shell, shellcon
             import pywintypes
@@ -48,7 +50,8 @@ if os.name == 'nt':
         # send notifications to deepest directories first
         for dir in sorted(dirs, key=len, reverse=True):
             try:
-                pidl, ignore = shell.SHILCreateFromPath(dir, 0)
+                pidl, ignore = shell.SHILCreateFromPath(
+                    pycompat.fsdecode(dir), 0)
             except pywintypes.com_error:
                 return
             if pidl is None:
@@ -64,7 +67,7 @@ if os.name == 'nt':
     def update_thgstatus(ui, root, wait=False):
         '''Rewrite the file .hg/thgstatus
 
-        Caches the information provided by repo.status() in the file 
+        Caches the information provided by repo.status() in the file
         .hg/thgstatus, which can then be read by the overlay shell extension
         to display overlay icons for directories.
 
@@ -89,63 +92,53 @@ if os.name == 'nt':
         if wait:
             tref = time.time()
             tdelta = float(int(tref)) + 1.0 - tref
-            if (tdelta > 0.0):
+            if tdelta > 0.0:
                 time.sleep(tdelta)
 
         repo = hg.repository(ui, root) # a fresh repo object is needed
-        repo.lfstatus = True
-        repostate = repo.status() # will update .hg/dirstate as a side effect
-        repo.lfstatus = False
-        modified, added, removed, deleted = repostate[:4]
+        with lfutil.lfstatus(repo):
+            repostate = repo.status() # will update dirstate as a side effect
 
-        dirstatus = {}
+        dirstatus = {}  # type: Dict[bytes, bytes]
         def dirname(f):
-            return '/'.join(f.split('/')[:-1])
-        for fn in added:
-            dirstatus[dirname(fn)] = 'a'
-        for fn in modified:
-            dirstatus[dirname(fn)] = 'm'
-        for fn in removed + deleted:
-            dirstatus[dirname(fn)] = 'r'
+            # type: (bytes) -> bytes
+            return b'/'.join(f.split(b'/')[:-1])
+        for fn in repostate.added:
+            dirstatus[dirname(fn)] = b'a'
+        for fn in repostate.modified:
+            dirstatus[dirname(fn)] = b'm'
+        for fn in repostate.removed + repostate.deleted:
+            dirstatus[dirname(fn)] = b'r'
 
         update = False
-        f = None
+
         try:
-            f = repo.vfs('thgstatus', 'rb')
-            for dn in sorted(dirstatus):
-                s = dirstatus[dn]
-                e = f.readline()
-                if e.startswith('@@noicons'):
-                    break
-                if e == '' or e[0] != s or e[1:-1] != dn:
+            with repo.vfs(b'thgstatus', b'rb') as f:
+                for dn in sorted(dirstatus):
+                    s = dirstatus[dn]
+                    e = f.readline()  # type: bytes
+                    if e.startswith(b'@@noicons'):
+                        break
+                    if e == b'' or e[0] != s or e[1:-1] != dn:
+                        update = True
+                        break
+                if f.readline() != b'':
+                    # extra line in f, needs update
                     update = True
-                    break
-            if f.readline() != '':
-                # extra line in f, needs update
-                update = True
         except IOError:
             update = True
-        finally:
-            if f != None:
-                f.close()
 
         if update:
-            f = repo.vfs('thgstatus', 'wb', atomictemp=True)
-            for dn in sorted(dirstatus):
-                s = dirstatus[dn]
-                f.write(s + dn + '\n')
-                ui.note("%s %s\n" % (s, dn))
-            if hasattr(f, 'rename'):
-                # On Mercurial 1.9 and earlier, there was a rename() function
-                # that served the purpose now served by close(), while close()
-                # served the purpose now served by discard().
-                f.rename()
-            else:
-                f.close()
+            with repo.vfs(b'thgstatus', b'wb', atomictemp=True) as f:
+                for dn in sorted(dirstatus):
+                    s = dirstatus[dn]
+                    f.write(s + dn + b'\n')
+                    ui.note(b"%s %s\n" % (s, dn))
         return update
 
 else:
     def shell_notify(paths, noassoc=False):
+        # type: (List[bytes], bool) -> None
         if not paths:
             return
         notify = os.environ.get('THG_NOTIFY', '.tortoisehg/notify')
@@ -155,33 +148,14 @@ else:
         if not os.path.isfile(notify):
             return
         try:
-            f_notify = open(notify, 'w')
+            f_notify = open(notify, 'wb')
         except IOError:
             return
         try:
             abspaths = [os.path.abspath(path) for path in paths if path]
-            f_notify.write('\n'.join(abspaths))
+            f_notify.write(b'\n'.join(abspaths))
         finally:
             f_notify.close()
 
     def update_thgstatus(*args, **kws):
         pass
-
-    def browse_url(url):
-        def start_browser():
-            if sys.platform == 'darwin':
-                # use Mac OS X internet config module (removed in Python 3.0)
-                import ic
-                ic.launchurl(url)
-            else:
-                try:
-                    import gconf
-                    client = gconf.client_get_default()
-                    browser = client.get_string(
-                            '/desktop/gnome/url-handlers/http/command') + '&'
-                    os.system(browser % url)
-                except ImportError:
-                    # If gconf is not found, fall back to old standard
-                    os.system('firefox ' + url)
-        threading.Thread(target=start_browser).start()
-
